@@ -4,7 +4,7 @@
 # shellcheck source=contrib/jenkins_common.sh
 . $(dirname "$0")/jenkins_common.sh
 
-osmo-build-dep.sh libosmocore
+osmo-build-dep.sh libosmocore "" --disable-doxygen
 
 export PKG_CONFIG_PATH="$inst/lib/pkgconfig:$PKG_CONFIG_PATH"
 export LD_LIBRARY_PATH="$inst/lib"
@@ -14,7 +14,7 @@ osmo-build-dep.sh libosmo-abis
 cd "$deps"
 osmo-layer1-headers.sh lc15 "$FIRMWARE_VERSION"
 
-configure_flags="--with-litecell15=$deps/layer1-headers/inc/ --enable-litecell15"
+configure_flags="--enable-sanitize --with-litecell15=$deps/layer1-headers/inc/ --enable-litecell15"
 
 build_bts "osmo-bts-lc15" "$configure_flags"
 
