@@ -7,15 +7,12 @@
 export PKG_CONFIG_PATH="$inst/lib/pkgconfig:$PKG_CONFIG_PATH"
 export LD_LIBRARY_PATH="$inst/lib"
 
-osmo-build-dep.sh libosmocore
+osmo-build-dep.sh libosmocore "" --disable-doxygen
 
 osmo-build-dep.sh libosmo-abis
 
 cd "$deps"
 
-# Get osmo-pcu for pcuif_proto.h
-osmo-deps.sh osmo-pcu
-
 osmo-layer1-headers.sh oct "$FIRMWARE_VERSION"
 
 configure_flags="\
