@@ -4,7 +4,7 @@
 # shellcheck source=contrib/jenkins_common.sh
 . $(dirname "$0")/jenkins_common.sh
 
-osmo-build-dep.sh libosmocore
+osmo-build-dep.sh libosmocore "" --disable-doxygen
 
 export PKG_CONFIG_PATH="$inst/lib/pkgconfig:$PKG_CONFIG_PATH"
 export LD_LIBRARY_PATH="$inst/lib"
@@ -16,13 +16,13 @@ osmo-layer1-headers.sh sysmo "$FIRMWARE_VERSION"
 mkdir -p "$inst/include/sysmocom/femtobts"
 ln -s $deps/layer1-headers/include/* "$inst/include/sysmocom/femtobts/"
 
-configure_flags="--enable-sysmocom-bts"
-
-build_bts "osmo-bts-sysmo" "$configure_flags"
+configure_flags="--enable-sanitize --enable-sysmocom-bts --with-sysmobts=$inst/include/"
 
 # This will not work for the femtobts
 if [ $FIRMWARE_VERSION != "femtobts_v2.7" ]; then
-  $MAKE -C contrib/sysmobts-calib
+    configure_flags="$configure_flags --enable-sysmobts-calib"
 fi
 
+build_bts "osmo-bts-sysmo" "$configure_flags"
+
 osmo-clean-workspace.sh
