@@ -5,6 +5,7 @@
 #include <stdbool.h>
 #include <stdint.h>
 
+#include <osmocom/codec/ecu.h>
 #include <osmocom/core/timer.h>
 #include <osmocom/core/bitvec.h>
 #include <osmocom/core/statistics.h>
@@ -19,10 +20,7 @@
 #include <osmocom/gsm/protocol/gsm_12_21.h>
 
 #include <osmocom/abis/e1_input.h>
-
-#ifndef ROLE_BSC
 #include <osmocom/gsm/lapdm.h>
-#endif
 
 /* 16 is the max. number of SI2quater messages according to 3GPP TS 44.018 Table 10.5.2.33b.1:
    4-bit index is used (2#1111 = 10#15) */
@@ -31,19 +29,6 @@
 #define SI2Q_MAX_LEN 160
 #define SI2Q_MIN_LEN 18
 
-struct osmo_bsc_data;
-
-struct osmo_bsc_sccp_con;
-struct gsm_sms_queue;
-
-/* RRLP mode of operation */
-enum rrlp_mode {
-	RRLP_MODE_NONE,
-	RRLP_MODE_MS_BASED,
-	RRLP_MODE_MS_PREF,
-	RRLP_MODE_ASS_PREF,
-};
-
 /* Channel Request reason */
 enum gsm_chreq_reason_t {
 	GSM_CHREQ_REASON_EMERG,
@@ -79,13 +64,6 @@ enum gsm_hooks {
 	GSM_HOOK_RR_SECURITY,
 };
 
-enum gsm_paging_event {
-	GSM_PAGING_SUCCEEDED,
-	GSM_PAGING_EXPIRED,
-	GSM_PAGING_OOM,
-	GSM_PAGING_BUSY,
-};
-
 enum bts_gprs_mode {
 	BTS_GPRS_NONE = 0,
 	BTS_GPRS_GPRS = 1,
@@ -93,11 +71,9 @@ enum bts_gprs_mode {
 };
 
 struct gsm_lchan;
-struct gsm_subscriber;
-struct gsm_mncc;
 struct osmo_rtp_socket;
-struct rtp_socket;
-struct bsc_api;
+struct pcu_sock_state;
+struct smscb_msg;
 
 /* Network Management State */
 struct gsm_nm_state {
@@ -107,28 +83,22 @@ struct gsm_nm_state {
 };
 
 struct gsm_abis_mo {
+	/* A-bis OML Object Class */
 	uint8_t obj_class;
+	/* is there still some procedure pending? */
 	uint8_t procedure_pending;
+	/* A-bis OML Object Instance */
 	struct abis_om_obj_inst obj_inst;
+	/* human-readable name */
 	const char *name;
+	/* NM State */
 	struct gsm_nm_state nm_state;
+	/* Attributes configured in this MO */
 	struct tlv_parsed *nm_attr;
+	/* BTS to which this MO belongs */
 	struct gsm_bts *bts;
 };
 
-/* Ericsson OM2000 Managed Object */
-struct abis_om2k_mo {
-	uint8_t class;
-	uint8_t bts;
-	uint8_t assoc_so;
-	uint8_t inst;
-} __attribute__ ((packed));
-
-struct om2k_mo {
-	struct abis_om2k_mo addr;
-	struct osmo_fsm_inst *fsm;
-};
-
 #define MAX_A5_KEY_LEN	(128/8)
 #define A38_XOR_MIN_KEY_LEN	12
 #define A38_XOR_MAX_KEY_LEN	16
@@ -161,8 +131,8 @@ enum gsm_lchan_state {
 struct bts_ul_meas {
 	/* BER in units of 0.01%: 10.000 == 100% ber, 0 == 0% ber */
 	uint16_t ber10k;
-	/* timing advance offset (in quarter bits) */
-	int16_t ta_offs_qbits;
+	/* timing advance offset (in 1/256 bits) */
+	int16_t ta_offs_256bits;
 	/* C/I ratio in dB */
 	float c_i;
 	/* flags */
@@ -238,11 +208,9 @@ struct gsm_lchan {
 	} encr;
 
 	/* AMR bits */
-	uint8_t mr_ms_lv[7];
 	uint8_t mr_bts_lv[7];
 
 	/* Established data link layer services */
-	uint8_t sapis[8];
 	int sacch_deact;
 
 	struct {
@@ -254,46 +222,13 @@ struct gsm_lchan {
 		uint8_t rtp_payload;
 		uint8_t rtp_payload2;
 		uint8_t speech_mode;
-#ifdef ROLE_BSC
-		struct rtp_socket *rtp_socket;
-#else
 		struct osmo_rtp_socket *rtp_socket;
-#endif
 	} abis_ip;
 
 	uint8_t rqd_ta;
 
 	char *name;
 
-#ifdef ROLE_BSC
-	struct osmo_timer_list T3101;
-	struct osmo_timer_list T3109;
-	struct osmo_timer_list T3111;
-	struct osmo_timer_list error_timer;
-	struct osmo_timer_list act_timer;
-	struct osmo_timer_list rel_work;
-	uint8_t error_cause;
-
-	/* table of neighbor cell measurements */
-	struct neigh_meas_proc neigh_meas[MAX_NEIGH_MEAS];
-
-	/* cache of last measurement reports on this lchan */
-	struct gsm_meas_rep meas_rep[6];
-	int meas_rep_idx;
-
-	/* GSM Random Access data */
-	struct gsm48_req_ref *rqd_ref;
-
-	struct gsm_subscriber_connection *conn;
-
-	struct {
-		/* channel activation type and handover ref */
-		uint8_t act_type;
-		uint8_t ho_ref;
-		struct gsm48_req_ref *rqd_ref;
-		uint8_t rqd_ta;
-	} dyn;
-#else
 	/* Number of different GsmL1_Sapi_t used in osmo_bts_sysmo is 23.
 	 * Currently we don't share these headers so this is a magic number. */
 	struct llist_head sapi_cmds;
@@ -321,6 +256,7 @@ struct gsm_lchan {
 		/* last L1 header from the MS */
 		uint8_t l1_info[2];
 		struct gsm_meas_rep_unidir ul_res;
+		int16_t ms_toa256;
 	} meas;
 	struct {
 		struct amr_multirate_conf amr_mr;
@@ -374,9 +310,18 @@ struct gsm_lchan {
 	} ms_power_ctrl;
 
 	struct msgb *pending_rel_ind_msg;
-#endif
+
+	/* ECU (Error Concealment Unit) state */
+	union {
+		struct osmo_ecu_fr_state fr;
+	} ecu_state;
 };
 
+extern const struct value_string lchan_ciph_state_names[];
+static inline const char *lchan_ciph_state_name(uint8_t state) {
+	return get_value_string(lchan_ciph_state_names, state);
+}
+
 enum {
 	TS_F_PDCH_ACTIVE =		0x1000,
 	TS_F_PDCH_ACT_PENDING =		0x2000,
@@ -418,15 +363,6 @@ struct gsm_bts_trx_ts {
 		uint8_t ma_data[8];	/* 10.5.2.21: max 8 bytes value part */
 	} hopping;
 
-	/* To which E1 subslot are we connected */
-	struct gsm_e1_subslot e1_link;
-
-	union {
-		struct {
-			struct om2k_mo om2k_mo;
-		} rbs2000;
-	};
-
 	struct gsm_lchan lchan[TS_MAX_LCHAN];
 };
 
@@ -441,7 +377,6 @@ struct gsm_bts_trx {
 	/* human readable name / description */
 	char *description;
 	/* how do we talk RSL with this TRX? */
-	struct gsm_e1_subslot rsl_e1_link;
 	uint8_t rsl_tei;
 	struct e1inp_sign_link *rsl_link;
 
@@ -458,40 +393,19 @@ struct gsm_bts_trx {
 	int nominal_power;		/* in dBm */
 	unsigned int max_power_red;	/* in actual dB */
 
-#ifndef ROLE_BSC
 	struct trx_power_params power_params;
 	int ms_power_control;
 
 	struct {
 		void *l1h;
 	} role_bts;
-#endif
 
 	union {
-		struct {
-			struct {
-				struct gsm_abis_mo mo;
-			} bbsig;
-			struct {
-				struct gsm_abis_mo mo;
-			} pa;
-		} bs11;
 		struct {
 			unsigned int test_state;
 			uint8_t test_nr;
 			struct rxlev_stats rxlev_stat;
 		} ipaccess;
-		struct {
-			struct {
-				struct om2k_mo om2k_mo;
-			} trxc;
-			struct {
-				struct om2k_mo om2k_mo;
-			} rx;
-			struct {
-				struct om2k_mo om2k_mo;
-			} tx;
-		} rbs2000;
 	};
 	struct gsm_bts_trx_ts ts[TRX_NR_TS];
 };
@@ -501,22 +415,14 @@ struct gsm_bts_trx {
 #define GSM_BTS_SI(bts, i)     (void *)((bts)->si_buf[i][0])
 #define GSM_LCHAN_SI(lchan, i) (void *)((lchan)->si.buf[i][0])
 
-enum gsm_bts_type {
-	GSM_BTS_TYPE_UNKNOWN,
-	GSM_BTS_TYPE_BS11,
-	GSM_BTS_TYPE_NANOBTS,
-	GSM_BTS_TYPE_RBS2000,
-	GSM_BTS_TYPE_NOKIA_SITE,
-	GSM_BTS_TYPE_OSMOBTS,
-	_NUM_GSM_BTS_TYPE
-};
-
 enum gsm_bts_type_variant {
 	BTS_UNKNOWN,
 	BTS_OSMO_LITECELL15,
 	BTS_OSMO_OCTPHY,
 	BTS_OSMO_SYSMO,
 	BTS_OSMO_TRX,
+	BTS_OSMO_VIRTUAL,
+	BTS_OSMO_OMLDUMMY,
 	_NUM_BTS_VARIANT
 };
 
@@ -529,32 +435,9 @@ enum bts_attribute {
 
 struct vty;
 
-struct gsm_bts_model {
-	struct llist_head list;
-
-	enum gsm_bts_type type;
-	enum gsm_bts_type_variant variant;
-	const char *name;
-
-	bool started;
-	int (*start)(struct gsm_network *net);
-	int (*oml_rcvmsg)(struct msgb *msg);
-
-	void (*e1line_bind_ops)(struct e1inp_line *line);
-
-	void (*config_write_bts)(struct vty *vty, struct gsm_bts *bts);
-	void (*config_write_trx)(struct vty *vty, struct gsm_bts_trx *trx);
-	void (*config_write_ts)(struct vty *vty, struct gsm_bts_trx_ts *ts);
-
-	struct tlv_definition nm_att_tlvdef;
-
-	/* features of a given BTS model set via gsm_bts_model_register() locally */
-	struct bitvec features;
-	uint8_t _features_data[MAX_BTS_FEATURES/8];
-};
-
 /* N. B: always add new features to the end of the list (right before _NUM_BTS_FEAT) to avoid breaking compatibility
-   with BTS compiled against earlier version of this header */
+   with BTS compiled against earlier version of this header. Also make sure that the description strings
+   gsm_bts_features_descs[] in gsm_data_shared.c are also updated accordingly! */
 enum gsm_bts_features {
 	BTS_FEAT_HSCSD,
 	BTS_FEAT_GPRS,
@@ -565,35 +448,16 @@ enum gsm_bts_features {
 	BTS_FEAT_OML_ALERTS,
 	BTS_FEAT_AGCH_PCH_PROP,
 	BTS_FEAT_CBCH,
+	BTS_FEAT_SPEECH_F_V1,
+	BTS_FEAT_SPEECH_H_V1,
+	BTS_FEAT_SPEECH_F_EFR,
+	BTS_FEAT_SPEECH_F_AMR,
+	BTS_FEAT_SPEECH_H_AMR,
 	_NUM_BTS_FEAT
 };
 
 extern const struct value_string gsm_bts_features_descs[];
 
-/*
- * This keeps track of the paging status of one BTS. It
- * includes a number of pending requests, a back pointer
- * to the gsm_bts, a timer and some more state.
- */
-struct gsm_bts_paging_state {
-	/* pending requests */
-	struct llist_head pending_requests;
-	struct gsm_bts *bts;
-
-	struct osmo_timer_list work_timer;
-	struct osmo_timer_list credit_timer;
-
-	/* free chans needed */
-	int free_chans_need;
-
-	/* load */
-	uint16_t available_slots;
-};
-
-struct gsm_envabtse {
-	struct gsm_abis_mo mo;
-};
-
 struct gsm_bts_gprs_nsvc {
 	struct gsm_bts *bts;
 	/* data read via VTY config file, to configure the BTS
@@ -650,30 +514,6 @@ struct gprs_rlc_cfg {
 	uint8_t initial_mcs;
 };
 
-
-enum neigh_list_manual_mode {
-	NL_MODE_AUTOMATIC = 0,
-	NL_MODE_MANUAL = 1,
-	NL_MODE_MANUAL_SI5SEP = 2, /* SI2 and SI5 have separate neighbor lists */
-};
-
-enum bts_loc_fix {
-	BTS_LOC_FIX_INVALID = 0,
-	BTS_LOC_FIX_2D = 1,
-	BTS_LOC_FIX_3D = 2,
-};
-
-extern const struct value_string bts_loc_fix_names[];
-
-struct bts_location {
-	struct llist_head list;
-	time_t tstamp;
-	enum bts_loc_fix valid;
-	double lat;
-	double lon;
-	double height;
-};
-
 /* One BTS */
 struct gsm_bts {
 	/* list header in net->bts_list */
@@ -694,9 +534,7 @@ struct gsm_bts {
 	 * which is used as TSC for the CCCH */
 	uint8_t bsic;
 	/* type of BTS */
-	enum gsm_bts_type type;
 	enum gsm_bts_type_variant variant;
-	struct gsm_bts_model *model;
 	enum gsm_band band;
 	char version[MAX_VERSION_LENGTH];
 	char sub_model[MAX_VERSION_LENGTH];
@@ -712,11 +550,8 @@ struct gsm_bts {
 	int ms_max_power;
 
 	/* how do we talk OML with this TRX? */
-	struct gsm_e1_subslot oml_e1_link;
 	uint8_t oml_tei;
 	struct e1inp_sign_link *oml_link;
-	/* when OML link was established */
-	time_t uptime;
 
 	/* Abis network management O&M handle */
 	struct abis_nm_h *nmh;
@@ -730,9 +565,6 @@ struct gsm_bts {
 	enum gsm48_dtx_mode dtxu;
 	bool dtxd;
 
-	/* paging state and control */
-	struct gsm_bts_paging_state paging;
-
 	/* CCCH is on C0 */
 	struct gsm_bts_trx *c0;
 
@@ -759,51 +591,6 @@ struct gsm_bts {
 			uint32_t flags;
 			uint32_t rsl_ip;
 		} ip_access;
-		struct {
-			struct {
-				struct gsm_abis_mo mo;
-			} cclk;
-			struct {
-				struct gsm_abis_mo mo;
-			} rack;
-			struct gsm_envabtse envabtse[4];
-		} bs11;
-		struct {
-			struct {
-				struct om2k_mo om2k_mo;
-				struct gsm_abis_mo mo;
-				struct llist_head conn_groups;
-			} cf;
-			struct {
-				struct om2k_mo om2k_mo;
-				struct gsm_abis_mo mo;
-				struct llist_head conn_groups;
-			} is;
-			struct {
-				struct om2k_mo om2k_mo;
-				struct gsm_abis_mo mo;
-				struct llist_head conn_groups;
-			} con;
-			struct {
-				struct om2k_mo om2k_mo;
-				struct gsm_abis_mo mo;
-			} dp;
-			struct {
-				struct om2k_mo om2k_mo;
-				struct gsm_abis_mo mo;
-			} tf;
-			uint32_t use_superchannel:1;
-		} rbs2000;
-		struct {
-			uint8_t bts_type;
-			unsigned int configured:1,
-				skip_reset:1,
-				no_loc_rel_cnf:1,
-				bts_reset_timer_cnf,
-				did_reset:1,
-				wait_reset:1;
-			struct osmo_timer_list reset_timer;
-		} nokia;
 	};
 
 	/* Not entirely sure how ip.access specific this is */
@@ -839,69 +626,98 @@ struct gsm_bts {
 	int force_combined_si;
 	int bcch_change_mark;
 
-#ifdef ROLE_BSC
-	/* Abis NM queue */
-	struct llist_head abis_queue;
-	int abis_nm_pend;
+	struct rate_ctr_group *ctrs;
+	bool supp_meas_toa256;
 
-	struct gsm_network *network;
-
-	/* should the channel allocator allocate channels from high TRX to TRX0,
-	 * rather than starting from TRX0 and go upwards? */
-	int chan_alloc_reverse;
-
-	enum neigh_list_manual_mode neigh_list_manual_mode;
-	/* parameters from which we build SYSTEM INFORMATION */
 	struct {
-		struct gsm48_rach_control rach_control;
-		uint8_t ncc_permitted;
-		struct gsm48_cell_sel_par cell_sel_par;
-		struct gsm48_si_selection_params cell_ro_sel_par; /* rest octet */
-		struct gsm48_cell_options cell_options;
-		struct gsm48_control_channel_descr chan_desc;
-		struct bitvec neigh_list;
-		struct bitvec cell_alloc;
-		struct bitvec si5_neigh_list;
-		struct osmo_earfcn_si2q si2quater_neigh_list;
-		size_t uarfcn_length; /* index for uarfcn and scramble lists */
+		/* Interference Boundaries for OML */
+		int16_t boundary[6];
+		uint8_t intave;
+	} interference;
+	unsigned int t200_ms[7];
+	unsigned int t3105_ms;
+	struct {
+		uint8_t overload_period;
 		struct {
-			/* bitmask large enough for all possible ARFCN's */
-			uint8_t neigh_list[1024/8];
-			uint8_t cell_alloc[1024/8];
-			/* If the user wants a different neighbor list in SI5 than in SI2 */
-			uint8_t si5_neigh_list[1024/8];
-			uint8_t meas_bw_list[MAX_EARFCN_LIST];
-			uint16_t earfcn_list[MAX_EARFCN_LIST];
-			uint16_t uarfcn_list[MAX_EARFCN_LIST];
-			uint16_t scramble_list[MAX_EARFCN_LIST];
-		} data;
-	} si_common;
-	bool early_classmark_allowed;
-	/* for testing only: Have an infinitely long radio link timeout */
-	bool infinite_radio_link_timeout;
+			/* Input parameters from OML */
+			uint8_t load_ind_thresh;	/* percent */
+			uint8_t load_ind_period;	/* seconds */
+			/* Internal data */
+			struct osmo_timer_list timer;
+			unsigned int pch_total;
+			unsigned int pch_used;
+		} ccch;
+		struct {
+			/* Input parameters from OML */
+			int16_t busy_thresh;		/* in dBm */
+			uint16_t averaging_slots;
+			/* Internal data */
+			unsigned int total;	/* total nr */
+			unsigned int busy;	/* above busy_thresh */
+			unsigned int access;	/* access bursts */
+		} rach;
+	} load;
+	uint8_t ny1;
+	uint8_t max_ta;
 
-	/* do we use static (user-defined) system information messages? (bitmask) */
-	uint32_t si_mode_static;
+	/* AGCH queuing */
+	struct {
+		struct llist_head queue;
+		int length;
+		int max_length;
 
-	/* exclude the BTS from the global RF Lock handling */
-	int excl_from_rf_lock;
+		int thresh_level;	/* Cleanup threshold in percent of max len */
+		int low_level;		/* Low water mark in percent of max len */
+		int high_level;		/* High water mark in percent of max len */
 
-	/* supported codecs beside FR */
-	struct bts_codec_conf codec;
+		/* TODO: Use a rate counter group instead */
+		uint64_t dropped_msgs;
+		uint64_t merged_msgs;
+		uint64_t rejected_msgs;
+		uint64_t agch_msgs;
+		uint64_t pch_msgs;
+	} agch_queue;
 
-	/* BTS dependencies bit field */
-	uint32_t depends_on[256/(8*4)];
+	struct paging_state *paging_state;
+	char *bsc_oml_host;
+	struct llist_head oml_queue;
+	unsigned int rtp_jitter_buf_ms;
+	bool rtp_jitter_adaptive;
+	struct {
+		uint8_t ciphers;	/* flags A5/1==0x1, A5/2==0x2, A5/3==0x4 */
+	} support;
+	struct {
+		uint8_t tc4_ctr;
+	} si;
+	struct gsm_time gsm_time;
+	/* Radio Link Timeout counter. -1 disables timeout for
+	 * lab/measurement purpose */
+	int radio_link_timeout;
 
-	/* full and half rate multirate config */
-	struct amr_multirate_conf mr_full;
-	struct amr_multirate_conf mr_half;
+	int ul_power_target;		/* Uplink Rx power target */
 
-	/* PCU socket state */
-	char *pcu_sock_path;
-	struct pcu_sock_state *pcu_state;
+	/* used by the sysmoBTS to adjust band */
+	uint8_t auto_band;
+
+	struct {
+		struct llist_head queue;	/* list of struct smscb_msg */
+		struct smscb_msg *cur_msg;	/* current SMS-CB */
+	} smscb_state;
+
+	float min_qual_rach;	/* minimum quality for RACH bursts */
+	float min_qual_norm;	/* minimum quality for normal daata */
+	uint16_t max_ber10k_rach;	/* Maximum permitted RACH BER in 0.01% */
+
+	struct {
+		char *sock_path;
+	} pcu;
+
+	struct {
+		uint32_t last_fn;
+		struct timeval tv_clock;
+		struct osmo_timer_list fn_timer;
+	} vbts;
 
-#endif /* ROLE_BSC */
-	void *role;
 };
 
 
@@ -911,9 +727,6 @@ struct gsm_bts *gsm_bts_num(struct gsm_network *net, int num);
 struct gsm_bts_trx *gsm_bts_trx_alloc(struct gsm_bts *bts);
 struct gsm_bts_trx *gsm_bts_trx_num(const struct gsm_bts *bts, int num);
 
-enum gsm_bts_type str2btstype(const char *arg);
-const char *btstype2str(enum gsm_bts_type type);
-
 enum bts_attribute str2btsattr(const char *s);
 const char *btsatttr2str(enum bts_attribute v);
 
@@ -981,6 +794,8 @@ struct gsm_lchan *gsm_bts_get_cbch(struct gsm_bts *bts);
 int gsm_parse_reg(void *ctx, regex_t *reg, char **str,
 		int argc, const char **argv) __attribute__ ((warn_unused_result));
 
+#define BSIC2BCC(bsic) ((bsic) & 0x3)
+
 static inline uint8_t gsm_ts_tsc(const struct gsm_bts_trx_ts *ts)
 {
 	if (ts->tsc != -1)
@@ -995,5 +810,6 @@ struct gsm_lchan *rsl_lchan_lookup(struct gsm_bts_trx *trx, uint8_t chan_nr,
 enum gsm_phys_chan_config ts_pchan(struct gsm_bts_trx_ts *ts);
 uint8_t ts_subslots(struct gsm_bts_trx_ts *ts);
 bool ts_is_tch(struct gsm_bts_trx_ts *ts);
+const char *gsm_trx_unit_id(struct gsm_bts_trx *trx);
 
 #endif
