@@ -32,6 +32,8 @@
 #include <osmocom/core/timer.h>
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/talloc.h>
+#include <osmocom/core/stats.h>
+#include <osmocom/core/rate_ctr.h>
 #include <osmocom/gsm/protocol/gsm_12_21.h>
 #include <osmocom/gsm/lapdm.h>
 #include <osmocom/trau/osmo_ortp.h>
@@ -41,7 +43,7 @@
 #include <osmo-bts/bts.h>
 #include <osmo-bts/bts_model.h>
 #include <osmo-bts/dtx_dl_amr_fsm.h>
-#include <osmo-bts/pcu_if.h>
+#include <osmo-bts/pcuif_proto.h>
 #include <osmo-bts/rsl.h>
 #include <osmo-bts/oml.h>
 #include <osmo-bts/signal.h>
@@ -79,11 +81,33 @@ static int bts_signal_cbfn(unsigned int subsys, unsigned int signal,
 	return 0;
 }
 
+static const struct rate_ctr_desc bts_ctr_desc[] = {
+	[BTS_CTR_PAGING_RCVD] =		{"paging:rcvd", "Received paging requests (Abis)"},
+	[BTS_CTR_PAGING_DROP] =		{"paging:drop", "Dropped paging requests (Abis)"},
+	[BTS_CTR_PAGING_SENT] =		{"paging:sent", "Sent paging requests (Um)"},
+
+	[BTS_CTR_RACH_RCVD] =		{"rach:rcvd", "Received RACH requests (Um)"},
+	[BTS_CTR_RACH_DROP] =		{"rach:drop", "Dropped RACH requests (Um)"},
+	[BTS_CTR_RACH_HO] =		{"rach:handover", "Received RACH requests (Handover)"},
+	[BTS_CTR_RACH_CS] =		{"rach:cs", "Received RACH requests (CS/Abis)"},
+	[BTS_CTR_RACH_PS] =		{"rach:ps", "Received RACH requests (PS/PCU)"},
+
+	[BTS_CTR_AGCH_RCVD] =		{"agch:rcvd", "Received AGCH requests (Abis)"},
+	[BTS_CTR_AGCH_SENT] =		{"agch:sent", "Sent AGCH requests (Abis)"},
+	[BTS_CTR_AGCH_DELETED] =	{"agch:delete", "Sent AGCH DELETE IND (Abis)"},
+};
+static const struct rate_ctr_group_desc bts_ctrg_desc = {
+	"bts",
+	"base transceiver station",
+	OSMO_STATS_CLASS_GLOBAL,
+	ARRAY_SIZE(bts_ctr_desc),
+	bts_ctr_desc
+};
+
 /* Initialize the BTS (and TRX) data structures, called before config
  * file reading */
 int bts_init(struct gsm_bts *bts)
 {
-	struct gsm_bts_role_bts *btsb;
 	struct gsm_bts_trx *trx;
 	int rc, i;
 	static int initialized = 0;
@@ -94,39 +118,40 @@ int bts_init(struct gsm_bts *bts)
 
 	bts->band = GSM_BAND_1800;
 
-	bts->role = btsb = talloc_zero(bts, struct gsm_bts_role_bts);
+	INIT_LLIST_HEAD(&bts->agch_queue.queue);
+	bts->agch_queue.length = 0;
 
-	INIT_LLIST_HEAD(&btsb->agch_queue);
-	btsb->agch_queue_length = 0;
+	bts->ctrs = rate_ctr_group_alloc(bts, &bts_ctrg_desc, bts->nr);
 
 	/* enable management with default levels,
 	 * raise threshold to GSM_BTS_AGCH_QUEUE_THRESH_LEVEL_DISABLE to
 	 * disable this feature.
 	 */
-	btsb->agch_queue_low_level = GSM_BTS_AGCH_QUEUE_LOW_LEVEL_DEFAULT;
-	btsb->agch_queue_high_level = GSM_BTS_AGCH_QUEUE_HIGH_LEVEL_DEFAULT;
-	btsb->agch_queue_thresh_level = GSM_BTS_AGCH_QUEUE_THRESH_LEVEL_DEFAULT;
+	bts->agch_queue.low_level = GSM_BTS_AGCH_QUEUE_LOW_LEVEL_DEFAULT;
+	bts->agch_queue.high_level = GSM_BTS_AGCH_QUEUE_HIGH_LEVEL_DEFAULT;
+	bts->agch_queue.thresh_level = GSM_BTS_AGCH_QUEUE_THRESH_LEVEL_DEFAULT;
 
 	/* configurable via VTY */
-	btsb->paging_state = paging_init(btsb, 200, 0);
-	btsb->ul_power_target = -75;	/* dBm default */
-	btsb->rtp_jitter_adaptive = false;
+	bts->paging_state = paging_init(bts, 200, 0);
+	bts->ul_power_target = -75;	/* dBm default */
+	bts->rtp_jitter_adaptive = false;
 
 	/* configurable via OML */
-	btsb->load.ccch.load_ind_period = 112;
+	bts->load.ccch.load_ind_period = 112;
 	load_timer_start(bts);
-	btsb->rtp_jitter_buf_ms = 100;
-	btsb->max_ta = 63;
-	btsb->ny1 = 4;
-	btsb->t3105_ms = 300;
-	btsb->min_qual_rach = MIN_QUAL_RACH;
-	btsb->min_qual_norm = MIN_QUAL_NORM;
-	btsb->pcu.sock_path = talloc_strdup(btsb, PCU_SOCK_DEFAULT);
-	for (i = 0; i < ARRAY_SIZE(btsb->t200_ms); i++)
-		btsb->t200_ms[i] = oml_default_t200_ms[i];
+	bts->rtp_jitter_buf_ms = 100;
+	bts->max_ta = 63;
+	bts->ny1 = 4;
+	bts->t3105_ms = 300;
+	bts->min_qual_rach = MIN_QUAL_RACH;
+	bts->min_qual_norm = MIN_QUAL_NORM;
+	bts->max_ber10k_rach = 1707; /* 7 of 41 bits is Eb/N0 of 0 dB = 0.1707 */
+	bts->pcu.sock_path = talloc_strdup(bts, PCU_SOCK_DEFAULT);
+	for (i = 0; i < ARRAY_SIZE(bts->t200_ms); i++)
+		bts->t200_ms[i] = oml_default_t200_ms[i];
 
 	/* default RADIO_LINK_TIMEOUT */
-	btsb->radio_link_timeout = 32;
+	bts->radio_link_timeout = 32;
 
 	/* Start with the site manager */
 	oml_mo_state_init(&bts->site_mgr.mo, NM_OPSTATE_ENABLED, NM_AVSTATE_OK);
@@ -176,8 +201,8 @@ int bts_init(struct gsm_bts *bts)
 		initialized = 1;
 	}
 
-	INIT_LLIST_HEAD(&btsb->smscb_state.queue);
-	INIT_LLIST_HEAD(&btsb->oml_queue);
+	INIT_LLIST_HEAD(&bts->smscb_state.queue);
+	INIT_LLIST_HEAD(&bts->oml_queue);
 
 	/* register DTX DL FSM */
 	rc = osmo_fsm_register(&dtx_dl_amr_fsm);
@@ -358,22 +383,21 @@ int bts_agch_max_queue_length(int T, int bcch_conf)
 
 static void bts_update_agch_max_queue_length(struct gsm_bts *bts)
 {
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 	struct gsm48_system_information_type_3 *si3;
-	int old_max_length = btsb->agch_max_queue_length;
+	int old_max_length = bts->agch_queue.max_length;
 
 	if (!(bts->si_valid & (1<<SYSINFO_TYPE_3)))
 		return;
 
 	si3 = GSM_BTS_SI(bts, SYSINFO_TYPE_3);
 
-	btsb->agch_max_queue_length =
+	bts->agch_queue.max_length =
 		bts_agch_max_queue_length(si3->rach_control.tx_integer,
 					  si3->control_channel_desc.ccch_conf);
 
-	if (btsb->agch_max_queue_length != old_max_length)
+	if (bts->agch_queue.max_length != old_max_length)
 		LOGP(DRSL, LOGL_INFO, "Updated AGCH max queue length to %d\n",
-		     btsb->agch_max_queue_length);
+		     bts->agch_queue.max_length);
 }
 
 #define REQ_REFS_PER_IMM_ASS_REJ 4
@@ -505,47 +529,45 @@ static int try_merge_imm_ass_rej(struct gsm48_imm_ass_rej *old_rej,
 
 int bts_agch_enqueue(struct gsm_bts *bts, struct msgb *msg)
 {
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 	int hard_limit = 1000;
 	struct gsm48_imm_ass_rej *imm_ass_cmd = msgb_l3(msg);
 
-	if (btsb->agch_queue_length > hard_limit) {
+	if (bts->agch_queue.length > hard_limit) {
 		LOGP(DSUM, LOGL_ERROR,
 		     "AGCH: too many messages in queue, "
 		     "refusing message type 0x%02x, length = %d/%d\n",
 		     ((struct gsm48_imm_ass *)msgb_l3(msg))->msg_type,
-		     btsb->agch_queue_length, btsb->agch_max_queue_length);
+		     bts->agch_queue.length, bts->agch_queue.max_length);
 
-		btsb->agch_queue_rejected_msgs++;
+		bts->agch_queue.rejected_msgs++;
 		return -ENOMEM;
 	}
 
-	if (btsb->agch_queue_length > 0) {
+	if (bts->agch_queue.length > 0) {
 		struct msgb *last_msg =
-			llist_entry(btsb->agch_queue.prev, struct msgb, list);
+			llist_entry(bts->agch_queue.queue.prev, struct msgb, list);
 		struct gsm48_imm_ass_rej *last_imm_ass_rej = msgb_l3(last_msg);
 
 		if (try_merge_imm_ass_rej(last_imm_ass_rej, imm_ass_cmd)) {
-			btsb->agch_queue_merged_msgs++;
+			bts->agch_queue.merged_msgs++;
 			msgb_free(msg);
 			return 0;
 		}
 	}
 
-	msgb_enqueue(&btsb->agch_queue, msg);
-	btsb->agch_queue_length++;
+	msgb_enqueue(&bts->agch_queue.queue, msg);
+	bts->agch_queue.length++;
 
 	return 0;
 }
 
 struct msgb *bts_agch_dequeue(struct gsm_bts *bts)
 {
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
-	struct msgb *msg = msgb_dequeue(&btsb->agch_queue);
+	struct msgb *msg = msgb_dequeue(&bts->agch_queue.queue);
 	if (!msg)
 		return NULL;
 
-	btsb->agch_queue_length--;
+	bts->agch_queue.length--;
 	return msg;
 }
 
@@ -557,19 +579,18 @@ struct msgb *bts_agch_dequeue(struct gsm_bts *bts)
  */
 static void compact_agch_queue(struct gsm_bts *bts)
 {
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 	struct msgb *msg, *msg2;
 	int max_len, slope, offs;
-	int level_low = btsb->agch_queue_low_level;
-	int level_high = btsb->agch_queue_high_level;
-	int level_thres = btsb->agch_queue_thresh_level;
+	int level_low = bts->agch_queue.low_level;
+	int level_high = bts->agch_queue.high_level;
+	int level_thres = bts->agch_queue.thresh_level;
 
-	max_len = btsb->agch_max_queue_length;
+	max_len = bts->agch_queue.max_length;
 
 	if (max_len == 0)
 		max_len = 1;
 
-	if (btsb->agch_queue_length < max_len * level_thres / 100)
+	if (bts->agch_queue.length < max_len * level_thres / 100)
 		return;
 
 	/* p^
@@ -586,7 +607,7 @@ static void compact_agch_queue(struct gsm_bts *bts)
 	else
 		slope = 0x10000 * max_len; /* p_drop >= 1 if len > offs */
 
-	llist_for_each_entry_safe(msg, msg2, &btsb->agch_queue, list) {
+	llist_for_each_entry_safe(msg, msg2, &bts->agch_queue.queue, list) {
 		struct gsm48_imm_ass *imm_ass_cmd = msgb_l3(msg);
 		int p_drop;
 
@@ -595,16 +616,16 @@ static void compact_agch_queue(struct gsm_bts *bts)
 
 		/* IMMEDIATE ASSIGN REJECT */
 
-		p_drop = (btsb->agch_queue_length - offs) * slope / max_len;
+		p_drop = (bts->agch_queue.length - offs) * slope / max_len;
 
 		if ((random() & 0xffff) >= p_drop)
 			return;
 
 		llist_del(&msg->list);
-		btsb->agch_queue_length--;
+		bts->agch_queue.length--;
 		msgb_free(msg);
 
-		btsb->agch_queue_dropped_msgs++;
+		bts->agch_queue.dropped_msgs++;
 	}
 	return;
 }
@@ -613,7 +634,6 @@ int bts_ccch_copy_msg(struct gsm_bts *bts, uint8_t *out_buf, struct gsm_time *gt
 		      int is_ag_res)
 {
 	struct msgb *msg = NULL;
-	struct gsm_bts_role_bts *btsb = bts->role;
 	int rc = 0;
 	int is_empty = 1;
 
@@ -626,7 +646,7 @@ int bts_ccch_copy_msg(struct gsm_bts *bts, uint8_t *out_buf, struct gsm_time *gt
 
 	/* Check for paging messages first if this is PCH */
 	if (!is_ag_res)
-		rc = paging_gen_msg(btsb->paging_state, out_buf, gt, &is_empty);
+		rc = paging_gen_msg(bts->paging_state, out_buf, gt, &is_empty);
 
 	/* Check whether the block may be overwritten */
 	if (!is_empty)
@@ -636,20 +656,22 @@ int bts_ccch_copy_msg(struct gsm_bts *bts, uint8_t *out_buf, struct gsm_time *gt
 	if (!msg)
 		return rc;
 
+	rate_ctr_inc2(bts->ctrs, BTS_CTR_AGCH_SENT);
+
 	/* Copy AGCH message */
 	memcpy(out_buf, msgb_l3(msg), msgb_l3len(msg));
 	rc = msgb_l3len(msg);
 	msgb_free(msg);
 
 	if (is_ag_res)
-		btsb->agch_queue_agch_msgs++;
+		bts->agch_queue.agch_msgs++;
 	else
-		btsb->agch_queue_pch_msgs++;
+		bts->agch_queue.pch_msgs++;
 
 	return rc;
 }
 
-int bts_supports_cipher(struct gsm_bts_role_bts *bts, int rsl_cipher)
+int bts_supports_cipher(struct gsm_bts *bts, int rsl_cipher)
 {
 	int sup;
 
@@ -671,7 +693,48 @@ int trx_ms_pwr_ctrl_is_osmo(struct gsm_bts_trx *trx)
 
 struct gsm_time *get_time(struct gsm_bts *bts)
 {
-	struct gsm_bts_role_bts *btsb = bts->role;
+	return &bts->gsm_time;
+}
 
-	return &btsb->gsm_time;
+int bts_supports_cm(struct gsm_bts *bts, enum gsm_phys_chan_config pchan,
+		    enum gsm48_chan_mode cm)
+{
+	enum gsm_bts_features feature = _NUM_BTS_FEAT;
+
+	/* Before the requested pchan/cm combination can be checked, we need to
+	 * convert it to a feature identifier we can check */
+	if (pchan == GSM_PCHAN_TCH_F) {
+		switch(cm) {
+		case GSM48_CMODE_SPEECH_V1:
+			feature	= BTS_FEAT_SPEECH_F_V1;
+			break;
+		case GSM48_CMODE_SPEECH_EFR:
+			feature	= BTS_FEAT_SPEECH_F_EFR;
+			break;
+		case GSM48_CMODE_SPEECH_AMR:
+			feature = BTS_FEAT_SPEECH_F_AMR;
+			break;
+		default:
+			/* Invalid speech codec type => Not supported! */
+			return 0;
+		}
+	} else if (pchan == GSM_PCHAN_TCH_H) {
+		switch(cm) {
+		case GSM48_CMODE_SPEECH_V1:
+			feature	= BTS_FEAT_SPEECH_H_V1;
+			break;
+		case GSM48_CMODE_SPEECH_AMR:
+			feature = BTS_FEAT_SPEECH_H_AMR;
+			break;
+		default:
+			/* Invalid speech codec type => Not supported! */
+			return 0;
+		}
+	}
+
+	/* Check if the feature is supported by this BTS */
+	if (gsm_bts_has_feature(bts, feature))
+		return 1;
+
+	return 0;
 }
