@@ -74,6 +74,8 @@ const struct value_string osmo_bts_variant_names[_NUM_BTS_VARIANT + 1] = {
 	{ BTS_OSMO_OCTPHY,	"osmo-bts-octphy" },
 	{ BTS_OSMO_SYSMO,	"osmo-bts-sysmo" },
 	{ BTS_OSMO_TRX,		"omso-bts-trx" },
+	{ BTS_OSMO_VIRTUAL,	"omso-bts-virtual" },
+	{ BTS_OSMO_OMLDUMMY,	"omso-bts-omldummy" },
 	{ 0, NULL }
 };
 
@@ -87,26 +89,6 @@ const char *btsvariant2str(enum gsm_bts_type_variant v)
 	return get_value_string(osmo_bts_variant_names, v);
 }
 
-const struct value_string bts_type_names[_NUM_GSM_BTS_TYPE + 1] = {
-	{ GSM_BTS_TYPE_UNKNOWN,		"unknown" },
-	{ GSM_BTS_TYPE_BS11,		"bs11" },
-	{ GSM_BTS_TYPE_NANOBTS,		"nanobts" },
-	{ GSM_BTS_TYPE_RBS2000,		"rbs2000" },
-	{ GSM_BTS_TYPE_NOKIA_SITE,	"nokia_site" },
-	{ GSM_BTS_TYPE_OSMOBTS,		"sysmobts" },
-	{ 0, NULL }
-};
-
-enum gsm_bts_type str2btstype(const char *arg)
-{
-	return get_string_value(bts_type_names, arg);
-}
-
-const char *btstype2str(enum gsm_bts_type type)
-{
-	return get_value_string(bts_type_names, type);
-}
-
 const struct value_string gsm_bts_features_descs[] = {
 	{ BTS_FEAT_HSCSD,		"HSCSD" },
 	{ BTS_FEAT_GPRS,		"GPRS" },
@@ -117,6 +99,11 @@ const struct value_string gsm_bts_features_descs[] = {
 	{ BTS_FEAT_OML_ALERTS,		"OML Alerts" },
 	{ BTS_FEAT_AGCH_PCH_PROP,	"AGCH/PCH proportional allocation" },
 	{ BTS_FEAT_CBCH,		"CBCH" },
+	{ BTS_FEAT_SPEECH_F_V1,		"Fullrate speech V1" },
+	{ BTS_FEAT_SPEECH_H_V1,		"Halfrate speech V1" },
+	{ BTS_FEAT_SPEECH_F_EFR,	"Fullrate speech EFR" },
+	{ BTS_FEAT_SPEECH_F_AMR,	"Fullrate speech AMR" },
+	{ BTS_FEAT_SPEECH_H_AMR,	"Halfrate speech AMR" },
 	{ 0, NULL }
 };
 
@@ -271,9 +258,7 @@ struct gsm_bts_trx *gsm_bts_trx_alloc(struct gsm_bts *bts)
 
 			name = gsm_lchan_name_compute(lchan);
 			lchan->name = talloc_strdup(trx, name);
-#ifndef ROLE_BSC
 			INIT_LLIST_HEAD(&lchan->sapi_cmds);
-#endif
 		}
 	}
 
@@ -357,7 +342,6 @@ struct gsm_bts *gsm_bts_alloc(void *ctx, uint8_t bts_num)
 
 	bts->rach_b_thresh = -1;
 	bts->rach_ldavg_slots = -1;
-	bts->paging.free_chans_need = -1;
 	bts->features.data = &bts->_features_data[0];
 	bts->features.data_len = sizeof(bts->_features_data);
 
@@ -525,35 +509,6 @@ gsm_objclass2mo(struct gsm_bts *bts, uint8_t obj_class,
 	case NM_OC_SITE_MANAGER:
 		mo = &bts->site_mgr.mo;
 		break;
-	case NM_OC_BS11:
-		switch (obj_inst->bts_nr) {
-		case BS11_OBJ_CCLK:
-			mo = &bts->bs11.cclk.mo;
-			break;
-		case BS11_OBJ_BBSIG:
-			if (obj_inst->ts_nr > bts->num_trx)
-				return NULL;
-			trx = gsm_bts_trx_num(bts, obj_inst->trx_nr);
-			mo = &trx->bs11.bbsig.mo;
-			break;
-		case BS11_OBJ_PA:
-			if (obj_inst->ts_nr > bts->num_trx)
-				return NULL;
-			trx = gsm_bts_trx_num(bts, obj_inst->trx_nr);
-			mo = &trx->bs11.pa.mo;
-			break;
-		default:
-			return NULL;
-		}
-		break;
-	case NM_OC_BS11_RACK:
-		mo = &bts->bs11.rack.mo;
-		break;
-	case NM_OC_BS11_ENVABTSE:
-		if (obj_inst->trx_nr >= ARRAY_SIZE(bts->bs11.envabtse))
-			return NULL;
-		mo = &bts->bs11.envabtse[obj_inst->trx_nr].mo;
-		break;
 	case NM_OC_GPRS_NSE:
 		mo = &bts->gprs.nse.mo;
 		break;
@@ -642,9 +597,11 @@ uint8_t gsm_pchan2chan_nr(enum gsm_phys_chan_config pchan,
 {
 	uint8_t cbits, chan_nr;
 
+	OSMO_ASSERT(pchan != GSM_PCHAN_TCH_F_TCH_H_PDCH);
+	OSMO_ASSERT(pchan != GSM_PCHAN_TCH_F_PDCH);
+
 	switch (pchan) {
 	case GSM_PCHAN_TCH_F:
-	case GSM_PCHAN_TCH_F_PDCH:
 		OSMO_ASSERT(lchan_nr == 0);
 		cbits = 0x01;
 		break;
@@ -677,19 +634,15 @@ uint8_t gsm_pchan2chan_nr(enum gsm_phys_chan_config pchan,
 		cbits = 0x08;
 		cbits += lchan_nr;
 		break;
-	default:
 	case GSM_PCHAN_CCCH:
-#ifdef ROLE_BSC
-		OSMO_ASSERT(lchan_nr == 0);
-#else
-		/*
+	default:
+		/* OSMO_ASSERT(lchan_nr == 0);
 		 * FIXME: On octphy and litecell, we hit above assertion (see
 		 * Max's comment at https://gerrit.osmocom.org/589 ); disabled
 		 * for BTS until this is clarified; remove the #ifdef when it
-		 * is fixed.
+		 * is fixed. Tracked in OS#2906.
 		 */
-#warning "fix caller that passes lchan_nr != 0"
-#endif
+#pragma message "fix caller that passes lchan_nr != 0"
 		cbits = 0x10;
 		break;
 	}
@@ -701,11 +654,19 @@ uint8_t gsm_pchan2chan_nr(enum gsm_phys_chan_config pchan,
 
 uint8_t gsm_lchan2chan_nr(const struct gsm_lchan *lchan)
 {
-	enum gsm_phys_chan_config pchan = lchan->ts->pchan;
-	if (pchan == GSM_PCHAN_TCH_F_TCH_H_PDCH)
+	switch (lchan->ts->pchan) {
+	case GSM_PCHAN_TCH_F_TCH_H_PDCH:
+		/* Return chan_nr reflecting the current TS pchan, either a standard TCH kind, or the
+		 * nonstandard value reflecting PDCH for Osmocom style dyn TS. */
 		return gsm_lchan_as_pchan2chan_nr(lchan,
 						  lchan->ts->dyn.pchan_is);
-	return gsm_pchan2chan_nr(lchan->ts->pchan, lchan->ts->nr, lchan->nr);
+	case GSM_PCHAN_TCH_F_PDCH:
+		/* For ip.access style dyn TS, we always want to use the chan_nr as if it was TCH/F.
+		 * We're using custom PDCH ACT and DEACT messages that use the usual chan_nr values. */
+		return gsm_lchan_as_pchan2chan_nr(lchan, GSM_PCHAN_TCH_F);
+	default:
+		return gsm_pchan2chan_nr(lchan->ts->pchan, lchan->ts->nr, lchan->nr);
+	}
 }
 
 uint8_t gsm_lchan_as_pchan2chan_nr(const struct gsm_lchan *lchan,
@@ -755,17 +716,13 @@ struct gsm_lchan *rsl_lchan_lookup(struct gsm_bts_trx *trx, uint8_t chan_nr,
 		lch_idx = 0;	/* TCH/F */	
 		if (ts->pchan != GSM_PCHAN_TCH_F &&
 		    ts->pchan != GSM_PCHAN_PDCH &&
-		    ts->pchan != GSM_PCHAN_TCH_F_PDCH
-		    && !(ts->pchan == GSM_PCHAN_TCH_F_TCH_H_PDCH
-			 && (ts->dyn.pchan_is == GSM_PCHAN_TCH_F
-			     || ts->dyn.pchan_want == GSM_PCHAN_TCH_F)))
+		    ts->pchan != GSM_PCHAN_TCH_F_PDCH &&
+		    ts->pchan != GSM_PCHAN_TCH_F_TCH_H_PDCH)
 			ok = false;
 	} else if ((cbits & 0x1e) == 0x02) {
 		lch_idx = cbits & 0x1;	/* TCH/H */
-		if (ts->pchan != GSM_PCHAN_TCH_H
-		    && !(ts->pchan == GSM_PCHAN_TCH_F_TCH_H_PDCH
-			 && (ts->dyn.pchan_is == GSM_PCHAN_TCH_H
-			     || ts->dyn.pchan_want == GSM_PCHAN_TCH_H)))
+		if (ts->pchan != GSM_PCHAN_TCH_H &&
+		    ts->pchan != GSM_PCHAN_TCH_F_TCH_H_PDCH)
 			ok = false;
 	} else if ((cbits & 0x1c) == 0x04) {
 		lch_idx = cbits & 0x3;	/* SDCCH/4 */
@@ -851,3 +808,22 @@ bool ts_is_tch(struct gsm_bts_trx_ts *ts)
 {
 	return pchan_is_tch(ts_pchan(ts));
 }
+
+const char *gsm_trx_unit_id(struct gsm_bts_trx *trx)
+{
+	static char buf[23];
+
+	snprintf(buf, sizeof(buf), "%u/%u/%u", trx->bts->ip_access.site_id,
+		trx->bts->ip_access.bts_id, trx->nr);
+	return buf;
+}
+
+const struct value_string lchan_ciph_state_names[] = {
+	{ LCHAN_CIPH_NONE,	"NONE" },
+	{ LCHAN_CIPH_RX_REQ,	"RX_REQ" },
+	{ LCHAN_CIPH_RX_CONF,	"RX_CONF" },
+	{ LCHAN_CIPH_RXTX_REQ,	"RXTX_REQ" },
+	{ LCHAN_CIPH_RX_CONF_TX_REQ,	"RX_CONF_TX_REQ" },
+	{ LCHAN_CIPH_RXTX_CONF,	"RXTX_CONF" },
+	{ 0, NULL }
+};
