@@ -91,7 +91,7 @@ static uint32_t fn_ms_adj(uint32_t fn, const struct gsm_lchan *lchan)
 		r -= r % GSM_RTP_DURATION;
 
 		if (r != GSM_RTP_DURATION)
-			LOGP(DL1P, LOGL_ERROR, "RTP clock out of sync with lower layer:"
+			LOGP(DRTP, LOGL_ERROR, "RTP clock out of sync with lower layer:"
 				" %"PRIu32" vs %d (%"PRIu32"->%"PRIu32")\n",
 				r, GSM_RTP_DURATION, lchan->tch.last_fn, fn);
 	}
@@ -117,7 +117,9 @@ static void queue_limit_to(const char *prefix, struct llist_head *queue, unsigne
  * in front and behind data pointer */
 struct msgb *l1sap_msgb_alloc(unsigned int l2_len)
 {
-	struct msgb *msg = msgb_alloc_headroom(512, 128, "l1sap_prim");
+	int headroom = 128;
+	int size = headroom + sizeof(struct osmo_phsap_prim) + l2_len;
+	struct msgb *msg = msgb_alloc_headroom(size, headroom, "l1sap_prim");
 
 	if (!msg)
 		return NULL;
@@ -233,14 +235,14 @@ const struct value_string gsmtap_sapi_names[] = {
 
 /* send primitive as gsmtap */
 static int gsmtap_ph_data(struct osmo_phsap_prim *l1sap, uint8_t *chan_type,
-			  uint8_t *ss, uint32_t fn, uint8_t **data, int *len,
+			  uint8_t *ss, uint32_t fn, uint8_t **data, unsigned int *len,
 			  uint8_t num_agch)
 {
 	struct msgb *msg = l1sap->oph.msg;
 	uint8_t chan_nr, link_id;
 
-	*data = msg->data + sizeof(struct osmo_phsap_prim);
-	*len = msg->len - sizeof(struct osmo_phsap_prim);
+	*data = msgb_l2(msg);
+	*len = msgb_l2len(msg);
 	chan_nr = l1sap->u.data.chan_nr;
 	link_id = l1sap->u.data.link_id;
 
@@ -274,18 +276,18 @@ static int gsmtap_ph_data(struct osmo_phsap_prim *l1sap, uint8_t *chan_type,
 }
 
 static int gsmtap_pdch(struct osmo_phsap_prim *l1sap, uint8_t *chan_type,
-		       uint8_t *ss, uint32_t fn, uint8_t **data, int *len)
+		       uint8_t *ss, uint32_t fn, uint8_t **data, unsigned int *len)
 {
 	struct msgb *msg = l1sap->oph.msg;
 
-	*data = msg->data + sizeof(struct osmo_phsap_prim);
-	*len = msg->len - sizeof(struct osmo_phsap_prim);
+	*data = msgb_l2(msg);
+	*len = msgb_l2len(msg);
 
 	if (L1SAP_IS_PTCCH(fn)) {
 		*chan_type = GSMTAP_CHANNEL_PTCCH;
 		*ss = L1SAP_FN2PTCCHBLOCK(fn);
-		if (l1sap->oph.primitive
-				== PRIM_OP_INDICATION) {
+		if (l1sap->oph.primitive == PRIM_OP_INDICATION) {
+			OSMO_ASSERT(len > 0);
 			if ((*data[0]) == 7)
 				return -EINVAL;
 			(*data)++;
@@ -298,7 +300,7 @@ static int gsmtap_pdch(struct osmo_phsap_prim *l1sap, uint8_t *chan_type,
 }
 
 static int gsmtap_ph_rach(struct osmo_phsap_prim *l1sap, uint8_t *chan_type,
-	uint8_t *tn, uint8_t *ss, uint32_t *fn, uint8_t **data, int *len)
+	uint8_t *tn, uint8_t *ss, uint32_t *fn, uint8_t **data, unsigned int *len)
 {
 	uint8_t chan_nr;
 
@@ -335,15 +337,15 @@ static bool is_fill_frame(uint8_t chan_type, const uint8_t *data, unsigned int l
 		if (!memcmp(data, paging_fill, GSM_MACBLOCK_LEN))
 			return true;
 		break;
-	default:
-		return false;
+	/* don't use 'default' case here as the above only conditionally return true */
 	}
+	return false;
 }
 
 static int to_gsmtap(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 {
 	uint8_t *data;
-	int len;
+	unsigned int len;
 	uint8_t chan_type = 0, tn = 0, ss = 0;
 	uint32_t fn;
 	uint16_t uplink = GSMTAP_ARFCN_F_UPLINK;
@@ -446,29 +448,28 @@ static int l1sap_info_time_ind(struct gsm_bts *bts,
 			       struct osmo_phsap_prim *l1sap,
 			       struct info_time_ind_param *info_time_ind)
 {
-	struct gsm_bts_trx *trx;
-	struct gsm_bts_role_bts *btsb = bts->role;
 	int frames_expired;
 
-	DEBUGP(DL1P, "MPH_INFO time ind %u\n", info_time_ind->fn);
+	DEBUGPFN(DL1P, info_time_ind->fn, "Rx MPH_INFO time ind\n");
 
 	/* Calculate and check frame difference */
-	frames_expired = info_time_ind->fn - btsb->gsm_time.fn;
+	frames_expired = info_time_ind->fn - bts->gsm_time.fn;
 	if (frames_expired > 1) {
-		LOGP(DL1P, LOGL_ERROR,
-		     "Invalid condition detected: Frame difference is %"PRIu32"-%"PRIu32" > 1!\n",
-	             info_time_ind->fn, btsb->gsm_time.fn);
+		if (bts->gsm_time.fn)
+			LOGPFN(DL1P, LOGL_ERROR, info_time_ind->fn,
+			     "Invalid condition detected: Frame difference is %"PRIu32"-%"PRIu32"=%d > 1!\n",
+			     info_time_ind->fn, bts->gsm_time.fn, frames_expired);
 	}
 
 	/* Update our data structures with the current GSM time */
-	gsm_fn2gsmtime(&btsb->gsm_time, info_time_ind->fn);
+	gsm_fn2gsmtime(&bts->gsm_time, info_time_ind->fn);
 
 	/* Update time on PCU interface */
 	pcu_tx_time_ind(info_time_ind->fn);
 
 	/* increment number of RACH slots that have passed by since the
 	 * last time indication */
-	btsb->load.rach.total +=
+	bts->load.rach.total +=
 	    calc_exprd_rach_frames(bts, info_time_ind->fn) * frames_expired;
 
 	return 0;
@@ -504,13 +505,14 @@ static int l1sap_info_meas_ind(struct gsm_bts_trx *trx,
 
 	lchan = get_active_lchan_by_chan_nr(trx, info_meas_ind->chan_nr);
 	if (!lchan) {
-		LOGP(DL1P, LOGL_ERROR, "No lchan for MPH INFO MEAS IND (chan_nr=%u)\n",
-		     info_meas_ind->chan_nr);
+		LOGPFN(DL1P, LOGL_ERROR, info_meas_ind->fn,
+			"No lchan for MPH INFO MEAS IND (chan_nr=%u)\n", info_meas_ind->chan_nr);
 		return 0;
 	}
 
-	DEBUGP(DL1P, "%s MPH_INFO meas ind, ta_offs_qbits=%d, ber10k=%d, inv_rssi=%u\n",
-		gsm_lchan_name(lchan), info_meas_ind->ta_offs_qbits,
+	DEBUGPFN(DL1P, info_meas_ind->fn,
+		"%s MPH_INFO meas ind, ta_offs_256bits=%d, ber10k=%d, inv_rssi=%u\n",
+		gsm_lchan_name(lchan), info_meas_ind->ta_offs_256bits,
 		info_meas_ind->ber10k, info_meas_ind->inv_rssi);
 
 	/* in the GPRS case we are not interested in measurement
@@ -519,14 +521,15 @@ static int l1sap_info_meas_ind(struct gsm_bts_trx *trx,
 		return 0;
 
 	memset(&ulm, 0, sizeof(ulm));
-	ulm.ta_offs_qbits = info_meas_ind->ta_offs_qbits;
+	ulm.ta_offs_256bits = info_meas_ind->ta_offs_256bits;
 	ulm.ber10k = info_meas_ind->ber10k;
 	ulm.inv_rssi = info_meas_ind->inv_rssi;
+	ulm.is_sub = info_meas_ind->is_sub;
 
 	/* we assume that symbol period is 1 bit: */
-	set_ms_to_data(lchan, info_meas_ind->ta_offs_qbits / 4, true);
+	set_ms_to_data(lchan, info_meas_ind->ta_offs_256bits / 256, true);
 
-	lchan_new_ul_meas(lchan, &ulm);
+	lchan_new_ul_meas(lchan, &ulm, info_meas_ind->fn);
 
 	/* Check measurement period end and prepare the UL measurment
 	 * report at Meas period End*/
@@ -544,9 +547,9 @@ static int l1sap_mph_info_ind(struct gsm_bts_trx *trx,
 	switch (info->type) {
 	case PRIM_INFO_TIME:
 		if (trx != trx->bts->c0) {
-			LOGP(DL1P, LOGL_NOTICE, "BTS model is sending us "
-			     "PRIM_INFO_TIME for TRX %u, please fix it\n",
-			     trx->nr);
+			LOGPFN(DL1P, LOGL_NOTICE, info->u.time_ind.fn,
+				"BTS model is sending us PRIM_INFO_TIME for TRX %u, please fix it\n",
+				trx->nr);
 			rc = -1;
 		} else
 			rc = l1sap_info_time_ind(trx->bts, l1sap,
@@ -571,7 +574,7 @@ static int l1sap_info_act_cnf(struct gsm_bts_trx *trx,
 {
 	struct gsm_lchan *lchan;
 
-	LOGP(DL1P, LOGL_INFO, "activate confirm chan_nr=0x%02x trx=%d\n",
+	LOGP(DL1C, LOGL_INFO, "activate confirm chan_nr=0x%02x trx=%d\n",
 		info_act_cnf->chan_nr, trx->nr);
 
 	lchan = get_lchan_by_chan_nr(trx, info_act_cnf->chan_nr);
@@ -596,7 +599,7 @@ static int l1sap_info_rel_cnf(struct gsm_bts_trx *trx,
 {
 	struct gsm_lchan *lchan;
 
-	LOGP(DL1P, LOGL_INFO, "deactivate confirm chan_nr=0x%02x trx=%d\n",
+	LOGP(DL1C, LOGL_INFO, "deactivate confirm chan_nr=0x%02x trx=%d\n",
 		info_act_cnf->chan_nr, trx->nr);
 
 	lchan = get_lchan_by_chan_nr(trx, info_act_cnf->chan_nr);
@@ -627,7 +630,7 @@ static int l1sap_mph_info_cnf(struct gsm_bts_trx *trx,
 		rc = l1sap_info_rel_cnf(trx, l1sap, &info->u.act_cnf);
 		break;
 	default:
-		LOGP(DL1P, LOGL_NOTICE, "unknown MPH_INFO cnf type %d\n",
+		LOGP(DL1C, LOGL_NOTICE, "unknown MPH_INFO cnf type %d\n",
 			info->type);
 		break;
 	}
@@ -652,14 +655,14 @@ static int lchan_pdtch_ph_rts_ind_loop(struct gsm_lchan *lchan,
 	/* de-queue response message (loopback) */
 	loop_msg = msgb_dequeue(&lchan->dl_tch_queue);
 	if (!loop_msg) {
-		LOGP(DL1P, LOGL_NOTICE, "%s %s: no looped PDTCH message, sending empty\n",
-		     gsm_lchan_name(lchan), osmo_dump_gsmtime(tm));
+		LOGPGT(DL1P, LOGL_NOTICE, tm, "%s: no looped PDTCH message, sending empty\n",
+		     gsm_lchan_name(lchan));
 		/* empty downlink message */
 		p = msgb_put(msg, GSM_MACBLOCK_LEN);
 		memset(p, 0, GSM_MACBLOCK_LEN);
 	} else {
-		LOGP(DL1P, LOGL_NOTICE, "%s %s: looped PDTCH message of %u bytes\n",
-		     gsm_lchan_name(lchan), osmo_dump_gsmtime(tm), msgb_l2len(loop_msg));
+		LOGPGT(DL1P, LOGL_NOTICE, tm, "%s: looped PDTCH message of %u bytes\n",
+		     gsm_lchan_name(lchan), msgb_l2len(loop_msg));
 		/* copy over data from queued response message */
 		p = msgb_put(msg, msgb_l2len(loop_msg));
 		memcpy(p, msgb_l2(loop_msg), msgb_l2len(loop_msg));
@@ -691,13 +694,11 @@ static int l1sap_ph_rts_ind(struct gsm_bts_trx *trx,
 
 	gsm_fn2gsmtime(&g_time, fn);
 
-	DEBUGP(DL1P, "Rx PH-RTS.ind %s chan_nr=0x%02x link_id=0x%02xd\n",
-		osmo_dump_gsmtime(&g_time), chan_nr, link_id);
+	DEBUGPGT(DL1P, &g_time, "Rx PH-RTS.ind chan_nr=0x%02x link_id=0x%02xd\n", chan_nr, link_id);
 
 	/* reuse PH-RTS.ind for PH-DATA.req */
 	if (!msg) {
-		LOGP(DL1P, LOGL_FATAL, "RTS without msg to be reused. Please "
-			"fix!\n");
+		LOGPGT(DL1P, LOGL_FATAL, &g_time, "RTS without msg to be reused. Please fix!\n");
 		abort();
 	}
 	msgb_trim(msg, sizeof(*l1sap));
@@ -734,7 +735,7 @@ static int l1sap_ph_rts_ind(struct gsm_bts_trx *trx,
 	} else if (!(chan_nr & 0x80)) { /* only TCH/F, TCH/H, SDCCH/4 and SDCCH/8 have C5 bit cleared */
 		lchan = get_active_lchan_by_chan_nr(trx, chan_nr);
 		if (!lchan) {
-			LOGP(DL1P, LOGL_ERROR, "No lchan for PH-RTS.ind (chan_nr=%u)\n", chan_nr);
+			LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for PH-RTS.ind (chan_nr=%u)\n", chan_nr);
 			return 0;
 		}
 		if (L1SAP_IS_LINK_SACCH(link_id)) {
@@ -787,8 +788,7 @@ static int l1sap_ph_rts_ind(struct gsm_bts_trx *trx,
 			memcpy(p, fill_frame, GSM_MACBLOCK_LEN);
 	}
 
-	DEBUGP(DL1P, "Tx PH-DATA.req %s chan_nr=0x%02x link_id=0x%02x\n",
-		osmo_dump_gsmtime(&g_time), chan_nr, link_id);
+	DEBUGPGT(DL1P, &g_time, "Tx PH-DATA.req chan_nr=0x%02x link_id=0x%02x\n", chan_nr, link_id);
 
 	l1sap_down(trx, l1sap);
 
@@ -796,25 +796,45 @@ static int l1sap_ph_rts_ind(struct gsm_bts_trx *trx,
 	return 1;
 }
 
-static int check_acc_delay(struct ph_rach_ind_param *rach_ind,
-	struct gsm_bts_role_bts *btsb, uint8_t *acc_delay)
+static bool rtppayload_is_octet_aligned(const uint8_t *rtp_pl, uint8_t payload_len)
 {
-	*acc_delay = rach_ind->acc_delay;
-	return *acc_delay <= btsb->max_ta;
+	/*
+	 * Logic: If 1st bit padding is not zero, packet is either:
+	 * - bandwidth-efficient AMR payload.
+	 * - malformed packet.
+	 * However, Bandwidth-efficient AMR 4,75 frame last in payload(F=0, FT=0)
+	 * with 4th,5ht,6th AMR payload to 0 matches padding==0.
+	 * Furthermore, both AMR 4,75 bw-efficient and octet alignment are 14 bytes long (AMR 4,75 encodes 95b):
+	 * bw-efficient: 95b, + 4b hdr + 6b ToC = 105b, + padding = 112b = 14B.
+	 * octet-aligned: 1B hdr + 1B ToC + 95b = 111b, + padding = 112b = 14B.
+	 * We cannot use other fields to match since they are inside the AMR
+	 * payload bits which are unknown.
+	 * As a result, this function may return false positive (true) for some AMR
+	 * 4,75 AMR frames, but given the length, CMR and FT read is the same as a
+	 * consequence, the damage in here is harmless other than being unable to
+	 * decode the audio at the other side.
+	 */
+	#define AMR_PADDING1(rtp_pl) (rtp_pl[0] & 0x0f)
+	#define AMR_PADDING2(rtp_pl) (rtp_pl[1] & 0x03)
+
+	if(payload_len < 2 || AMR_PADDING1(rtp_pl) || AMR_PADDING2(rtp_pl))
+		return false;
+
+	return true;
 }
 
-/* special case where handover RACH is detected */
-static int l1sap_handover_rach(struct gsm_bts_trx *trx,
-	struct osmo_phsap_prim *l1sap, struct ph_rach_ind_param *rach_ind)
+static bool rtppayload_is_valid(struct gsm_lchan *lchan, struct msgb *resp_msg)
 {
-	struct gsm_lchan *lchan;
-
-	lchan = get_lchan_by_chan_nr(trx, rach_ind->chan_nr);
-
-	handover_rach(lchan, rach_ind->ra, rach_ind->acc_delay);
-
-	/* must return 0, so in case of msg at l1sap, it will be freed */
-	return 0;
+	/* Avoid sending bw-efficient AMR to lower layers, most bts models
+	 * don't support it. */
+	if(lchan->tch_mode == GSM48_CMODE_SPEECH_AMR &&
+		!rtppayload_is_octet_aligned(resp_msg->data, resp_msg->len)) {
+		LOGP(DL1P, LOGL_NOTICE,
+			"%s RTP->L1: Dropping unexpected AMR encoding (bw-efficient?) %s\n",
+			gsm_lchan_name(lchan), osmo_hexdump(resp_msg->data, resp_msg->len));
+		return false;
+	}
+	return true;
 }
 
 /* TCH-RTS-IND prim recevied from bts model */
@@ -826,8 +846,7 @@ static int l1sap_tch_rts_ind(struct gsm_bts_trx *trx,
 	struct gsm_time g_time;
 	struct gsm_lchan *lchan;
 	uint8_t chan_nr, marker = 0;
-	uint16_t seq;
-	uint32_t fn, timestamp;
+	uint32_t fn;
 	int rc;
 
 	chan_nr = rts_ind->chan_nr;
@@ -835,11 +854,11 @@ static int l1sap_tch_rts_ind(struct gsm_bts_trx *trx,
 
 	gsm_fn2gsmtime(&g_time, fn);
 
-	DEBUGP(DL1P, "Rx TCH-RTS.ind %s chan_nr=0x%02x\n", osmo_dump_gsmtime(&g_time), chan_nr);
+	DEBUGPGT(DL1P, &g_time, "Rx TCH-RTS.ind chan_nr=0x%02x\n", chan_nr);
 
 	lchan = get_active_lchan_by_chan_nr(trx, chan_nr);
 	if (!lchan) {
-		LOGP(DL1P, LOGL_ERROR, "No lchan for PH-RTS.ind (chan_nr=%u)\n", chan_nr);
+		LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for PH-RTS.ind (chan_nr=%u)\n", chan_nr);
 		return 0;
 	}
 
@@ -856,16 +875,15 @@ static int l1sap_tch_rts_ind(struct gsm_bts_trx *trx,
 	/* get a msgb from the dl_tx_queue */
 	resp_msg = msgb_dequeue(&lchan->dl_tch_queue);
 	if (!resp_msg) {
-		LOGP(DL1P, LOGL_DEBUG, "%s DL TCH Tx queue underrun\n",
-			gsm_lchan_name(lchan));
+		DEBUGPGT(DL1P, &g_time, "%s DL TCH Tx queue underrun\n", gsm_lchan_name(lchan));
+		resp_l1sap = &empty_l1sap;
+	} else if(!rtppayload_is_valid(lchan, resp_msg)) {
+		msgb_free(resp_msg);
+		resp_msg = NULL;
 		resp_l1sap = &empty_l1sap;
 	} else {
 		/* Obtain RTP header Marker bit from control buffer */
 		marker = rtpmsg_marker_bit(resp_msg);
-		/* Obtain RTP header Sequence Number from control buffer */
-		seq = rtpmsg_seq(resp_msg);
-		/* Obtain RTP header Timestamp from control buffer */
-		timestamp = rtpmsg_ts(resp_msg);
 
 		resp_msg->l2h = resp_msg->data;
 		msgb_push(resp_msg, sizeof(*resp_l1sap));
@@ -875,7 +893,7 @@ static int l1sap_tch_rts_ind(struct gsm_bts_trx *trx,
 
 	/* check for pending REL_IND */
 	if (lchan->pending_rel_ind_msg) {
-		LOGP(DRSL, LOGL_INFO, "%s Forward REL_IND to L3\n", gsm_lchan_name(lchan));
+		LOGPGT(DRSL, LOGL_INFO, &g_time, "%s Forward REL_IND to L3\n", gsm_lchan_name(lchan));
 		/* Forward it to L3 */
 		rc = abis_bts_rsl_sendmsg(lchan->pending_rel_ind_msg);
 		lchan->pending_rel_ind_msg = NULL;
@@ -890,7 +908,7 @@ static int l1sap_tch_rts_ind(struct gsm_bts_trx *trx,
 	resp_l1sap->u.tch.fn = fn;
 	resp_l1sap->u.tch.marker = marker;
 
-	DEBUGP(DL1P, "Tx TCH.req %s chan_nr=0x%02x\n", osmo_dump_gsmtime(&g_time), chan_nr);
+	DEBUGPGT(DL1P, &g_time, "Tx TCH.req chan_nr=0x%02x\n", chan_nr);
 
 	l1sap_down(trx, resp_l1sap);
 
@@ -902,10 +920,10 @@ static int l1sap_tch_rts_ind(struct gsm_bts_trx *trx,
  * network operator." */
 static void radio_link_timeout(struct gsm_lchan *lchan, int bad_frame)
 {
-	struct gsm_bts_role_bts *btsb = lchan->ts->trx->bts->role;
+	struct gsm_bts *bts = lchan->ts->trx->bts;
 
 	/* Bypass radio link timeout if set to -1 */
-	if (btsb->radio_link_timeout < 0)
+	if (bts->radio_link_timeout < 0)
 		return;
 
 	/* if link loss criterion already reached */
@@ -925,11 +943,11 @@ static void radio_link_timeout(struct gsm_lchan *lchan, int bad_frame)
 		return;
 	}
 
-	if (lchan->s < btsb->radio_link_timeout) {
+	if (lchan->s < bts->radio_link_timeout) {
 		/* count up radio link counter S */
 		lchan->s += 2;
-		if (lchan->s > btsb->radio_link_timeout)
-			lchan->s = btsb->radio_link_timeout;
+		if (lchan->s > bts->radio_link_timeout)
+			lchan->s = bts->radio_link_timeout;
 		DEBUGP(DMEAS, "%s counting up radio link counter S=%d\n",
 			gsm_lchan_name(lchan), lchan->s);
 	}
@@ -992,13 +1010,13 @@ static int l1sap_ph_data_ind(struct gsm_bts_trx *trx,
 
 	gsm_fn2gsmtime(&g_time, fn);
 
-	DEBUGP(DL1P, "Rx PH-DATA.ind %s chan_nr=0x%02x link_id=0x%02x len=%d\n",
-		osmo_dump_gsmtime(&g_time), chan_nr, link_id, len);
+	DEBUGPGT(DL1P, &g_time, "Rx PH-DATA.ind chan_nr=0x%02x link_id=0x%02x len=%d\n",
+		 chan_nr, link_id, len);
 
 	if (ts_is_pdch(&trx->ts[tn])) {
 		lchan = get_lchan_by_chan_nr(trx, chan_nr);
 		if (!lchan)
-			LOGP(DL1P, LOGL_ERROR, "No lchan for chan_nr=0x%02x\n", chan_nr);
+			LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for chan_nr=0x%02x\n", chan_nr);
 		if (lchan && lchan->loopback && !L1SAP_IS_PTCCH(fn)) {
 			/* we are in loopback mode (for BER testing)
 			 * mode and need to enqeue the frame to be
@@ -1018,7 +1036,7 @@ static int l1sap_ph_data_ind(struct gsm_bts_trx *trx,
 			pcu_tx_data_ind(&trx->ts[tn], PCU_IF_SAPI_PTCCH, fn,
 					0 /* ARFCN */, L1SAP_FN2PTCCHBLOCK(fn),
 					data, len, rssi, data_ind->ber10k,
-					data_ind->ta_offs_qbits,
+					data_ind->ta_offs_256bits/64,
 					data_ind->lqual_cb);
 		} else {
 			/* drop incomplete UL block */
@@ -1027,14 +1045,14 @@ static int l1sap_ph_data_ind(struct gsm_bts_trx *trx,
 			/* PDTCH / PACCH frame handling */
 			pcu_tx_data_ind(&trx->ts[tn], PCU_IF_SAPI_PDTCH, fn, 0 /* ARFCN */,
 					L1SAP_FN2MACBLOCK(fn), data, len, rssi, data_ind->ber10k,
-					data_ind->ta_offs_qbits, data_ind->lqual_cb);
+					data_ind->ta_offs_256bits/64, data_ind->lqual_cb);
 		}
 		return 0;
 	}
 
 	lchan = get_active_lchan_by_chan_nr(trx, chan_nr);
 	if (!lchan) {
-		LOGP(DL1P, LOGL_ERROR, "No lchan for chan_nr=%d\n", chan_nr);
+		LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for chan_nr=%d\n", chan_nr);
 		return 0;
 	}
 
@@ -1054,8 +1072,7 @@ static int l1sap_ph_data_ind(struct gsm_bts_trx *trx,
 		le = &lchan->lapdm_ch.lapdm_acch;
 		/* save the SACCH L1 header in the lchan struct for RSL MEAS RES */
 		if (len < 2) {
-			LOGP(DL1P, LOGL_NOTICE, "SACCH with size %u<2 !?!\n",
-				len);
+			LOGPGT(DL1P, LOGL_NOTICE, &g_time, "SACCH with size %u<2 !?!\n", len);
 			return -EINVAL;
 		}
 		/* Some brilliant engineer decided that the ordering of
@@ -1087,23 +1104,23 @@ static int l1sap_ph_data_ind(struct gsm_bts_trx *trx,
 static int l1sap_tch_ind(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap,
 	struct ph_tch_param *tch_ind)
 {
+	struct gsm_bts *bts = trx->bts;
 	struct msgb *msg = l1sap->oph.msg;
 	struct gsm_time g_time;
 	struct gsm_lchan *lchan;
 	uint8_t  chan_nr;
 	uint32_t fn;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(trx->bts);
 
 	chan_nr = tch_ind->chan_nr;
 	fn = tch_ind->fn;
 
 	gsm_fn2gsmtime(&g_time, fn);
 
-	DEBUGP(DL1P, "Rx TCH.ind %s chan_nr=0x%02x\n", osmo_dump_gsmtime(&g_time), chan_nr);
+	LOGPGT(DL1P, LOGL_INFO, &g_time, "Rx TCH.ind chan_nr=0x%02x\n", chan_nr);
 
 	lchan = get_active_lchan_by_chan_nr(trx, chan_nr);
 	if (!lchan) {
-		LOGP(DL1P, LOGL_ERROR, "No lchan for TCH.ind (chan_nr=%u)\n", chan_nr);
+		LOGPGT(DL1P, LOGL_ERROR, &g_time, "No lchan for TCH.ind (chan_nr=%u)\n", chan_nr);
 		return 0;
 	}
 
@@ -1113,7 +1130,7 @@ static int l1sap_tch_ind(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap,
 	 * the content is not available due to decoding issues. Content not
 	 * available is expected as empty payload. We also check if quality is
 	 * good enough. */
-	if (msg->len && tch_ind->lqual_cb / 10 >= btsb->min_qual_norm) {
+	if (msg->len && tch_ind->lqual_cb / 10 >= bts->min_qual_norm) {
 		/* hand msg to RTP code for transmission */
 		if (lchan->abis_ip.rtp_socket)
 			osmo_rtp_send_frame_ext(lchan->abis_ip.rtp_socket,
@@ -1130,7 +1147,8 @@ static int l1sap_tch_ind(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap,
 		/* Only clear the marker bit once we have sent a RTP packet with it */
 		lchan->rtp_tx_marker = false;
 	} else {
-		DEBUGP(DL1P, "Skipping RTP frame with lost payload\n");
+		DEBUGPGT(DRTP, &g_time, "Skipping RTP frame with lost payload (chan_nr=0x%02x)\n",
+			 chan_nr);
 		if (lchan->abis_ip.rtp_socket)
 			osmo_rtp_skipped_frame(lchan->abis_ip.rtp_socket, fn_ms_adj(fn, lchan));
 		lchan->rtp_tx_marker = true;
@@ -1140,38 +1158,94 @@ static int l1sap_tch_ind(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap,
 	return 0;
 }
 
+#define RACH_MIN_TOA256 -2 * 256
+
+static bool rach_pass_filter(struct ph_rach_ind_param *rach_ind, struct gsm_bts *bts)
+{
+	int16_t toa256 = rach_ind->acc_delay_256bits;
+
+	/* Check for RACH exceeding BER threshold (ghost RACH) */
+	if (rach_ind->ber10k > bts->max_ber10k_rach) {
+		LOGPFN(DL1C, LOGL_INFO, rach_ind->fn, "Ignoring RACH request: "
+			"BER10k(%u) > BER10k_MAX(%u)\n",
+			rach_ind->ber10k, bts->max_ber10k_rach);
+		return false;
+	}
+
+	/**
+	 * Make sure that ToA (Timing of Arrival) is acceptable.
+	 * We allow early arrival up to 2 symbols, and delay
+	 * according to maximal allowed Timing Advance value.
+	 */
+	if (toa256 < RACH_MIN_TOA256 || toa256 > bts->max_ta * 256) {
+		LOGPFN(DL1C, LOGL_INFO, rach_ind->fn, "Ignoring RACH request: "
+			"ToA(%d) exceeds the allowed range (%d..%d)\n",
+			toa256, RACH_MIN_TOA256, bts->max_ta * 256);
+		return false;
+	}
+
+	return true;
+}
+
+/* Special case where handover RACH is detected */
+static int l1sap_handover_rach(struct gsm_bts_trx *trx,
+	struct osmo_phsap_prim *l1sap, struct ph_rach_ind_param *rach_ind)
+{
+	/* Filter out noise / interference / ghosts */
+	if (!rach_pass_filter(rach_ind, trx->bts)) {
+		rate_ctr_inc2(trx->bts->ctrs, BTS_CTR_RACH_DROP);
+		return 0;
+	}
+
+	handover_rach(get_lchan_by_chan_nr(trx, rach_ind->chan_nr),
+		rach_ind->ra, rach_ind->acc_delay);
+
+	/* must return 0, so in case of msg at l1sap, it will be freed */
+	return 0;
+}
+
 /* RACH received from bts model */
 static int l1sap_ph_rach_ind(struct gsm_bts_trx *trx,
 	 struct osmo_phsap_prim *l1sap, struct ph_rach_ind_param *rach_ind)
 {
 	struct gsm_bts *bts = trx->bts;
-	struct gsm_bts_role_bts *btsb = bts->role;
 	struct lapdm_channel *lc;
-	uint8_t acc_delay;
 
-	DEBUGP(DL1P, "Rx PH-RA.ind");
+	DEBUGPFN(DL1P, rach_ind->fn, "Rx PH-RA.ind");
 
-	lc = &trx->ts[0].lchan[CCCH_LCHAN].lapdm_ch;
+	/* check for handover access burst on dedicated channels */
+	if (!L1SAP_IS_CHAN_RACH(rach_ind->chan_nr)) {
+		rate_ctr_inc2(trx->bts->ctrs, BTS_CTR_RACH_HO);
+		return l1sap_handover_rach(trx, l1sap, rach_ind);
+	}
 
-	/* check for under/overflow / sign */
-	if (!check_acc_delay(rach_ind, btsb, &acc_delay)) {
-		LOGP(DL1C, LOGL_INFO, "ignoring RACH request %u > max_ta(%u)\n",
-		     acc_delay, btsb->max_ta);
+	rate_ctr_inc2(trx->bts->ctrs, BTS_CTR_RACH_RCVD);
+
+	/* increment number of busy RACH slots, if required */
+	if (rach_ind->rssi >= bts->load.rach.busy_thresh)
+		bts->load.rach.busy++;
+
+	/* Filter out noise / interference / ghosts */
+	if (!rach_pass_filter(rach_ind, bts)) {
+		rate_ctr_inc2(trx->bts->ctrs, BTS_CTR_RACH_DROP);
 		return 0;
 	}
 
-	/* According to 3GPP TS 48.058 § 9.3.17 Access Delay is expressed same way as TA (number of symbols) */
-	set_ms_to_data(get_lchan_by_chan_nr(trx, rach_ind->chan_nr), acc_delay, false);
+	/* increment number of RACH slots with valid non-handover RACH burst */
+	bts->load.rach.access++;
 
-	/* check for handover rach */
-	if (!L1SAP_IS_CHAN_RACH(rach_ind->chan_nr))
-		return l1sap_handover_rach(trx, l1sap, rach_ind);
+	lc = &trx->ts[0].lchan[CCCH_LCHAN].lapdm_ch;
+
+	/* According to 3GPP TS 48.058 § 9.3.17 Access Delay is expressed same way as TA (number of symbols) */
+	set_ms_to_data(get_lchan_by_chan_nr(trx, rach_ind->chan_nr),
+		rach_ind->acc_delay, false);
 
 	/* check for packet access */
 	if ((trx == bts->c0 && L1SAP_IS_PACKET_RACH(rach_ind->ra)) ||
 		(trx == bts->c0 && rach_ind->is_11bit)) {
+		rate_ctr_inc2(trx->bts->ctrs, BTS_CTR_RACH_PS);
 
-		LOGP(DL1P, LOGL_INFO, "RACH for packet access (toa=%d, ra=%d)\n",
+		LOGPFN(DL1P, LOGL_INFO, rach_ind->fn, "RACH for packet access (toa=%d, ra=%d)\n",
 			rach_ind->acc_delay, rach_ind->ra);
 
 		pcu_tx_rach_ind(bts, rach_ind->acc_delay << 2,
@@ -1180,14 +1254,19 @@ static int l1sap_ph_rach_ind(struct gsm_bts_trx *trx,
 		return 0;
 	}
 
-	LOGP(DL1P, LOGL_INFO, "RACH for RR access (toa=%d, ra=%d)\n",
+	LOGPFN(DL1P, LOGL_INFO, rach_ind->fn, "RACH for RR access (toa=%d, ra=%d)\n",
 		rach_ind->acc_delay, rach_ind->ra);
+	rate_ctr_inc2(trx->bts->ctrs, BTS_CTR_RACH_CS);
 	lapdm_phsap_up(&l1sap->oph, &lc->lapdm_dcch);
 
 	return 0;
 }
 
-/* any L1 prim received from bts model, takes ownership of the msgb */
+/* Process any L1 prim received from bts model.
+ *
+ * This function takes ownership of the msgb.
+ * If l1sap contains a msgb, it assumes that msgb->l2h was set by lower layer.
+ */
 int l1sap_up(struct gsm_bts_trx *trx, struct osmo_phsap_prim *l1sap)
 {
 	struct msgb *msg = l1sap->oph.msg;
@@ -1319,12 +1398,11 @@ static int l1sap_chan_act_dact_modify(struct gsm_bts_trx *trx, uint8_t chan_nr,
 
 int l1sap_chan_act(struct gsm_bts_trx *trx, uint8_t chan_nr, struct tlv_parsed *tp)
 {
-	struct gsm_bts_role_bts *btsb = trx->bts->role;
 	struct gsm_lchan *lchan = get_lchan_by_chan_nr(trx, chan_nr);
 	struct gsm48_chan_desc *cd;
 	int rc;
 
-	LOGP(DL1P, LOGL_INFO, "activating channel chan_nr=0x%02x trx=%d\n",
+	LOGP(DL1C, LOGL_INFO, "activating channel chan_nr=0x%02x trx=%d\n",
 		chan_nr, trx->nr);
 
 	/* osmo-pcu calls this without a valid 'tp' parameter, so we
@@ -1338,33 +1416,41 @@ int l1sap_chan_act(struct gsm_bts_trx *trx, uint8_t chan_nr, struct tlv_parsed *
 		 * one one TRX, so we need to make sure not to activate
 		 * channels with a different TSC!! */
 		if (cd->h0.tsc != (lchan->ts->trx->bts->bsic & 7)) {
-			LOGP(DRSL, LOGL_ERROR, "lchan TSC %u != BSIC-TSC %u\n",
+			LOGP(DL1C, LOGL_ERROR, "lchan TSC %u != BSIC-TSC %u\n",
 				cd->h0.tsc, lchan->ts->trx->bts->bsic & 7);
 			return -RSL_ERR_SERV_OPT_UNIMPL;
 		}
 	}
 
 	lchan->sacch_deact = 0;
-	lchan->s = btsb->radio_link_timeout;
+	lchan->s = lchan->ts->trx->bts->radio_link_timeout;
 
 	rc = l1sap_chan_act_dact_modify(trx, chan_nr, PRIM_INFO_ACTIVATE, 0);
 	if (rc)
 		return -RSL_ERR_EQUIPMENT_FAIL;
 
 	/* Init DTX DL FSM if necessary */
-	if (trx->bts->dtxd && lchan->type != GSM_LCHAN_SDCCH)
+	if (trx->bts->dtxd && lchan->type != GSM_LCHAN_SDCCH) {
+		char name[32];
+		snprintf(name, sizeof(name), "bts%u-trx%u-ts%u-ss%u", lchan->ts->trx->bts->nr,
+			 lchan->ts->trx->nr, lchan->ts->nr, lchan->nr);
 		lchan->tch.dtx.dl_amr_fsm = osmo_fsm_inst_alloc(&dtx_dl_amr_fsm,
 								tall_bts_ctx,
 								lchan,
 								LOGL_DEBUG,
-								lchan->name);
+								name);
+		if (!lchan->tch.dtx.dl_amr_fsm) {
+			l1sap_chan_act_dact_modify(trx, chan_nr, PRIM_INFO_DEACTIVATE, 0);
+			return -RSL_ERR_EQUIPMENT_FAIL;
+		}
+	}
 	return 0;
 }
 
 int l1sap_chan_rel(struct gsm_bts_trx *trx, uint8_t chan_nr)
 {
 	struct gsm_lchan *lchan = get_lchan_by_chan_nr(trx, chan_nr);
-	LOGP(DL1P, LOGL_INFO, "deactivating channel chan_nr=0x%02x trx=%d\n",
+	LOGP(DL1C, LOGL_INFO, "deactivating channel chan_nr=0x%02x trx=%d\n",
 		chan_nr, trx->nr);
 
 	if (lchan->tch.dtx.dl_amr_fsm) {
@@ -1380,7 +1466,7 @@ int l1sap_chan_deact_sacch(struct gsm_bts_trx *trx, uint8_t chan_nr)
 {
 	struct gsm_lchan *lchan = get_lchan_by_chan_nr(trx, chan_nr);
 
-	LOGP(DL1P, LOGL_INFO, "deactivating sacch chan_nr=0x%02x trx=%d\n",
+	LOGP(DL1C, LOGL_INFO, "deactivating sacch chan_nr=0x%02x trx=%d\n",
 		chan_nr, trx->nr);
 
 	lchan->sacch_deact = 1;
@@ -1391,7 +1477,7 @@ int l1sap_chan_deact_sacch(struct gsm_bts_trx *trx, uint8_t chan_nr)
 
 int l1sap_chan_modify(struct gsm_bts_trx *trx, uint8_t chan_nr)
 {
-	LOGP(DL1P, LOGL_INFO, "modifying channel chan_nr=0x%02x trx=%d\n",
+	LOGP(DL1C, LOGL_INFO, "modifying channel chan_nr=0x%02x trx=%d\n",
 		chan_nr, trx->nr);
 
 	return l1sap_chan_act_dact_modify(trx, chan_nr, PRIM_INFO_MODIFY, 0);
