@@ -21,7 +21,7 @@
  */
 
 /*
- * Operation and Maintainance Messages
+ * Operation and Maintenance Messages
  */
 
 #include "btsconfig.h"
@@ -156,7 +156,7 @@ char *gsm_abis_mo_name(const struct gsm_abis_mo *mo)
 
 static inline void add_bts_attrs(struct msgb *msg, const struct gsm_bts *bts)
 {
-	abis_nm_put_sw_file(msg, btstype2str(GSM_BTS_TYPE_OSMOBTS), PACKAGE_VERSION, true);
+	abis_nm_put_sw_file(msg, "osmobts", PACKAGE_VERSION, true);
 	abis_nm_put_sw_file(msg, btsatttr2str(BTS_TYPE_VARIANT), btsvariant2str(bts->variant), true);
 
 	if (strlen(bts->sub_model))
@@ -173,7 +173,7 @@ static inline void add_trx_attr(struct msgb *msg, struct gsm_bts_trx *trx)
 {
 	const struct phy_instance *pinst = trx_phy_instance(trx);
 
-	abis_nm_put_sw_file(msg, btsatttr2str(TRX_PHY_VERSION), strlen(pinst->version) ? pinst->version : "Unknown",
+	abis_nm_put_sw_file(msg, btsatttr2str(TRX_PHY_VERSION), pinst && strlen(pinst->version) ? pinst->version : "Unknown",
 			    true);
 }
 
@@ -211,7 +211,7 @@ static inline int handle_attrs_trx(uint8_t *out, struct gsm_bts_trx *trx, const
 	struct msgb *attr_buf = oml_msgb_alloc();
 
 	if (!attr_buf)
-		return -ENOMEM;
+		return -NM_NACK_CANT_PERFORM;
 
 	for (i = 0; i < attr_len; i++) {
 		bool processed = false;
@@ -244,7 +244,7 @@ static inline int handle_attrs_bts(uint8_t *out, const struct gsm_bts *bts, cons
 	struct msgb *attr_buf = oml_msgb_alloc();
 
 	if (!attr_buf)
-		return -ENOMEM;
+		return -NM_NACK_CANT_PERFORM;
 
 	for (i = 0; i < attr_len; i++) {
 		switch (attr[i]) {
@@ -277,7 +277,7 @@ static int oml_tx_attr_resp(struct gsm_bts *bts, const struct abis_om_fom_hdr *f
 	     get_value_string(abis_nm_obj_class_names, foh->obj_class));
 
 	if (!nmsg)
-		return -ENOMEM;
+		return -NM_NACK_CANT_PERFORM;
 
 	switch (foh->obj_class) {
 	case NM_OC_BTS:
@@ -289,7 +289,7 @@ static int oml_tx_attr_resp(struct gsm_bts *bts, const struct abis_om_fom_hdr *f
 	default:
 		LOGP(DOML, LOGL_ERROR, "Unsupported MO class %s in Get Attribute Response\n",
 		     get_value_string(abis_nm_obj_class_names, foh->obj_class));
-		len = -EINVAL;
+		len = -NM_NACK_RES_NOTIMPL;
 	}
 
 	if (len < 0) {
@@ -301,7 +301,8 @@ static int oml_tx_attr_resp(struct gsm_bts *bts, const struct abis_om_fom_hdr *f
 	/* §9.4.64 Get Attribute Response Info */
 	msgb_tl16v_put(nmsg, NM_ATT_GET_ARI, len, resp);
 
-	return oml_mo_send_msg(&bts->mo, nmsg, NM_MT_GET_ATTR_RESP);
+	len = oml_mo_send_msg(&bts->mo, nmsg, NM_MT_GET_ATTR_RESP);
+	return (len < 0) ? -NM_NACK_CANT_PERFORM : len;
 }
 
 /* 8.8.1 sending State Changed Event Report */
@@ -321,6 +322,9 @@ int oml_tx_state_changed(struct gsm_abis_mo *mo)
 	/* 9.4.7 Availability Status */
 	msgb_tl16v_put(nmsg, NM_ATT_AVAIL_STATUS, 1, &mo->nm_state.availability);
 
+	/* 9.4.4 Administrative Status -- not in spec but also sent by nanobts */
+	msgb_tv_put(nmsg, NM_ATT_ADM_STATE, mo->nm_state.administrative);
+
 	return oml_mo_send_msg(mo, nmsg, NM_MT_STATECHG_EVENT_REP);
 }
 
@@ -382,6 +386,7 @@ int oml_mo_fom_ack_nack(struct gsm_abis_mo *mo, uint8_t orig_msg_type,
 int oml_mo_statechg_ack(struct gsm_abis_mo *mo)
 {
 	struct msgb *msg;
+	int rc = 0;
 
 	msg = oml_msgb_alloc();
 	if (!msg)
@@ -389,7 +394,12 @@ int oml_mo_statechg_ack(struct gsm_abis_mo *mo)
 
 	msgb_tv_put(msg, NM_ATT_ADM_STATE, mo->nm_state.administrative);
 
-	return oml_mo_send_msg(mo, msg, NM_MT_CHG_ADM_STATE_ACK);
+	rc = oml_mo_send_msg(mo, msg, NM_MT_CHG_ADM_STATE_ACK);
+	if (rc != 0)
+		return rc;
+
+	/* Emulate behaviour of ipaccess nanobts: Send a 'State Changed Event Report' as well. */
+	return oml_tx_state_changed(mo);
 }
 
 int oml_mo_statechg_nack(struct gsm_abis_mo *mo, uint8_t nack_cause)
@@ -464,8 +474,8 @@ int oml_mo_tx_sw_act_rep(struct gsm_abis_mo *mo)
 	return oml_mo_send_msg(mo, nmsg, NM_MT_SW_ACTIVATED_REP);
 }
 
-/* the below defaults correpsond to the libocmocore default of 1s for
- * DCCH and 2s for ACCH. The BSC should overried this via OML anyway. */
+/* The defaults below correspond to the libosmocore default of 1s for
+ * DCCH and 2s for ACCH. The BSC should override this via OML anyway. */
 const unsigned int oml_default_t200_ms[7] = {
 	[T200_SDCCH]		= 1000,
 	[T200_FACCH_F]		= 1000,
@@ -486,29 +496,28 @@ static void dl_set_t200(struct lapdm_datalink *dl, unsigned int t200_msec)
 int oml_set_lchan_t200(struct gsm_lchan *lchan)
 {
 	struct gsm_bts *bts = lchan->ts->trx->bts;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 	struct lapdm_channel *lc = &lchan->lapdm_ch;
 	unsigned int t200_dcch, t200_dcch_sapi3, t200_acch, t200_acch_sapi3;
 
 	/* set T200 for main and associated channel */
 	switch (lchan->type) {
 	case GSM_LCHAN_SDCCH:
-		t200_dcch = btsb->t200_ms[T200_SDCCH];
-		t200_dcch_sapi3 = btsb->t200_ms[T200_SDCCH_SAPI3];
-		t200_acch = btsb->t200_ms[T200_SACCH_SDCCH];
-		t200_acch_sapi3 = btsb->t200_ms[T200_SACCH_SDCCH];
+		t200_dcch = bts->t200_ms[T200_SDCCH];
+		t200_dcch_sapi3 = bts->t200_ms[T200_SDCCH_SAPI3];
+		t200_acch = bts->t200_ms[T200_SACCH_SDCCH];
+		t200_acch_sapi3 = bts->t200_ms[T200_SACCH_SDCCH];
 		break;
 	case GSM_LCHAN_TCH_F:
-		t200_dcch = btsb->t200_ms[T200_FACCH_F];
-		t200_dcch_sapi3 = btsb->t200_ms[T200_FACCH_F];
-		t200_acch = btsb->t200_ms[T200_SACCH_TCH_SAPI0];
-		t200_acch_sapi3 = btsb->t200_ms[T200_SACCH_TCH_SAPI3];
+		t200_dcch = bts->t200_ms[T200_FACCH_F];
+		t200_dcch_sapi3 = bts->t200_ms[T200_FACCH_F];
+		t200_acch = bts->t200_ms[T200_SACCH_TCH_SAPI0];
+		t200_acch_sapi3 = bts->t200_ms[T200_SACCH_TCH_SAPI3];
 		break;
 	case GSM_LCHAN_TCH_H:
-		t200_dcch = btsb->t200_ms[T200_FACCH_H];
-		t200_dcch_sapi3 = btsb->t200_ms[T200_FACCH_H];
-		t200_acch = btsb->t200_ms[T200_SACCH_TCH_SAPI0];
-		t200_acch_sapi3 = btsb->t200_ms[T200_SACCH_TCH_SAPI3];
+		t200_dcch = bts->t200_ms[T200_FACCH_H];
+		t200_dcch_sapi3 = bts->t200_ms[T200_FACCH_H];
+		t200_acch = bts->t200_ms[T200_SACCH_TCH_SAPI0];
+		t200_acch_sapi3 = bts->t200_ms[T200_SACCH_TCH_SAPI3];
 		break;
 	default:
 		return -1;
@@ -553,15 +562,8 @@ static int oml_rx_get_attr(struct gsm_bts *bts, struct msgb *msg)
 
 	rc = oml_tx_attr_resp(bts, foh, TLVP_VAL(&tp, NM_ATT_LIST_REQ_ATTR), TLVP_LEN(&tp, NM_ATT_LIST_REQ_ATTR));
 	if (rc < 0) {
-		LOGP(DOML, LOGL_ERROR, "Failed to respond to O&M Get Attributes message: %s\n", strerror(-rc));
-		switch (-rc) {
-		case ENOMEM:
-			return oml_fom_ack_nack(msg, NM_NACK_CANT_PERFORM);
-		case ENOTSUP:
-			return oml_fom_ack_nack(msg, NM_NACK_OBJCLASS_NOTSUPP);
-		default:
-			return oml_fom_ack_nack(msg, NM_NACK_RES_NOTIMPL);
-		}
+		LOGP(DOML, LOGL_ERROR, "responding to O&M Get Attributes message with NACK 0%x\n", -rc);
+		return oml_fom_ack_nack(msg, -rc);
 	}
 
 	return 0;
@@ -572,7 +574,6 @@ static int oml_rx_set_bts_attr(struct gsm_bts *bts, struct msgb *msg)
 {
 	struct abis_om_fom_hdr *foh = msgb_l3(msg);
 	struct tlv_parsed tp, *tp_merged;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 	int rc, i;
 	const uint8_t *payload;
 
@@ -627,12 +628,12 @@ static int oml_rx_set_bts_attr(struct gsm_bts *bts, struct msgb *msg)
 		payload = TLVP_VAL(&tp, NM_ATT_INTERF_BOUND);
 		for (i = 0; i < 6; i++) {
 			int16_t boundary = *payload;
-			btsb->interference.boundary[i] = -1 * boundary;
+			bts->interference.boundary[i] = -1 * boundary;
 		}
 	}
 	/* 9.4.24 Intave Parameter */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_INTAVE_PARAM, 1))
-		btsb->interference.intave = *TLVP_VAL(&tp, NM_ATT_INTAVE_PARAM);
+		bts->interference.intave = *TLVP_VAL(&tp, NM_ATT_INTAVE_PARAM);
 
 	/* 9.4.14 Connection Failure Criterion */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_CONN_FAIL_CRIT, 1)) {
@@ -642,34 +643,34 @@ static int oml_rx_set_bts_attr(struct gsm_bts *bts, struct msgb *msg)
 		case 0xFF: /* Osmocom specific Extension of TS 12.21 */
 			LOGP(DOML, LOGL_NOTICE, "WARNING: Radio Link Timeout "
 			     "explicitly disabled, only use this for lab testing!\n");
-			btsb->radio_link_timeout = -1;
+			bts->radio_link_timeout = -1;
 			break;
 		case 0x01: /* Based on uplink SACCH (radio link timeout) */
 			if (TLVP_LEN(&tp, NM_ATT_CONN_FAIL_CRIT) >= 2 &&
 			    val[1] >= 4 && val[1] <= 64) {
-				btsb->radio_link_timeout = val[1];
+				bts->radio_link_timeout = val[1];
 				break;
 			}
 			/* fall-through */
 		case 0x02: /* Based on RXLEV/RXQUAL measurements */
 		default:
 			LOGP(DOML, LOGL_NOTICE, "Given Conn. Failure Criterion "
-				"not supported. Please use critetion 0x01 with "
+				"not supported. Please use criterion 0x01 with "
 				"RADIO_LINK_TIMEOUT value of 4..64\n");
 			return oml_fom_ack_nack(msg, NM_NACK_PARAM_RANGE);
 		}
 	}
 
 	/* 9.4.53 T200 */
-	if (TLVP_PRES_LEN(&tp, NM_ATT_T200, ARRAY_SIZE(btsb->t200_ms))) {
+	if (TLVP_PRES_LEN(&tp, NM_ATT_T200, ARRAY_SIZE(bts->t200_ms))) {
 		payload = TLVP_VAL(&tp, NM_ATT_T200);
-		for (i = 0; i < ARRAY_SIZE(btsb->t200_ms); i++) {
+		for (i = 0; i < ARRAY_SIZE(bts->t200_ms); i++) {
 			uint32_t t200_ms = payload[i] * abis_nm_t200_ms[i];
 #if 0
-			btsb->t200_ms[i] = t200_ms;
+			bts->t200_ms[i] = t200_ms;
 			DEBUGP(DOML, "T200[%u]: OML=%u, mult=%u => %u ms\n",
 				i, payload[i], abis_nm_t200_mult[i],
-				btsb->t200_ms[i]);
+				bts->t200_ms[i]);
 #else
 			/* we'd rather use the 1s/2s (long) defaults by
 			 * libosmocore, as we appear to have some bug(s)
@@ -684,29 +685,31 @@ static int oml_rx_set_bts_attr(struct gsm_bts *bts, struct msgb *msg)
 
 	/* 9.4.31 Maximum Timing Advance */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_MAX_TA, 1))
-		btsb->max_ta = *TLVP_VAL(&tp, NM_ATT_MAX_TA);
+		bts->max_ta = *TLVP_VAL(&tp, NM_ATT_MAX_TA);
 
 	/* 9.4.39 Overload Period */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_OVERL_PERIOD, 1))
-		btsb->load.overload_period = *TLVP_VAL(&tp, NM_ATT_OVERL_PERIOD);
+		bts->load.overload_period = *TLVP_VAL(&tp, NM_ATT_OVERL_PERIOD);
 
 	/* 9.4.12 CCCH Load Threshold */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_CCCH_L_T, 1))
-		btsb->load.ccch.load_ind_thresh = *TLVP_VAL(&tp, NM_ATT_CCCH_L_T);
+		bts->load.ccch.load_ind_thresh = *TLVP_VAL(&tp, NM_ATT_CCCH_L_T);
 
 	/* 9.4.11 CCCH Load Indication Period */
-	if (TLVP_PRES_LEN(&tp, NM_ATT_CCCH_L_I_P, 1))
-		btsb->load.ccch.load_ind_period = *TLVP_VAL(&tp, NM_ATT_CCCH_L_I_P);
+	if (TLVP_PRES_LEN(&tp, NM_ATT_CCCH_L_I_P, 1)) {
+		bts->load.ccch.load_ind_period = *TLVP_VAL(&tp, NM_ATT_CCCH_L_I_P);
+		load_timer_start(bts);
+	}
 
 	/* 9.4.44 RACH Busy Threshold */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_RACH_B_THRESH, 1)) {
 		int16_t thresh = *TLVP_VAL(&tp, NM_ATT_RACH_B_THRESH);
-		btsb->load.rach.busy_thresh = -1 * thresh;
+		bts->load.rach.busy_thresh = -1 * thresh;
 	}
 
 	/* 9.4.45 RACH Load Averaging Slots */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_LDAVG_SLOTS, 2)) {
-		btsb->load.rach.averaging_slots =
+		bts->load.rach.averaging_slots =
 			ntohs(tlvp_val16_unal(&tp, NM_ATT_LDAVG_SLOTS));
 	}
 
@@ -718,13 +721,13 @@ static int oml_rx_set_bts_attr(struct gsm_bts *bts, struct msgb *msg)
 				"T3105 must have a value != 0.\n");
 			return oml_fom_ack_nack(msg, NM_NACK_PARAM_RANGE);
 		}
-		btsb->t3105_ms = t3105 * 10;
+		bts->t3105_ms = t3105 * 10;
 	}
 
 	/* 9.4.37 NY1 */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_NY1, 1))
-		btsb->ny1 = *TLVP_VAL(&tp, NM_ATT_NY1);
-	
+		bts->ny1 = *TLVP_VAL(&tp, NM_ATT_NY1);
+
 	/* 9.4.8 BCCH ARFCN */
 	if (TLVP_PRES_LEN(&tp, NM_ATT_BCCH_ARFCN, 2))
 		bts->c0->arfcn = ntohs(tlvp_val16_unal(&tp, NM_ATT_BCCH_ARFCN));
@@ -807,8 +810,7 @@ static int oml_rx_set_radio_attr(struct gsm_bts_trx *trx, struct msgb *msg)
 		if (length != 2) {
 			LOGP(DOML, LOGL_ERROR, "Expecting only one ARFCN, "
 				"because hopping not supported\n");
-			/* FIXME: send NACK */
-			return -ENOTSUP;
+			return oml_fom_ack_nack(msg, NM_NACK_MSGINCONSIST_PHYSCFG);
 		}
 		memcpy(&_value, value, 2);
 		arfcn = ntohs(_value);
@@ -948,6 +950,7 @@ static int oml_rx_set_chan_attr(struct gsm_bts_trx_ts *ts, struct msgb *msg)
 	/* Call into BTS driver to check attribute values */
 	rc = bts_model_check_oml(bts, foh->msg_type, ts->mo.nm_attr, tp_merged, ts);
 	if (rc < 0) {
+		LOGP(DOML, LOGL_ERROR, "SET CHAN ATTR: invalid attribute value, rc=%d\n", rc);
 		talloc_free(tp_merged);
 		/* Send NACK */
 		return oml_fom_ack_nack(msg, -rc);
@@ -963,6 +966,9 @@ static int oml_rx_set_chan_attr(struct gsm_bts_trx_ts *ts, struct msgb *msg)
 		ts->pchan = abis_nm_pchan4chcomb(comb);
 		rc = conf_lchans(ts);
 		if (rc < 0) {
+			LOGP(DOML, LOGL_ERROR, "SET CHAN ATTR: invalid Chan Comb 0x%x"
+			     " (pchan=%s, conf_lchans()->%d)\n",
+			     comb, gsm_pchan_name(ts->pchan), rc);
 			talloc_free(tp_merged);
 			/* Send NACK */
 			return oml_fom_ack_nack(msg, -rc);
@@ -976,10 +982,10 @@ static int oml_rx_set_chan_attr(struct gsm_bts_trx_ts *ts, struct msgb *msg)
 		ts->tsc = *TLVP_VAL(&tp, NM_ATT_TSC);
 	} else {
 		/* If there is no TSC specified, use the BCC */
-		ts->tsc = bts->bsic & 0x3;
+		ts->tsc = BSIC2BCC(bts->bsic);
 	}
-	LOGP(DOML, LOGL_INFO, "%s SET CHAN ATTR (TSC = %u)\n",
-		gsm_abis_mo_name(&ts->mo), ts->tsc);
+	LOGP(DOML, LOGL_INFO, "%s SET CHAN ATTR (TSC=%u pchan=%s)\n",
+		gsm_abis_mo_name(&ts->mo), ts->tsc, gsm_pchan_name(ts->pchan));
 
 	/* call into BTS driver to apply new attributes to hardware */
 	return bts_model_apply_oml(bts, msg, tp_merged, NM_OC_CHANNEL, ts);
@@ -1204,7 +1210,7 @@ static int oml_ipa_mo_set_attr_cell(void *obj, struct tlv_parsed *tp)
 
 	if (TLVP_PRES_LEN(tp, NM_ATT_IPACC_BVCI, 2))
 		bts->gprs.cell.bvci =
-			htons(tlvp_val16_unal(tp, NM_ATT_IPACC_BVCI));
+			ntohs(tlvp_val16_unal(tp, NM_ATT_IPACC_BVCI));
 
 	if (TLVP_PRES_LEN(tp, NM_ATT_IPACC_RLC_CFG, 9)) {
 		cur = TLVP_VAL(tp, NM_ATT_IPACC_RLC_CFG);
@@ -1362,8 +1368,11 @@ static int rx_oml_ipa_rsl_connect(struct gsm_bts_trx *trx, struct msgb *msg,
 	LOGP(DOML, LOGL_INFO, "Rx IPA RSL CONNECT IP=%s PORT=%u STREAM=0x%02x\n", 
 		inet_ntoa(in), port, stream_id);
 
-	rc = e1inp_ipa_bts_rsl_connect_n(oml_link->ts->line, inet_ntoa(in), port,
-		trx->nr);
+	if (trx->bts->variant == BTS_OSMO_OMLDUMMY) {
+		rc = 0;
+		LOGP(DOML, LOGL_NOTICE, "Not connecting RSL in OML-DUMMY!\n");
+	} else
+		rc = e1inp_ipa_bts_rsl_connect_n(oml_link->ts->line, inet_ntoa(in), port, trx->nr);
 	if (rc < 0) {
 		LOGP(DOML, LOGL_ERROR, "Error in abis_open(RSL): %d\n", rc);
 		return oml_fom_ack_nack(msg, NM_NACK_CANT_PERFORM);
