@@ -34,6 +34,7 @@
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/select.h>
 #include <osmocom/core/socket.h>
+#include <osmocom/gsm/gsm23003.h>
 #include <osmo-bts/logging.h>
 #include <osmo-bts/gsm_data.h>
 #include <osmo-bts/pcu_if.h>
@@ -143,8 +144,9 @@ int pcu_tx_info_ind(void)
 		info_ind->flags |= PCU_IF_FLAG_SYSMO;
 
 	/* RAI */
-	info_ind->mcc = net->mcc;
-	info_ind->mnc = net->mnc;
+	info_ind->mcc = net->plmn.mcc;
+	info_ind->mnc = net->plmn.mnc;
+	info_ind->mnc_3_digits = net->plmn.mnc_3_digits;
 	info_ind->lac = bts->location_area_code;
 	info_ind->rac = bts->gprs.rac;
 
@@ -221,8 +223,8 @@ int pcu_tx_info_ind(void)
 			if (ts->mo.nm_state.operational == NM_OPSTATE_ENABLED
 			    && ts_should_be_pdch(ts)) {
 				info_ind->trx[i].pdch_mask |= (1 << j);
-				info_ind->trx[i].tsc[j] =
-					(ts->tsc >= 0) ? ts->tsc : bts->bsic & 7;
+				info_ind->trx[i].tsc[j] = gsm_ts_tsc(ts);
+
 				LOGP(DPCU, LOGL_INFO, "trx=%d ts=%d: "
 					"available (tsc=%d arfcn=%d)\n",
 					trx->nr, ts->nr,
@@ -254,14 +256,7 @@ static int pcu_if_signal_cb(unsigned int subsys, unsigned int signal,
 			break;
 		si3 = (struct gsm48_system_information_type_3 *)
 						bts->si_buf[SYSINFO_TYPE_3];
-		net->mcc = ((si3->lai.digits[0] & 0x0f) << 8)
-			| (si3->lai.digits[0] & 0xf0)
-			| (si3->lai.digits[1] & 0x0f);
-		net->mnc = ((si3->lai.digits[2] & 0x0f) << 8)
-			| (si3->lai.digits[2] & 0xf0)
-			| ((si3->lai.digits[1] & 0xf0) >> 4);
-		if ((net->mnc & 0x00f) == 0x00f)
-			net->mnc >>= 4;
+		osmo_plmn_from_bcd(si3->lai.digits, &net->plmn);
 		bts->location_area_code = ntohs(si3->lai.lac);
 		bts->cell_identity = si3->cell_identity;
 		avail_lai = 1;
@@ -330,14 +325,13 @@ int pcu_tx_data_ind(struct gsm_bts_trx_ts *ts, uint8_t sapi, uint32_t fn,
 	struct gsm_pcu_if *pcu_prim;
 	struct gsm_pcu_if_data *data_ind;
 	struct gsm_bts *bts = ts->trx->bts;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
 	LOGP(DPCU, LOGL_DEBUG, "Sending data indication: sapi=%s arfcn=%d block=%d data=%s\n",
 	     sapi_string[sapi], arfcn, block_nr, osmo_hexdump(data, len));
 
-	if (lqual / 10 < btsb->min_qual_norm) {
+	if (lqual / 10 < bts->min_qual_norm) {
 		LOGP(DPCU, LOGL_DEBUG, "Link quality %"PRId16" is below threshold %f, dropping packet\n",
-			lqual, btsb->min_qual_norm);
+			lqual, bts->min_qual_norm);
 		return 0;
 	}
 
@@ -354,7 +348,6 @@ int pcu_tx_data_ind(struct gsm_bts_trx_ts *ts, uint8_t sapi, uint32_t fn,
 	data_ind->trx_nr = ts->trx->nr;
 	data_ind->ts_nr = ts->nr;
 	data_ind->block_nr = block_nr;
-	data_ind->rssi = rssi;
 	data_ind->ber10k = ber10k;
 	data_ind->ta_offs_qbits = bto;
 	data_ind->lqual_cb = lqual;
@@ -493,10 +486,7 @@ static int pcu_rx_data_req(struct gsm_bts *bts, uint8_t msg_type,
 			 * This might not be required, if PCU_IF_MSG_DATA_REQ
 			 * is used instead. */
 		} else {
-			struct gsm_bts_role_bts *btsb = bts->role;
-
-			paging_add_imm_ass(btsb->paging_state, data_req->data,
-				data_req->len);
+			paging_add_imm_ass(bts->paging_state, data_req->data, data_req->len);
 		}
 		break;
 	case PCU_IF_SAPI_AGCH:
@@ -521,7 +511,25 @@ static int pcu_rx_data_req(struct gsm_bts *bts, uint8_t msg_type,
 			rc = -EINVAL;
 			break;
 		}
+		if (data_req->ts_nr >= ARRAY_SIZE(trx->ts)) {
+			LOGP(DPCU, LOGL_ERROR, "Received PCU data request with "
+				"not existing TS %u\n", data_req->ts_nr);
+			rc = -EINVAL;
+			break;
+		}
 		ts = &trx->ts[data_req->ts_nr];
+		if (!ts_should_be_pdch(ts)) {
+			LOGP(DPCU, LOGL_ERROR, "%s: Received PCU DATA request for non-PDCH TS\n",
+				gsm_ts_name(ts));
+			rc = -EINVAL;
+			break;
+		}
+		if (ts->lchan[0].state != LCHAN_S_ACTIVE) {
+			LOGP(DPCU, LOGL_ERROR, "%s: Received PCU DATA request for inactive lchan\n",
+				gsm_ts_name(ts));
+			rc = -EINVAL;
+			break;
+		}
 		is_ptcch = (data_req->sapi == PCU_IF_SAPI_PTCCH);
 		rc = l1sap_pdch_req(ts, is_ptcch, data_req->fn, data_req->arfcn,
 			data_req->block_nr, data_req->data, data_req->len);
@@ -615,6 +623,10 @@ static int pcu_rx(struct gsm_network *net, uint8_t msg_type,
 	struct gsm_bts *bts;
 
 	/* FIXME: allow multiple BTS */
+	if (pcu_prim->bts_nr != 0) {
+		LOGP(DPCU, LOGL_ERROR, "Received PCU Prim for non-existent BTS %u\n", pcu_prim->bts_nr);
+		return -EINVAL;
+	}
 	bts = llist_entry(net->bts_list.next, struct gsm_bts, list);
 
 	switch (msg_type) {
@@ -749,6 +761,12 @@ static int pcu_sock_read(struct osmo_fd *bfd)
 		goto close;
 	}
 
+	if (rc < sizeof(*pcu_prim)) {
+		LOGP(DPCU, LOGL_ERROR, "Received %d bytes on PCU Socket, but primitive size "
+			"is %lu, discarding\n", rc, sizeof(*pcu_prim));
+		return 0;
+	}
+
 	rc = pcu_rx(state->net, pcu_prim->msg_type, pcu_prim);
 
 	/* as we always synchronously process the message in pcu_rx() and
