@@ -29,6 +29,7 @@
 
 #include <osmocom/core/talloc.h>
 #include <osmocom/gsm/abis_nm.h>
+#include <osmocom/gsm/protocol/gsm_04_08.h>
 #include <osmocom/vty/vty.h>
 #include <osmocom/vty/command.h>
 #include <osmocom/vty/logging.h>
@@ -48,13 +49,21 @@
 #include <osmo-bts/oml.h>
 #include <osmo-bts/signal.h>
 #include <osmo-bts/bts_model.h>
-#include <osmo-bts/pcu_if.h>
+#include <osmo-bts/pcuif_proto.h>
 #include <osmo-bts/measurement.h>
 #include <osmo-bts/vty.h>
 #include <osmo-bts/l1sap.h>
 
 #define VTY_STR	"Configure the VTY\n"
 
+#define BTS_NR_STR "BTS Number\n"
+#define TRX_NR_STR "TRX Number\n"
+#define TS_NR_STR "Timeslot Number\n"
+#define LCHAN_NR_STR "Logical Channel Number\n"
+#define BTS_TRX_STR BTS_NR_STR TRX_NR_STR
+#define BTS_TRX_TS_STR BTS_TRX_STR TS_NR_STR
+#define BTS_TRX_TS_LCHAN_STR BTS_TRX_TS_STR LCHAN_NR_STR
+
 int g_vty_port_num = OSMO_VTY_PORT_BTS;
 
 struct phy_instance *vty_get_phy_instance(struct vty *vty, int phy_nr, int inst_nr)
@@ -194,9 +203,8 @@ gDEFUN(cfg_bts_auto_band, cfg_bts_auto_band_cmd,
 	"Automatically select band for ARFCN based on configured band\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	btsb->auto_band = 1;
+	bts->auto_band = 1;
 	return CMD_SUCCESS;
 }
 
@@ -205,9 +213,8 @@ gDEFUN(cfg_bts_no_auto_band, cfg_bts_no_auto_band_cmd,
 	NO_STR "Automatically select band for ARFCN based on configured band\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	btsb->auto_band = 0;
+	bts->auto_band = 0;
 	return CMD_SUCCESS;
 }
 
@@ -238,7 +245,6 @@ DEFUN(cfg_bts_trx, cfg_bts_trx_cmd,
 
 static void config_write_bts_single(struct vty *vty, struct gsm_bts *bts)
 {
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 	struct gsm_bts_trx *trx;
 	char buf_casecnvt[256];
 	int i;
@@ -247,26 +253,26 @@ static void config_write_bts_single(struct vty *vty, struct gsm_bts *bts)
 	if (bts->description)
 		vty_out(vty, " description %s%s", bts->description, VTY_NEWLINE);
 	vty_out(vty, " band %s%s", gsm_band_name(bts->band), VTY_NEWLINE);
-	if (btsb->auto_band)
+	if (bts->auto_band)
 		vty_out(vty, " auto-band%s", VTY_NEWLINE);
 	vty_out(vty, " ipa unit-id %u %u%s",
 		bts->ip_access.site_id, bts->ip_access.bts_id, VTY_NEWLINE);
-	vty_out(vty, " oml remote-ip %s%s", btsb->bsc_oml_host, VTY_NEWLINE);
-	vty_out(vty, " rtp jitter-buffer %u", btsb->rtp_jitter_buf_ms);
-	if (btsb->rtp_jitter_adaptive)
+	vty_out(vty, " oml remote-ip %s%s", bts->bsc_oml_host, VTY_NEWLINE);
+	vty_out(vty, " rtp jitter-buffer %u", bts->rtp_jitter_buf_ms);
+	if (bts->rtp_jitter_adaptive)
 		vty_out(vty, " adaptive");
 	vty_out(vty, "%s", VTY_NEWLINE);
-	vty_out(vty, " paging queue-size %u%s", paging_get_queue_max(btsb->paging_state),
+	vty_out(vty, " paging queue-size %u%s", paging_get_queue_max(bts->paging_state),
 		VTY_NEWLINE);
-	vty_out(vty, " paging lifetime %u%s", paging_get_lifetime(btsb->paging_state),
+	vty_out(vty, " paging lifetime %u%s", paging_get_lifetime(bts->paging_state),
 		VTY_NEWLINE);
-	vty_out(vty, " uplink-power-target %d%s", btsb->ul_power_target, VTY_NEWLINE);
-	if (btsb->agch_queue_thresh_level != GSM_BTS_AGCH_QUEUE_THRESH_LEVEL_DEFAULT
-		 || btsb->agch_queue_low_level != GSM_BTS_AGCH_QUEUE_LOW_LEVEL_DEFAULT
-		 || btsb->agch_queue_high_level != GSM_BTS_AGCH_QUEUE_HIGH_LEVEL_DEFAULT)
+	vty_out(vty, " uplink-power-target %d%s", bts->ul_power_target, VTY_NEWLINE);
+	if (bts->agch_queue.thresh_level != GSM_BTS_AGCH_QUEUE_THRESH_LEVEL_DEFAULT
+		 || bts->agch_queue.low_level != GSM_BTS_AGCH_QUEUE_LOW_LEVEL_DEFAULT
+		 || bts->agch_queue.high_level != GSM_BTS_AGCH_QUEUE_HIGH_LEVEL_DEFAULT)
 		vty_out(vty, " agch-queue-mgmt threshold %d low %d high %d%s",
-			btsb->agch_queue_thresh_level, btsb->agch_queue_low_level,
-			btsb->agch_queue_high_level, VTY_NEWLINE);
+			bts->agch_queue.thresh_level, bts->agch_queue.low_level,
+			bts->agch_queue.high_level, VTY_NEWLINE);
 
 	for (i = 0; i < 32; i++) {
 		if (gsmtap_sapi_mask & (1 << i)) {
@@ -278,12 +284,16 @@ static void config_write_bts_single(struct vty *vty, struct gsm_bts *bts)
 		osmo_str2lower(buf_casecnvt, get_value_string(gsmtap_sapi_names, GSMTAP_CHANNEL_ACCH));
 		vty_out(vty, " gsmtap-sapi %s%s", buf_casecnvt, VTY_NEWLINE);
 	}
-	vty_out(vty, " min-qual-rach %.0f%s", btsb->min_qual_rach * 10.0f,
+	vty_out(vty, " min-qual-rach %.0f%s", bts->min_qual_rach * 10.0f,
 		VTY_NEWLINE);
-	vty_out(vty, " min-qual-norm %.0f%s", btsb->min_qual_norm * 10.0f,
+	vty_out(vty, " min-qual-norm %.0f%s", bts->min_qual_norm * 10.0f,
 		VTY_NEWLINE);
-	if (strcmp(btsb->pcu.sock_path, PCU_SOCK_DEFAULT))
-		vty_out(vty, " pcu-socket %s%s", btsb->pcu.sock_path, VTY_NEWLINE);
+	vty_out(vty, " max-ber10k-rach %u%s", bts->max_ber10k_rach,
+		VTY_NEWLINE);
+	if (strcmp(bts->pcu.sock_path, PCU_SOCK_DEFAULT))
+		vty_out(vty, " pcu-socket %s%s", bts->pcu.sock_path, VTY_NEWLINE);
+	if (bts->supp_meas_toa256)
+		vty_out(vty, " supp-meas-info toa256%s", VTY_NEWLINE);
 
 	bts_model_config_write_bts(vty, bts);
 
@@ -391,7 +401,6 @@ DEFUN(cfg_bts,
 	return CMD_SUCCESS;
 }
 
-#warning merge with OpenBSC?
 DEFUN(cfg_bts_unit_id,
       cfg_bts_unit_id_cmd,
       "ipa unit-id <0-65534> <0-255>",
@@ -442,12 +451,11 @@ DEFUN(cfg_bts_oml_ip,
       "OML Parameters\n" "OML IP Address\n" "OML IP Address\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	if (btsb->bsc_oml_host)
-		talloc_free(btsb->bsc_oml_host);
+	if (bts->bsc_oml_host)
+		talloc_free(bts->bsc_oml_host);
 
-	btsb->bsc_oml_host = talloc_strdup(btsb, argv[0]);
+	bts->bsc_oml_host = talloc_strdup(bts, argv[0]);
 
 	return CMD_SUCCESS;
 }
@@ -470,11 +478,10 @@ DEFUN(cfg_bts_rtp_jitbuf,
 	RTP_STR "RTP jitter buffer\n" "jitter buffer in ms\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	btsb->rtp_jitter_buf_ms = atoi(argv[0]);
+	bts->rtp_jitter_buf_ms = atoi(argv[0]);
 	if (argc > 1)
-		btsb->rtp_jitter_adaptive = true;
+		bts->rtp_jitter_adaptive = true;
 
 	return CMD_SUCCESS;
 }
@@ -488,9 +495,8 @@ DEFUN(cfg_bts_paging_queue_size,
 	        "Maximum length of BTS-internal paging queue\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	paging_set_queue_max(btsb->paging_state, atoi(argv[0]));
+	paging_set_queue_max(bts->paging_state, atoi(argv[0]));
 
 	return CMD_SUCCESS;
 }
@@ -502,9 +508,8 @@ DEFUN(cfg_bts_paging_lifetime,
 	        "Maximum lifetime of a paging record (secods)\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	paging_set_lifetime(btsb->paging_state, atoi(argv[0]));
+	paging_set_lifetime(bts->paging_state, atoi(argv[0]));
 
 	return CMD_SUCCESS;
 }
@@ -520,11 +525,10 @@ DEFUN(cfg_bts_agch_queue_mgmt_params,
 	"High water mark for cleanup\nin %% of the maximum queue length\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	btsb->agch_queue_thresh_level = atoi(argv[0]);
-	btsb->agch_queue_low_level = atoi(argv[1]);
-	btsb->agch_queue_high_level = atoi(argv[2]);
+	bts->agch_queue.thresh_level = atoi(argv[0]);
+	bts->agch_queue.low_level = atoi(argv[1]);
+	bts->agch_queue.high_level = atoi(argv[2]);
 
 	return CMD_SUCCESS;
 }
@@ -536,11 +540,10 @@ DEFUN(cfg_bts_agch_queue_mgmt_default,
 	"Reset clean parameters to default values\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	btsb->agch_queue_thresh_level = GSM_BTS_AGCH_QUEUE_THRESH_LEVEL_DEFAULT;
-	btsb->agch_queue_low_level = GSM_BTS_AGCH_QUEUE_LOW_LEVEL_DEFAULT;
-	btsb->agch_queue_high_level = GSM_BTS_AGCH_QUEUE_HIGH_LEVEL_DEFAULT;
+	bts->agch_queue.thresh_level = GSM_BTS_AGCH_QUEUE_THRESH_LEVEL_DEFAULT;
+	bts->agch_queue.low_level = GSM_BTS_AGCH_QUEUE_LOW_LEVEL_DEFAULT;
+	bts->agch_queue.high_level = GSM_BTS_AGCH_QUEUE_HIGH_LEVEL_DEFAULT;
 
 	return CMD_SUCCESS;
 }
@@ -551,9 +554,8 @@ DEFUN(cfg_bts_ul_power_target, cfg_bts_ul_power_target_cmd,
 	"Target uplink Rx level in dBm\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	btsb->ul_power_target = atoi(argv[0]);
+	bts->ul_power_target = atoi(argv[0]);
 
 	return CMD_SUCCESS;
 }
@@ -564,9 +566,8 @@ DEFUN(cfg_bts_min_qual_rach, cfg_bts_min_qual_rach_cmd,
 	"C/I level in tenth of dB\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	btsb->min_qual_rach = strtof(argv[0], NULL) / 10.0f;
+	bts->min_qual_rach = strtof(argv[0], NULL) / 10.0f;
 
 	return CMD_SUCCESS;
 }
@@ -577,9 +578,20 @@ DEFUN(cfg_bts_min_qual_norm, cfg_bts_min_qual_norm_cmd,
 	"C/I level in tenth of dB\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	btsb->min_qual_norm = strtof(argv[0], NULL) / 10.0f;
+	bts->min_qual_norm = strtof(argv[0], NULL) / 10.0f;
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_bts_max_ber_rach, cfg_bts_max_ber_rach_cmd,
+	"max-ber10k-rach <0-10000>",
+	"Set the maximum BER for valid RACH requests\n"
+	"BER in 1/10000 units (0=no BER; 100=1% BER)\n")
+{
+	struct gsm_bts *bts = vty->index;
+
+	bts->max_ber10k_rach = strtoul(argv[0], NULL, 10);
 
 	return CMD_SUCCESS;
 }
@@ -589,18 +601,39 @@ DEFUN(cfg_bts_pcu_sock, cfg_bts_pcu_sock_cmd,
 	"Configure the PCU socket file/path name\n")
 {
 	struct gsm_bts *bts = vty->index;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
-	if (btsb->pcu.sock_path) {
+	if (bts->pcu.sock_path) {
 		/* FIXME: close the interface? */
-		talloc_free(btsb->pcu.sock_path);
+		talloc_free(bts->pcu.sock_path);
 	}
-	btsb->pcu.sock_path = talloc_strdup(btsb, argv[0]);
+	bts->pcu.sock_path = talloc_strdup(bts, argv[0]);
 	/* FIXME: re-open the interface? */
 
 	return CMD_SUCCESS;
 }
 
+DEFUN(cfg_bts_supp_meas_toa256, cfg_bts_supp_meas_toa256_cmd,
+	"supp-meas-info toa256",
+	"Configure the RSL Supplementary Measurement Info\n"
+	"Report the TOA in 1/256th symbol periods\n")
+{
+	struct gsm_bts *bts = vty->index;
+
+	bts->supp_meas_toa256 = true;
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_bts_no_supp_meas_toa256, cfg_bts_no_supp_meas_toa256_cmd,
+	"no supp-meas-info toa256",
+	NO_STR "Configure the RSL Supplementary Measurement Info\n"
+	"Report the TOA in 1/256th symbol periods\n")
+{
+	struct gsm_bts *bts = vty->index;
+
+	bts->supp_meas_toa256 = false;
+	return CMD_SUCCESS;
+}
+
 
 #define DB_DBM_STR 							\
 	"Unit is dB (decibels)\n"					\
@@ -728,9 +761,26 @@ static unsigned int llist_length(struct llist_head *list)
 	return len;
 }
 
+static void bts_dump_vty_features(struct vty *vty, struct gsm_bts *bts)
+{
+	unsigned int i;
+	bool no_features = true;
+	vty_out(vty, "  Features:%s", VTY_NEWLINE);
+
+	for (i = 0; i < _NUM_BTS_FEAT; i++) {
+		if (gsm_bts_has_feature(bts, i)) {
+			vty_out(vty, "    %03u ", i);
+			vty_out(vty, "%-40s%s", get_value_string(gsm_bts_features_descs, i), VTY_NEWLINE);
+			no_features = false;
+		}
+	}
+
+	if (no_features)
+		vty_out(vty, "    (not available)%s", VTY_NEWLINE);
+}
+
 static void bts_dump_vty(struct vty *vty, struct gsm_bts *bts)
 {
-	struct gsm_bts_role_bts *btsb = bts->role;
 	struct gsm_bts_trx *trx;
 
 	vty_out(vty, "BTS %u is of %s type in band %s, has CI %u LAC %u, "
@@ -752,20 +802,20 @@ static void bts_dump_vty(struct vty *vty, struct gsm_bts *bts)
 		vty_out(vty, "  PCU version %s connected%s",
 			bts->pcu_version, VTY_NEWLINE);
 	vty_out(vty, "  Paging: Queue size %u, occupied %u, lifetime %us%s",
-		paging_get_queue_max(btsb->paging_state), paging_queue_length(btsb->paging_state),
-		paging_get_lifetime(btsb->paging_state), VTY_NEWLINE);
+		paging_get_queue_max(bts->paging_state), paging_queue_length(bts->paging_state),
+		paging_get_lifetime(bts->paging_state), VTY_NEWLINE);
 	vty_out(vty, "  AGCH: Queue limit %u, occupied %d, "
 		"dropped %"PRIu64", merged %"PRIu64", rejected %"PRIu64", "
 		"ag-res %"PRIu64", non-res %"PRIu64"%s",
-		btsb->agch_max_queue_length, btsb->agch_queue_length,
-		btsb->agch_queue_dropped_msgs, btsb->agch_queue_merged_msgs,
-		btsb->agch_queue_rejected_msgs, btsb->agch_queue_agch_msgs,
-		btsb->agch_queue_pch_msgs,
+		bts->agch_queue.max_length, bts->agch_queue.length,
+		bts->agch_queue.dropped_msgs, bts->agch_queue.merged_msgs,
+		bts->agch_queue.rejected_msgs, bts->agch_queue.agch_msgs,
+		bts->agch_queue.pch_msgs,
 		VTY_NEWLINE);
 	vty_out(vty, "  CBCH backlog queue length: %u%s",
-		llist_length(&btsb->smscb_state.queue), VTY_NEWLINE);
+		llist_length(&bts->smscb_state.queue), VTY_NEWLINE);
 	vty_out(vty, "  Paging: queue length %d, buffer space %d%s",
-		paging_queue_length(btsb->paging_state), paging_buffer_space(btsb->paging_state),
+		paging_queue_length(bts->paging_state), paging_buffer_space(bts->paging_state),
 		VTY_NEWLINE);
 	vty_out(vty, "  OML Link state: %s.%s",
 		bts->oml_link ? "connected" : "disconnected", VTY_NEWLINE);
@@ -780,12 +830,15 @@ static void bts_dump_vty(struct vty *vty, struct gsm_bts *bts)
 			vty_out(vty, "%s", VTY_NEWLINE);
 		}
 	}
+
+	bts_dump_vty_features(vty, bts);
+	vty_out_rate_ctr_group(vty, "  ", bts->ctrs);
 }
 
 
 DEFUN(show_bts, show_bts_cmd, "show bts <0-255>",
 	SHOW_STR "Display information about a BTS\n"
-		"BTS number")
+		BTS_NR_STR)
 {
 	struct gsm_network *net = gsmnet_from_vty(vty);
 	int bts_nr;
@@ -808,6 +861,427 @@ DEFUN(show_bts, show_bts_cmd, "show bts <0-255>",
 	return CMD_SUCCESS;
 }
 
+static void trx_dump_vty(struct vty *vty, struct gsm_bts_trx *trx)
+{
+	vty_out(vty, "TRX %u of BTS %u is on ARFCN %u%s",
+		trx->nr, trx->bts->nr, trx->arfcn, VTY_NEWLINE);
+	vty_out(vty, "Description: %s%s",
+		trx->description ? trx->description : "(null)", VTY_NEWLINE);
+	vty_out(vty, "  RF Nominal Power: %d dBm, reduced by %u dB, "
+		"resulting BS power: %d dBm%s",
+		trx->nominal_power, trx->max_power_red,
+		trx->nominal_power - trx->max_power_red, VTY_NEWLINE);
+	vty_out(vty, "  NM State: ");
+	net_dump_nmstate(vty, &trx->mo.nm_state);
+	vty_out(vty, "  RSL State: %s%s", trx->rsl_link? "connected" : "disconnected", VTY_NEWLINE);
+	vty_out(vty, "  Baseband Transceiver NM State: ");
+	net_dump_nmstate(vty, &trx->bb_transc.mo.nm_state);
+	vty_out(vty, "  IPA stream ID: 0x%02x%s", trx->rsl_tei, VTY_NEWLINE);
+}
+
+static inline void print_all_trx(struct vty *vty, const struct gsm_bts *bts)
+{
+	uint8_t trx_nr;
+	for (trx_nr = 0; trx_nr < bts->num_trx; trx_nr++)
+		trx_dump_vty(vty, gsm_bts_trx_num(bts, trx_nr));
+}
+
+DEFUN(show_trx,
+      show_trx_cmd,
+      "show trx [<0-255>] [<0-255>]",
+	SHOW_STR "Display information about a TRX\n"
+	BTS_TRX_STR)
+{
+	struct gsm_network *net = gsmnet_from_vty(vty);
+	struct gsm_bts *bts = NULL;
+	int bts_nr, trx_nr;
+
+	if (argc >= 1) {
+		/* use the BTS number that the user has specified */
+		bts_nr = atoi(argv[0]);
+		if (bts_nr >= net->num_bts) {
+			vty_out(vty, "%% can't find BTS '%s'%s", argv[0],
+				VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+		bts = gsm_bts_num(net, bts_nr);
+	}
+	if (argc >= 2) {
+		trx_nr = atoi(argv[1]);
+		if (trx_nr >= bts->num_trx) {
+			vty_out(vty, "%% can't find TRX '%s'%s", argv[1],
+				VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+		trx_dump_vty(vty, gsm_bts_trx_num(bts, trx_nr));
+		return CMD_SUCCESS;
+	}
+	if (bts) {
+		/* print all TRX in this BTS */
+		print_all_trx(vty, bts);
+		return CMD_SUCCESS;
+	}
+
+	for (bts_nr = 0; bts_nr < net->num_bts; bts_nr++)
+		print_all_trx(vty, gsm_bts_num(net, bts_nr));
+
+	return CMD_SUCCESS;
+}
+
+
+static void ts_dump_vty(struct vty *vty, struct gsm_bts_trx_ts *ts)
+{
+	vty_out(vty, "BTS %u, TRX %u, Timeslot %u, phys cfg %s, TSC %u",
+		ts->trx->bts->nr, ts->trx->nr, ts->nr,
+		gsm_pchan_name(ts->pchan), gsm_ts_tsc(ts));
+	if (ts->pchan == GSM_PCHAN_TCH_F_PDCH)
+		vty_out(vty, " (%s mode)",
+			ts->flags & TS_F_PDCH_ACTIVE ? "PDCH" : "TCH/F");
+	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "  NM State: ");
+	net_dump_nmstate(vty, &ts->mo.nm_state);
+}
+
+DEFUN(show_ts,
+      show_ts_cmd,
+      "show timeslot [<0-255>] [<0-255>] [<0-7>]",
+	SHOW_STR "Display information about a TS\n"
+	BTS_TRX_TS_STR)
+{
+	struct gsm_network *net = gsmnet_from_vty(vty);
+	struct gsm_bts *bts = NULL;
+	struct gsm_bts_trx *trx = NULL;
+	struct gsm_bts_trx_ts *ts = NULL;
+	int bts_nr, trx_nr, ts_nr;
+
+	if (argc >= 1) {
+		/* use the BTS number that the user has specified */
+		bts_nr = atoi(argv[0]);
+		if (bts_nr >= net->num_bts) {
+			vty_out(vty, "%% can't find BTS '%s'%s", argv[0],
+				VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+		bts = gsm_bts_num(net, bts_nr);
+	}
+	if (argc >= 2) {
+		trx_nr = atoi(argv[1]);
+		if (trx_nr >= bts->num_trx) {
+			vty_out(vty, "%% can't find TRX '%s'%s", argv[1],
+				VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+		trx = gsm_bts_trx_num(bts, trx_nr);
+	}
+	if (argc >= 3) {
+		ts_nr = atoi(argv[2]);
+		if (ts_nr >= TRX_NR_TS) {
+			vty_out(vty, "%% can't find TS '%s'%s", argv[2],
+				VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+		/* Fully Specified: print and exit */
+		ts = &trx->ts[ts_nr];
+		ts_dump_vty(vty, ts);
+		return CMD_SUCCESS;
+	}
+
+	if (bts && trx) {
+		/* Iterate over all TS in this TRX */
+		for (ts_nr = 0; ts_nr < TRX_NR_TS; ts_nr++) {
+			ts = &trx->ts[ts_nr];
+			ts_dump_vty(vty, ts);
+		}
+	} else if (bts) {
+		/* Iterate over all TRX in this BTS, TS in each TRX */
+		for (trx_nr = 0; trx_nr < bts->num_trx; trx_nr++) {
+			trx = gsm_bts_trx_num(bts, trx_nr);
+			for (ts_nr = 0; ts_nr < TRX_NR_TS; ts_nr++) {
+				ts = &trx->ts[ts_nr];
+				ts_dump_vty(vty, ts);
+			}
+		}
+	} else {
+		/* Iterate over all BTS, TRX in each BTS, TS in each TRX */
+		for (bts_nr = 0; bts_nr < net->num_bts; bts_nr++) {
+			bts = gsm_bts_num(net, bts_nr);
+			for (trx_nr = 0; trx_nr < bts->num_trx; trx_nr++) {
+				trx = gsm_bts_trx_num(bts, trx_nr);
+				for (ts_nr = 0; ts_nr < TRX_NR_TS; ts_nr++) {
+					ts = &trx->ts[ts_nr];
+					ts_dump_vty(vty, ts);
+				}
+			}
+		}
+	}
+
+	return CMD_SUCCESS;
+}
+
+/* FIXME: move this to libosmogsm */
+static const struct value_string gsm48_cmode_names[] = {
+	{ GSM48_CMODE_SIGN,		"signalling" },
+	{ GSM48_CMODE_SPEECH_V1,	"FR or HR" },
+	{ GSM48_CMODE_SPEECH_EFR,	"EFR" },
+	{ GSM48_CMODE_SPEECH_AMR,	"AMR" },
+	{ GSM48_CMODE_DATA_14k5,	"CSD(14k5)" },
+	{ GSM48_CMODE_DATA_12k0,	"CSD(12k0)" },
+	{ GSM48_CMODE_DATA_6k0,		"CSD(6k0)" },
+	{ GSM48_CMODE_DATA_3k6,		"CSD(3k6)" },
+	{ 0, NULL }
+};
+
+/* call vty_out() to print a string like " as TCH/H" for dynamic timeslots.
+ * Don't do anything if the ts is not dynamic. */
+static void vty_out_dyn_ts_status(struct vty *vty, struct gsm_bts_trx_ts *ts)
+{
+	switch (ts->pchan) {
+	case GSM_PCHAN_TCH_F_TCH_H_PDCH:
+		if (ts->dyn.pchan_is == ts->dyn.pchan_want)
+			vty_out(vty, " as %s",
+				gsm_pchan_name(ts->dyn.pchan_is));
+		else
+			vty_out(vty, " switching %s -> %s",
+				gsm_pchan_name(ts->dyn.pchan_is),
+				gsm_pchan_name(ts->dyn.pchan_want));
+		break;
+	case GSM_PCHAN_TCH_F_PDCH:
+		if ((ts->flags & TS_F_PDCH_PENDING_MASK) == 0)
+			vty_out(vty, " as %s",
+				(ts->flags & TS_F_PDCH_ACTIVE)? "PDCH"
+							      : "TCH/F");
+		else
+			vty_out(vty, " switching %s -> %s",
+				(ts->flags & TS_F_PDCH_ACTIVE)? "PDCH"
+							      : "TCH/F",
+				(ts->flags & TS_F_PDCH_ACT_PENDING)? "PDCH"
+								   : "TCH/F");
+		break;
+	default:
+		/* no dyn ts */
+		break;
+	}
+}
+
+static void lchan_dump_full_vty(struct vty *vty, struct gsm_lchan *lchan)
+{
+	struct in_addr ia;
+
+	vty_out(vty, "BTS %u, TRX %u, Timeslot %u, Lchan %u: Type %s%s",
+		lchan->ts->trx->bts->nr, lchan->ts->trx->nr, lchan->ts->nr,
+		lchan->nr, gsm_lchant_name(lchan->type), VTY_NEWLINE);
+	/* show dyn TS details, if applicable */
+	switch (lchan->ts->pchan) {
+	case GSM_PCHAN_TCH_F_TCH_H_PDCH:
+		vty_out(vty, "  Osmocom Dyn TS:");
+		vty_out_dyn_ts_status(vty, lchan->ts);
+		vty_out(vty, VTY_NEWLINE);
+		break;
+	case GSM_PCHAN_TCH_F_PDCH:
+		vty_out(vty, "  IPACC Dyn PDCH TS:");
+		vty_out_dyn_ts_status(vty, lchan->ts);
+		vty_out(vty, VTY_NEWLINE);
+		break;
+	default:
+		/* no dyn ts */
+		break;
+	}
+	vty_out(vty, "  State: %s%s%s%s",
+		gsm_lchans_name(lchan->state),
+		lchan->state == LCHAN_S_BROKEN ? " Error reason: " : "",
+		lchan->state == LCHAN_S_BROKEN ? lchan->broken_reason : "",
+		VTY_NEWLINE);
+	vty_out(vty, "  BS Power: %d dBm, MS Power: %u dBm%s",
+		lchan->ts->trx->nominal_power - lchan->ts->trx->max_power_red
+		- lchan->bs_power*2,
+		ms_pwr_dbm(lchan->ts->trx->bts->band, lchan->ms_power),
+		VTY_NEWLINE);
+	vty_out(vty, "  Channel Mode / Codec: %s%s",
+		get_value_string(gsm48_cmode_names, lchan->tch_mode),
+		VTY_NEWLINE);
+
+	if (lchan->abis_ip.bound_ip) {
+		ia.s_addr = htonl(lchan->abis_ip.bound_ip);
+		vty_out(vty, "  Bound IP: %s Port %u RTP_TYPE2=%u CONN_ID=%u%s",
+			inet_ntoa(ia), lchan->abis_ip.bound_port,
+			lchan->abis_ip.rtp_payload2, lchan->abis_ip.conn_id,
+			VTY_NEWLINE);
+	}
+	if (lchan->abis_ip.connect_ip) {
+		ia.s_addr = htonl(lchan->abis_ip.connect_ip);
+		vty_out(vty, "  Conn. IP: %s Port %u RTP_TYPE=%u SPEECH_MODE=0x%02u%s",
+			inet_ntoa(ia), lchan->abis_ip.connect_port,
+			lchan->abis_ip.rtp_payload, lchan->abis_ip.speech_mode,
+			VTY_NEWLINE);
+	}
+#define LAPDM_ESTABLISHED(link, sapi_idx) \
+		(link).datalink[sapi_idx].dl.state == LAPD_STATE_MF_EST
+	vty_out(vty, "  LAPDm SAPIs: DCCH %c%c, SACCH %c%c%s",
+		LAPDM_ESTABLISHED(lchan->lapdm_ch.lapdm_dcch, DL_SAPI0) ? '0' : '-',
+		LAPDM_ESTABLISHED(lchan->lapdm_ch.lapdm_dcch, DL_SAPI3) ? '3' : '-',
+		LAPDM_ESTABLISHED(lchan->lapdm_ch.lapdm_acch, DL_SAPI0) ? '0' : '-',
+		LAPDM_ESTABLISHED(lchan->lapdm_ch.lapdm_acch, DL_SAPI3) ? '3' : '-',
+		VTY_NEWLINE);
+#undef LAPDM_ESTABLISHED
+	vty_out(vty, "  Valid System Information: 0x%08x%s",
+		lchan->si.valid, VTY_NEWLINE);
+	/* TODO: L1 SAPI (but those are BTS speific :( */
+	/* TODO: AMR bits */
+	vty_out(vty, "  MS Timing Offset: %d, propagation delay: %d symbols %s",
+		lchan->ms_t_offs, lchan->p_offs, VTY_NEWLINE);
+	if (lchan->encr.alg_id) {
+		vty_out(vty, "  Ciphering A5/%u State: %s, N(S)=%u%s",
+			lchan->encr.alg_id-1, lchan_ciph_state_name(lchan->ciph_state),
+			lchan->ciph_ns, VTY_NEWLINE);
+	}
+	if (lchan->loopback)
+		vty_out(vty, "  RTP/PDCH Loopback Enabled%s", VTY_NEWLINE);
+	vty_out(vty, "  Radio Link Failure Counter 'S': %d%s", lchan->s, VTY_NEWLINE);
+	/* TODO: MS Power Control */
+}
+
+static void lchan_dump_short_vty(struct vty *vty, struct gsm_lchan *lchan)
+{
+	const struct gsm_meas_rep_unidir *mru = &lchan->meas.ul_res;
+
+	vty_out(vty, "BTS %u, TRX %u, Timeslot %u %s",
+		lchan->ts->trx->bts->nr, lchan->ts->trx->nr, lchan->ts->nr,
+		gsm_pchan_name(lchan->ts->pchan));
+	vty_out_dyn_ts_status(vty, lchan->ts);
+	vty_out(vty, ", Lchan %u, Type %s, State %s - "
+		"RXL-FULL-ul: %4d dBm%s",
+		lchan->nr,
+		gsm_lchant_name(lchan->type), gsm_lchans_name(lchan->state),
+		rxlev2dbm(mru->full.rx_lev),
+		VTY_NEWLINE);
+}
+
+static int dump_lchan_trx_ts(struct gsm_bts_trx_ts *ts, struct vty *vty,
+			     void (*dump_cb)(struct vty *, struct gsm_lchan *))
+{
+	int lchan_nr;
+	for (lchan_nr = 0; lchan_nr < TS_MAX_LCHAN; lchan_nr++) {
+		struct gsm_lchan *lchan = &ts->lchan[lchan_nr];
+		if (lchan->state == LCHAN_S_NONE)
+			continue;
+		dump_cb(vty, lchan);
+	}
+
+	return CMD_SUCCESS;
+}
+
+static int dump_lchan_trx(struct gsm_bts_trx *trx, struct vty *vty,
+			  void (*dump_cb)(struct vty *, struct gsm_lchan *))
+{
+	int ts_nr;
+
+	for (ts_nr = 0; ts_nr < TRX_NR_TS; ts_nr++) {
+		struct gsm_bts_trx_ts *ts = &trx->ts[ts_nr];
+		dump_lchan_trx_ts(ts, vty, dump_cb);
+	}
+
+	return CMD_SUCCESS;
+}
+
+static int dump_lchan_bts(struct gsm_bts *bts, struct vty *vty,
+			  void (*dump_cb)(struct vty *, struct gsm_lchan *))
+{
+	int trx_nr;
+
+	for (trx_nr = 0; trx_nr < bts->num_trx; trx_nr++) {
+		struct gsm_bts_trx *trx = gsm_bts_trx_num(bts, trx_nr);
+		dump_lchan_trx(trx, vty, dump_cb);
+	}
+
+	return CMD_SUCCESS;
+}
+
+static int lchan_summary(struct vty *vty, int argc, const char **argv,
+			 void (*dump_cb)(struct vty *, struct gsm_lchan *))
+{
+	struct gsm_network *net = gsmnet_from_vty(vty);
+	struct gsm_bts *bts;
+	struct gsm_bts_trx *trx;
+	struct gsm_bts_trx_ts *ts;
+	struct gsm_lchan *lchan;
+	int bts_nr, trx_nr, ts_nr, lchan_nr;
+
+	if (argc >= 1) {
+		/* use the BTS number that the user has specified */
+		bts_nr = atoi(argv[0]);
+		if (bts_nr >= net->num_bts) {
+			vty_out(vty, "%% can't find BTS %s%s", argv[0],
+				VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+		bts = gsm_bts_num(net, bts_nr);
+
+		if (argc == 1)
+			return dump_lchan_bts(bts, vty, dump_cb);
+	}
+	if (argc >= 2) {
+		trx_nr = atoi(argv[1]);
+		if (trx_nr >= bts->num_trx) {
+			vty_out(vty, "%% can't find TRX %s%s", argv[1],
+				VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+		trx = gsm_bts_trx_num(bts, trx_nr);
+
+		if (argc == 2)
+			return dump_lchan_trx(trx, vty, dump_cb);
+	}
+	if (argc >= 3) {
+		ts_nr = atoi(argv[2]);
+		if (ts_nr >= TRX_NR_TS) {
+			vty_out(vty, "%% can't find TS %s%s", argv[2],
+				VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+		ts = &trx->ts[ts_nr];
+
+		if (argc == 3)
+			return dump_lchan_trx_ts(ts, vty, dump_cb);
+	}
+	if (argc >= 4) {
+		lchan_nr = atoi(argv[3]);
+		if (lchan_nr >= TS_MAX_LCHAN) {
+			vty_out(vty, "%% can't find LCHAN %s%s", argv[3],
+				VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+		lchan = &ts->lchan[lchan_nr];
+		dump_cb(vty, lchan);
+		return CMD_SUCCESS;
+	}
+
+	for (bts_nr = 0; bts_nr < net->num_bts; bts_nr++) {
+		bts = gsm_bts_num(net, bts_nr);
+		dump_lchan_bts(bts, vty, dump_cb);
+	}
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(show_lchan,
+	show_lchan_cmd,
+	"show lchan [<0-255>] [<0-255>] [<0-7>] [<0-7>]",
+	SHOW_STR "Display information about a logical channel\n"
+	BTS_TRX_TS_LCHAN_STR)
+{
+	return lchan_summary(vty, argc, argv, lchan_dump_full_vty);
+}
+
+DEFUN(show_lchan_summary,
+	show_lchan_summary_cmd,
+	"show lchan summary [<0-255>] [<0-255>] [<0-7>] [<0-7>]",
+	SHOW_STR "Display information about a logical channel\n"
+	"Short summary\n"
+	BTS_TRX_TS_LCHAN_STR)
+{
+	return lchan_summary(vty, argc, argv, lchan_dump_short_vty);
+}
+
 static struct gsm_lchan *resolve_lchan(struct gsm_network *net,
 					const char **argv, int idx)
 {
@@ -987,7 +1461,6 @@ DEFUN(bts_t_t_l_jitter_buf,
 {
 	struct gsm_network *net = gsmnet_from_vty(vty);
 	struct gsm_lchan *lchan;
-	struct gsm_bts_role_bts *btsb;
 	int jitbuf_ms = atoi(argv[4]), rc;
 
 	lchan = resolve_lchan(net, argv, 0);
@@ -1000,9 +1473,8 @@ DEFUN(bts_t_t_l_jitter_buf,
 			VTY_NEWLINE);
 		return CMD_WARNING;
 	}
-	btsb = bts_role_bts(lchan->ts->trx->bts);
 	rc = osmo_rtp_socket_set_param(lchan->abis_ip.rtp_socket,
-				  btsb->rtp_jitter_adaptive ?
+				  lchan->ts->trx->bts->rtp_jitter_adaptive ?
 				  OSMO_RTP_P_JIT_ADAP : OSMO_RTP_P_JITBUF,
 				  jitbuf_ms);
 	if (rc < 0)
@@ -1067,13 +1539,17 @@ int bts_vty_init(struct gsm_bts *bts, const struct log_info *cat)
 						"\n", "", 0);
 
 	install_element_ve(&show_bts_cmd);
+	install_element_ve(&show_trx_cmd);
+	install_element_ve(&show_ts_cmd);
+	install_element_ve(&show_lchan_cmd);
+	install_element_ve(&show_lchan_summary_cmd);
 
 	logging_vty_add_cmds(cat);
+	osmo_talloc_vty_add_cmds();
 
 	install_node(&bts_node, config_write_bts);
 	install_element(CONFIG_NODE, &cfg_bts_cmd);
 	install_element(CONFIG_NODE, &cfg_vty_telnet_port_cmd);
-	install_default(BTS_NODE);
 	install_element(BTS_NODE, &cfg_bts_unit_id_cmd);
 	install_element(BTS_NODE, &cfg_bts_oml_ip_cmd);
 	install_element(BTS_NODE, &cfg_bts_rtp_bind_ip_cmd);
@@ -1088,7 +1564,10 @@ int bts_vty_init(struct gsm_bts *bts, const struct log_info *cat)
 	install_element(BTS_NODE, &cfg_bts_ul_power_target_cmd);
 	install_element(BTS_NODE, &cfg_bts_min_qual_rach_cmd);
 	install_element(BTS_NODE, &cfg_bts_min_qual_norm_cmd);
+	install_element(BTS_NODE, &cfg_bts_max_ber_rach_cmd);
 	install_element(BTS_NODE, &cfg_bts_pcu_sock_cmd);
+	install_element(BTS_NODE, &cfg_bts_supp_meas_toa256_cmd);
+	install_element(BTS_NODE, &cfg_bts_no_supp_meas_toa256_cmd);
 
 	install_element(BTS_NODE, &cfg_trx_gsmtap_sapi_cmd);
 	install_element(BTS_NODE, &cfg_trx_no_gsmtap_sapi_cmd);
@@ -1096,7 +1575,6 @@ int bts_vty_init(struct gsm_bts *bts, const struct log_info *cat)
 	/* add and link to TRX config node */
 	install_element(BTS_NODE, &cfg_bts_trx_cmd);
 	install_node(&trx_node, config_write_dummy);
-	install_default(TRX_NODE);
 
 	install_element(TRX_NODE, &cfg_trx_user_gain_cmd);
 	install_element(TRX_NODE, &cfg_trx_pr_max_initial_cmd);
@@ -1111,12 +1589,10 @@ int bts_vty_init(struct gsm_bts *bts, const struct log_info *cat)
 
 	install_element(CONFIG_NODE, &cfg_phy_cmd);
 	install_node(&phy_node, config_write_phy);
-	install_default(PHY_NODE);
 	install_element(PHY_NODE, &cfg_phy_inst_cmd);
 	install_element(PHY_NODE, &cfg_phy_no_inst_cmd);
 
 	install_node(&phy_inst_node, config_write_dummy);
-	install_default(PHY_INST_NODE);
 
 	return 0;
 }
