@@ -343,8 +343,7 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 	int len;
 
 	if (!msg) {
-		LOGP(DL1C, LOGL_FATAL, "PH-DATA.req without msg. "
-			"Please fix!\n");
+		LOGPFN(DL1C, LOGL_FATAL, l1sap->u.data.fn, "PH-DATA.req without msg. Please fix!\n");
 		abort();
 	}
 
@@ -394,7 +393,7 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 		else
 			sapi = GsmL1_Sapi_Agch;
 	} else {
-		LOGP(DL1C, LOGL_NOTICE, "unknown prim %d op %d "
+		LOGPFN(DL1C, LOGL_NOTICE, u32Fn, "unknown prim %d op %d "
 			"chan_nr %d link_id %d\n", l1sap->oph.primitive,
 			l1sap->oph.operation, chan_nr, link_id);
 		msgb_free(l1msg);
@@ -457,7 +456,7 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 			memcpy(l1p->u.phDataReq.msgUnitParam.u8Buffer, msg->l2h,
 			       msgb_l2len(msg));
 		}
-		LOGP(DL1P, LOGL_DEBUG, "PH-DATA.req(%s)\n",
+		LOGPFN(DL1P, LOGL_DEBUG, u32Fn, "PH-DATA.req(%s)\n",
 		     osmo_hexdump(l1p->u.phDataReq.msgUnitParam.u8Buffer,
 					  l1p->u.phDataReq.msgUnitParam.u8Size));
 	} else {
@@ -469,7 +468,7 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 
 	/* send message to DSP's queue */
 	if (osmo_wqueue_enqueue(&fl1->write_q[MQ_L1_WRITE], l1msg) != 0) {
-		LOGP(DL1P, LOGL_ERROR, "MQ_L1_WRITE queue full. Dropping msg.\n");
+		LOGPFN(DL1P, LOGL_ERROR, u32Fn, "MQ_L1_WRITE queue full. Dropping msg.\n");
 		msgb_free(l1msg);
 	} else
 		dtx_int_signal(lchan);
@@ -838,8 +837,7 @@ static int handle_ph_readytosend_ind(struct lc15l1_hdl *fl1,
 
 	gsm_fn2gsmtime(&g_time, rts_ind->u32Fn);
 
-	DEBUGP(DL1P, "Rx PH-RTS.ind %02u/%02u/%02u SAPI=%s\n",
-		g_time.t1, g_time.t2, g_time.t3,
+	DEBUGPGT(DL1P, &g_time, "Rx PH-RTS.ind SAPI=%s\n",
 		get_value_string(lc15bts_l1sapi_names, rts_ind->sapi));
 
 	/* in all other cases, we need to allocate a new PH-DATA.ind
@@ -877,7 +875,7 @@ tx:
 
 	/* transmit */
 	if (osmo_wqueue_enqueue(&fl1->write_q[MQ_L1_WRITE], resp_msg) != 0) {
-		LOGP(DL1C, LOGL_ERROR, "MQ_L1_WRITE queue full. Dropping msg.\n");
+		LOGPGT(DL1C, LOGL_ERROR, &g_time, "MQ_L1_WRITE queue full. Dropping msg.\n");
 		msgb_free(resp_msg);
 	}
 
@@ -909,8 +907,8 @@ static int process_meas_res(struct gsm_bts_trx *trx, uint8_t chan_nr,
 		PRIM_OP_INDICATION, NULL);
 	l1sap.u.info.type = PRIM_INFO_MEAS;
 	l1sap.u.info.u.meas_ind.chan_nr = chan_nr;
-	l1sap.u.info.u.meas_ind.ta_offs_qbits = m->i16BurstTiming;
-	l1sap.u.info.u.meas_ind.ber10k = (unsigned int) (m->fBer * 100);
+	l1sap.u.info.u.meas_ind.ta_offs_256bits = m->i16BurstTiming*64;
+	l1sap.u.info.u.meas_ind.ber10k = (unsigned int) (m->fBer * 10000);
 	l1sap.u.info.u.meas_ind.inv_rssi = (uint8_t) (m->fRssi * -1);
 	l1sap.u.info.u.meas_ind.fn = fn;
 
@@ -933,24 +931,23 @@ static int handle_ph_data_ind(struct lc15l1_hdl *fl1, GsmL1_PhDataInd_t *data_in
 
 	chan_nr = chan_nr_by_sapi(&trx->ts[data_ind->u8Tn], data_ind->sapi,
 		data_ind->subCh, data_ind->u8Tn, data_ind->u32Fn);
+	fn = data_ind->u32Fn;
+	link_id =  (data_ind->sapi == GsmL1_Sapi_Sacch) ? LID_SACCH : LID_DEDIC;
+	gsm_fn2gsmtime(&g_time, fn);
+
 	if (!chan_nr) {
-		LOGP(DL1C, LOGL_ERROR, "PH-DATA-INDICATION for unknown sapi %s (%d)\n",
+		LOGPGT(DL1C, LOGL_ERROR, &g_time, "PH-DATA-INDICATION for unknown sapi %s (%d)\n",
 		     get_value_string(lc15bts_l1sapi_names, data_ind->sapi), data_ind->sapi);
 		msgb_free(l1p_msg);
 		return ENOTSUP;
 	}
-	fn = data_ind->u32Fn;
-	link_id =  (data_ind->sapi == GsmL1_Sapi_Sacch) ? LID_SACCH : LID_DEDIC;
 
 	process_meas_res(trx, chan_nr, &data_ind->measParam, fn);
 
-	gsm_fn2gsmtime(&g_time, fn);
 
-	DEBUGP(DL1P, "Rx PH-DATA.ind %s %s (hL2 %08x): %s\n",
-		get_value_string(lc15bts_l1sapi_names, data_ind->sapi),
-		osmo_dump_gsmtime(&g_time), (uint32_t)data_ind->hLayer2,
-		osmo_hexdump(data_ind->msgUnitParam.u8Buffer,
-			     data_ind->msgUnitParam.u8Size));
+	DEBUGPGT(DL1P, &g_time, "Rx PH-DATA.ind %s (hL2 %08x): %s\n",
+		get_value_string(lc15bts_l1sapi_names, data_ind->sapi), (uint32_t)data_ind->hLayer2,
+		osmo_hexdump(data_ind->msgUnitParam.u8Buffer, data_ind->msgUnitParam.u8Size));
 	dump_meas_res(LOGL_DEBUG, &data_ind->measParam);
 
 	/* check for TCH */
@@ -986,7 +983,7 @@ static int handle_ph_data_ind(struct lc15l1_hdl *fl1, GsmL1_PhDataInd_t *data_in
 	l1sap->u.data.rssi = rssi;
 	if (!pcu_direct) {
 		l1sap->u.data.ber10k = data_ind->measParam.fBer * 10000;
-		l1sap->u.data.ta_offs_qbits = data_ind->measParam.i16BurstTiming;
+		l1sap->u.data.ta_offs_256bits = data_ind->measParam.i16BurstTiming*64;
 		l1sap->u.data.lqual_cb = data_ind->measParam.fLinkQuality * 10;
 	}
 	return l1sap_up(trx, l1sap);
@@ -997,93 +994,94 @@ static int handle_ph_ra_ind(struct lc15l1_hdl *fl1, GsmL1_PhRaInd_t *ra_ind,
 {
 	struct gsm_bts_trx *trx = lc15l1_hdl_trx(fl1);
 	struct gsm_bts *bts = trx->bts;
-	struct gsm_bts_role_bts *btsb = bts->role;
 	struct gsm_lchan *lchan;
 	struct osmo_phsap_prim *l1sap;
-	uint32_t fn;
-	uint8_t acc_delay = 0;
-	uint16_t ra = 0, is_11bit = 0, burst_type = 0, temp = 0;
 	int rc;
+	struct ph_rach_ind_param rach_ind_param;
 
-	/* increment number of busy RACH slots, if required */
-	if (trx == bts->c0 &&
-	    ra_ind->measParam.fRssi >= btsb->load.rach.busy_thresh)
-		btsb->load.rach.busy++;
-
-	if (ra_ind->measParam.fLinkQuality < btsb->min_qual_rach) {
+	/* FIXME: this should be deprecated/obsoleted as it bypasses rach.busy counting */
+	if (ra_ind->measParam.fLinkQuality < bts->min_qual_rach) {
 		msgb_free(l1p_msg);
 		return 0;
 	}
 
-	if (ra_ind->measParam.i16BurstTiming > 0)
-		acc_delay = ra_ind->measParam.i16BurstTiming >> 2;
-
-	/* increment number of RACH slots with valid non-handover RACH burst */
-	lchan = l1if_hLayer_to_lchan(trx, (uint32_t)ra_ind->hLayer2);
-	if (trx == bts->c0 && !(lchan && lchan->ho.active == HANDOVER_ENABLED))
-		btsb->load.rach.access++;
-
 	dump_meas_res(LOGL_DEBUG, &ra_ind->measParam);
-	burst_type = ra_ind->burstType;
 
 	if ((ra_ind->msgUnitParam.u8Size != 1) &&
 		(ra_ind->msgUnitParam.u8Size != 2)) {
-		LOGP(DL1C, LOGL_ERROR, "PH-RACH-INDICATION has %d bits\n",
-			ra_ind->sapi);
+		LOGPFN(DL1P, LOGL_ERROR, ra_ind->u32Fn, "PH-RACH-INDICATION has %d bits\n", ra_ind->sapi);
 		msgb_free(l1p_msg);
 		return 0;
 	}
 
+	/* We need to evaluate ra_ind before below msgb_trim(), since that invalidates *ra_ind. */
+	rach_ind_param = (struct ph_rach_ind_param) {
+		/* .chan_nr set below */
+		/* .ra set below */
+		.acc_delay = 0,
+		.fn = ra_ind->u32Fn,
+		/* .is_11bit set below */
+		/* .burst_type set below */
+		.rssi = (int8_t) ra_ind->measParam.fRssi,
+		.ber10k = (unsigned int) (ra_ind->measParam.fBer * 10000.0),
+		.acc_delay_256bits = ra_ind->measParam.i16BurstTiming * 64,
+	};
+
+	lchan = l1if_hLayer_to_lchan(trx, (uint32_t)ra_ind->hLayer2);
+	if (!lchan || lchan->ts->pchan == GSM_PCHAN_CCCH ||
+	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4 ||
+	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH)
+		rach_ind_param.chan_nr = 0x88;
+	else
+		rach_ind_param.chan_nr = gsm_lchan2chan_nr(lchan);
 
 	if (ra_ind->msgUnitParam.u8Size == 2) {
-		is_11bit = 1;
-		ra = ra_ind->msgUnitParam.u8Buffer[0];
+		uint16_t temp;
+		uint16_t ra = ra_ind->msgUnitParam.u8Buffer[0];
 		ra = ra << 3;
 		temp = (ra_ind->msgUnitParam.u8Buffer[1] & 0x7);
 		ra = ra | temp;
+		rach_ind_param.is_11bit = 1;
+		rach_ind_param.ra = ra;
 	} else {
-		is_11bit = 0;
-		ra = ra_ind->msgUnitParam.u8Buffer[0];
+		rach_ind_param.is_11bit = 0;
+		rach_ind_param.ra = ra_ind->msgUnitParam.u8Buffer[0];
 	}
 
-	fn = ra_ind->u32Fn;
+	/* the old legacy full-bits acc_delay cannot express negative values */
+	if (ra_ind->measParam.i16BurstTiming > 0)
+		rach_ind_param.acc_delay = ra_ind->measParam.i16BurstTiming >> 2;
+
+	/* mapping of the burst type, the values are specific to
+	 * osmo-bts-litecell15 */
+	switch (ra_ind->burstType) {
+	case GsmL1_BurstType_Access_0:
+		rach_ind_param.burst_type =
+			GSM_L1_BURST_TYPE_ACCESS_0;
+		break;
+	case GsmL1_BurstType_Access_1:
+		rach_ind_param.burst_type =
+			GSM_L1_BURST_TYPE_ACCESS_1;
+		break;
+	case GsmL1_BurstType_Access_2:
+		rach_ind_param.burst_type =
+			GSM_L1_BURST_TYPE_ACCESS_2;
+		break;
+	default:
+		rach_ind_param.burst_type =
+			GSM_L1_BURST_TYPE_NONE;
+		break;
+	}
+
+	/* msgb_trim() invalidates ra_ind, make that abundantly clear: */
+	ra_ind = NULL;
 	rc = msgb_trim(l1p_msg, sizeof(*l1sap));
 	if (rc < 0)
 		MSGB_ABORT(l1p_msg, "No room for primitive data\n");
 	l1sap = msgb_l1sap_prim(l1p_msg);
 	osmo_prim_init(&l1sap->oph, SAP_GSM_PH, PRIM_PH_RACH, PRIM_OP_INDICATION,
 		l1p_msg);
-	l1sap->u.rach_ind.ra = ra;
-	l1sap->u.rach_ind.acc_delay = acc_delay;
-	l1sap->u.rach_ind.fn = fn;
-	l1sap->u.rach_ind.is_11bit = is_11bit; /* no of bits in 11 bit RACH */
-
-	/* mapping of the burst type, the values are specific to
-	 * osmo-bts-litecell15 */
-	switch (burst_type) {
-	case GsmL1_BurstType_Access_0:
-		l1sap->u.rach_ind.burst_type =
-			GSM_L1_BURST_TYPE_ACCESS_0;
-		break;
-	case GsmL1_BurstType_Access_1:
-		l1sap->u.rach_ind.burst_type =
-			GSM_L1_BURST_TYPE_ACCESS_1;
-		break;
-	case GsmL1_BurstType_Access_2:
-		l1sap->u.rach_ind.burst_type =
-			GSM_L1_BURST_TYPE_ACCESS_2;
-		break;
-	default:
-		l1sap->u.rach_ind.burst_type =
-			GSM_L1_BURST_TYPE_NONE;
-		break;
-	}
-
-	if (!lchan || lchan->ts->pchan == GSM_PCHAN_CCCH ||
-	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4)
-		l1sap->u.rach_ind.chan_nr = 0x88;
-	else
-		l1sap->u.rach_ind.chan_nr = gsm_lchan2chan_nr(lchan);
+	l1sap->u.rach_ind = rach_ind_param;
 
 	return l1sap_up(trx, l1sap);
 }
