@@ -979,95 +979,75 @@ int lc15bts_mgr_vty_init(void)
 
 	install_node(&mgr_node, config_write_mgr);
 	install_element(CONFIG_NODE, &cfg_mgr_cmd);
-	vty_install_default(MGR_NODE);
 
 	/* install the limit nodes */
 	install_node(&limit_supply_temp_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_supply_temp_cmd);
-	vty_install_default(LIMIT_SUPPLY_TEMP_NODE);
 
 	install_node(&limit_soc_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_soc_temp_cmd);
-	vty_install_default(LIMIT_SOC_NODE);
 
 	install_node(&limit_fpga_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_fpga_temp_cmd);
-	vty_install_default(LIMIT_FPGA_NODE);
 
 	install_node(&limit_rmsdet_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_rmsdet_temp_cmd);
-	vty_install_default(LIMIT_RMSDET_NODE);
 
 	install_node(&limit_ocxo_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_ocxo_temp_cmd);
-	vty_install_default(LIMIT_OCXO_NODE);
 
 	install_node(&limit_tx0_temp_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_tx0_temp_cmd);
-	vty_install_default(LIMIT_TX0_TEMP_NODE);
 
 	install_node(&limit_tx1_temp_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_tx1_temp_cmd);
-	vty_install_default(LIMIT_TX1_TEMP_NODE);
 
 	install_node(&limit_pa0_temp_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_pa0_temp_cmd);
-	vty_install_default(LIMIT_PA0_TEMP_NODE);
 
 	install_node(&limit_pa1_temp_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_pa1_temp_cmd);
-	vty_install_default(LIMIT_PA1_TEMP_NODE);
 
 	install_node(&limit_supply_volt_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_supply_volt_cmd);
 	register_limit(LIMIT_SUPPLY_VOLT_NODE, MGR_LIMIT_TYPE_VOLT);
-	vty_install_default(LIMIT_SUPPLY_VOLT_NODE);
 
 	install_node(&limit_tx0_vswr_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_tx0_vswr_cmd);
 	register_limit(LIMIT_TX0_VSWR_NODE, MGR_LIMIT_TYPE_VSWR);
-	vty_install_default(LIMIT_TX0_VSWR_NODE);
 
 	install_node(&limit_tx1_vswr_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_tx1_vswr_cmd);
 	register_limit(LIMIT_TX1_VSWR_NODE, MGR_LIMIT_TYPE_VSWR);
-	vty_install_default(LIMIT_TX1_VSWR_NODE);
 
 	install_node(&limit_supply_pwr_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_supply_pwr_cmd);
 	register_limit(LIMIT_SUPPLY_PWR_NODE, MGR_LIMIT_TYPE_PWR);
-	vty_install_default(LIMIT_SUPPLY_PWR_NODE);
 
 	install_node(&limit_pa0_pwr_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_pa0_pwr_cmd);
 	register_limit(LIMIT_PA0_PWR_NODE, MGR_LIMIT_TYPE_PWR);
-	vty_install_default(LIMIT_PA0_PWR_NODE);
 
 	install_node(&limit_pa1_pwr_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_pa1_pwr_cmd);
 	register_limit(LIMIT_PA1_PWR_NODE, MGR_LIMIT_TYPE_PWR);
-	vty_install_default(LIMIT_PA1_PWR_NODE);
 
 	install_node(&limit_gps_fix_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_limit_gps_fix_cmd);
-	vty_install_default(LIMIT_GPS_FIX_NODE);
 
 	/* install the normal node */
 	install_node(&act_norm_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_action_normal_cmd);
 	register_normal_action(ACT_NORM_NODE);
-	vty_install_default(ACT_NORM_NODE);
 
 	/* install the warning and critical node */
 	install_node(&act_warn_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_action_warn_cmd);
 	register_action(ACT_WARN_NODE);
-	vty_install_default(ACT_WARN_NODE);
 
 	install_node(&act_crit_node, config_write_dummy);
 	install_element(MGR_NODE, &cfg_action_critical_cmd);
 	register_action(ACT_CRIT_NODE);
-	vty_install_default(ACT_CRIT_NODE);
 
 	/* install LED pattern command for debugging purpose */
 	install_element_ve(&set_led_pattern_cmd);
