@@ -478,7 +478,7 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 	int rc;
 
 	if (!msg) {
-		LOGP(DL1C, LOGL_FATAL, "L1SAP PH-DATA.req without msg. "
+		LOGPFN(DL1C, LOGL_FATAL, l1sap->u.data.fn, "L1SAP PH-DATA.req without msg. "
 		     "Please fix!\n");
 		abort();
 	}
@@ -516,13 +516,10 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 	} else if (L1SAP_IS_CHAN_BCCH(chan_nr)) {
 		sapi = cOCTVC1_GSM_SAPI_ENUM_BCCH;
 	} else if (L1SAP_IS_CHAN_AGCH_PCH(chan_nr)) {
-#warning Set BS_AG_BLKS_RES
 		sapi = cOCTVC1_GSM_SAPI_ENUM_PCH_AGCH;
 	} else {
-		LOGP(DL1C, LOGL_NOTICE, "unknown prim %d op %d "
-		     "chan_nr %d link_id %d\n",
-		     l1sap->oph.primitive, l1sap->oph.operation,
-		     chan_nr, link_id);
+		LOGPFN(DL1C, LOGL_NOTICE, u32Fn, "unknown prim %d op %d chan_nr %d link_id %d\n",
+		     l1sap->oph.primitive, l1sap->oph.operation, chan_nr, link_id);
 		rc = -EINVAL;
 		goto done;
 	}
@@ -532,7 +529,7 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 
 		l1msg = l1p_msgb_alloc();
 		if (!l1msg) {
-			LOGP(DL1C, LOGL_FATAL, "L1SAP PH-DATA.req msg alloc failed\n");
+			LOGPFN(DL1C, LOGL_FATAL, u32Fn, "L1SAP PH-DATA.req msg alloc failed\n");
 			rc = -ENOMEM;
 			goto done;
 		}
@@ -594,7 +591,7 @@ static int ph_tch_req(struct gsm_bts_trx *trx, struct msgb *msg,
 	if (msg) {
 		nmsg = l1p_msgb_alloc();
 		if (!nmsg) {
-			LOGP(DL1C, LOGL_FATAL, "L1SAP PH-TCH.req msg alloc failed\n");
+			LOGPFN(DL1C, LOGL_FATAL, u32Fn, "L1SAP PH-TCH.req msg alloc failed\n");
 			return -ENOMEM;
 		}
 
@@ -663,7 +660,7 @@ static int mph_info_req(struct gsm_bts_trx *trx, struct msgb *msg,
 		if (l1sap->u.info.type == PRIM_INFO_ACTIVATE)
 			l1if_rsl_chan_act(lchan);
 		else if (l1sap->u.info.type == PRIM_INFO_MODIFY) {
-#warning "Mode Modify is currently not not supported for Octasic PHY"
+#pragma message ("Mode Modify is currently not supported for Octasic PHY (OS#3015)")
 			/* l1if_rsl_mode_modify(lchan); */
 		} else if (l1sap->u.info.u.act_req.sacch_only)
 			l1if_rsl_deact_sacch(lchan);
@@ -770,19 +767,21 @@ int bts_model_phy_link_open(struct phy_link *plink)
 
 int bts_model_init(struct gsm_bts *bts)
 {
-	struct gsm_bts_role_bts *btsb;
-
 	LOGP(DL1C, LOGL_NOTICE, "model_init()\n");
 
 	bts->variant = BTS_OSMO_OCTPHY;
-	btsb = bts_role_bts(bts);
-	btsb->support.ciphers = CIPHER_A5(1) | CIPHER_A5(2) | CIPHER_A5(3);
+	bts->support.ciphers = CIPHER_A5(1) | CIPHER_A5(2) | CIPHER_A5(3);
 
 	/* FIXME: what is the nominal transmit power of the PHY/board? */
 	bts->c0->nominal_power = 15;
 
 	gsm_bts_set_feature(bts, BTS_FEAT_GPRS);
 	gsm_bts_set_feature(bts, BTS_FEAT_OML_ALERTS);
+#if defined(cOCTVC1_GSM_LOGICAL_CHANNEL_COMBINATION_ENUM_FCCH_SCH_BCCH_CCCH_SDCCH4_CBCH_SACCHC4) && defined(cOCTVC1_GSM_LOGICAL_CHANNEL_COMBINATION_ENUM_SDCCH8_CBCH_SACCHC8)
+	gsm_bts_set_feature(bts, BTS_FEAT_CBCH);
+#endif
+	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_F_V1);
+	gsm_bts_set_feature(bts, BTS_FEAT_SPEECH_H_V1);
 
 	bts_model_vty_init(bts);
 
@@ -844,6 +843,21 @@ static uint32_t translate_tch_meas_rep_fn104_reverse(uint32_t fn)
 	return (fn - fn_mod) + new_fn_mod;
 }
 
+static unsigned int oct_meas2ber10k(const tOCTVC1_GSM_MEASUREMENT_INFO *m)
+{
+	if (m->usBERTotalBitCnt != 0) {
+		return (unsigned int)((m->usBERCnt * BER_10K) / m->usBERTotalBitCnt);
+	} else {
+		return 0;
+	}
+}
+
+static int oct_meas2rssi_dBm(const tOCTVC1_GSM_MEASUREMENT_INFO *m)
+{
+	/* rssi is in q8 format */
+	return (m->sRSSIDbm >> 8);
+}
+
 static void process_meas_res(struct gsm_bts_trx *trx, uint8_t chan_nr,
 			     uint32_t fn, uint32_t data_len,
 			     tOCTVC1_GSM_MEASUREMENT_INFO * m)
@@ -859,7 +873,7 @@ static void process_meas_res(struct gsm_bts_trx *trx, uint8_t chan_nr,
 	/* Update Timing offset for valid radio block */
 	if (data_len != 0) {
 		/* burst timing in 1x */
-		l1sap.u.info.u.meas_ind.ta_offs_qbits = m->sBurstTiming;
+		l1sap.u.info.u.meas_ind.ta_offs_256bits = m->sBurstTiming4x*64;
 	} else {
 		/* FIXME, In current implementation, OCTPHY would send DATA_IND
 		 * for all radio blocks (valid or invalid) But timing offset
@@ -867,19 +881,13 @@ static void process_meas_res(struct gsm_bts_trx *trx, uint8_t chan_nr,
 		 * counter to accumulate Timing offset.. even we add zero for
 		 * invalid block.. timing offset average calucation would not
 		 * correct. */
-		l1sap.u.info.u.meas_ind.ta_offs_qbits = 0;
+		l1sap.u.info.u.meas_ind.ta_offs_256bits = 0;
 	}
 
-	if (m->usBERTotalBitCnt != 0) {
-		l1sap.u.info.u.meas_ind.ber10k =
-		    (unsigned int)((m->usBERCnt * BER_10K) /
-				   m->usBERTotalBitCnt);
-	} else {
-		l1sap.u.info.u.meas_ind.ber10k = 0;
-	}
+	l1sap.u.info.u.meas_ind.ber10k = oct_meas2ber10k(m);
 
 	/* rssi is in q8 format */
-	l1sap.u.info.u.meas_ind.inv_rssi = (uint8_t) ((m->sRSSIDbm >> 8) * -1);
+	l1sap.u.info.u.meas_ind.inv_rssi = (uint8_t) oct_meas2rssi_dBm(m);
 
 	/* copy logical frame number to MEAS IND data structure */
 	l1sap.u.info.u.meas_ind.fn = translate_tch_meas_rep_fn104_reverse(fn);
@@ -946,8 +954,7 @@ static int handle_ph_readytosend_ind(struct octphy_hdl *fl1,
 
 	gsm_fn2gsmtime(&g_time, fn);
 
-	DEBUGP(DL1P, "Rx PH-RTS.ind %02u/%02u/%02u SAPI=%s\n",
-	       g_time.t1, g_time.t2, g_time.t3,
+	DEBUGPGT(DL1P, &g_time, "Rx PH-RTS.ind SAPI=%s\n",
 	       get_value_string(octphy_l1sapi_names, sapi));
 
 	/* in case we need to forward primitive to common part */
@@ -1028,14 +1035,14 @@ static int handle_ph_readytosend_ind(struct octphy_hdl *fl1,
 		/* send empty frame request */
 		rc = Logical_Channel_Empty_Frame_Cmd(empty_frame_req);
 		if (cOCTVC1_RC_OK != rc) {
-			LOGP(DL1P, LOGL_ERROR,
+			LOGPGT(DL1P, LOGL_ERROR, &g_time,
 			     "Sending Empty Frame Request Failed! (%s)\n",
 			     octvc1_rc2string(rc));
 		}
 		break;
 #endif
 	default:
-		LOGP(DL1P, LOGL_ERROR, "SAPI %s not handled via L1SAP!\n",
+		LOGPGT(DL1P, LOGL_ERROR, &g_time, "SAPI %s not handled via L1SAP!\n",
 			get_value_string(octphy_l1sapi_names, sapi));
 #if 0
 		data_req->Data.ulDataLength = GSM_MACBLOCK_LEN;
@@ -1059,9 +1066,8 @@ static int handle_ph_data_ind(struct octphy_hdl *fl1,
 	struct osmo_phsap_prim *l1sap;
 	uint32_t fn;
 	uint8_t *data;
-	uint16_t len, b_total, b_error;
+	uint16_t len;
 	int16_t snr;
-	int8_t rssi;
 	int rc;
 
 	uint8_t sapi = (uint8_t) data_ind->LchId.bySAPI;
@@ -1074,8 +1080,7 @@ static int handle_ph_data_ind(struct octphy_hdl *fl1,
 	/* chan_nr and link_id */
 	chan_nr = chan_nr_by_sapi(&trx->ts[ts_num], sapi, sc, ts_num, fn);
 	if (!chan_nr) {
-		LOGP(DL1C, LOGL_ERROR,
-		     "Rx PH-DATA.ind for unknown L1 SAPI %s\n",
+		LOGPFN(DL1C, LOGL_ERROR, fn, "Rx PH-DATA.ind for unknown L1 SAPI %s\n",
 		     get_value_string(octphy_l1sapi_names, sapi));
 		return ENOTSUP;
 	}
@@ -1091,11 +1096,10 @@ static int handle_ph_data_ind(struct octphy_hdl *fl1,
 	process_meas_res(trx, chan_nr, fn, data_ind->Data.ulDataLength,
 			 &data_ind->MeasurementInfo);
 
-	DEBUGP(DL1C, "Rx PH-DATA.ind %s: %s data_len:%d \n",
-	       get_value_string(octphy_l1sapi_names, sapi),
-	       osmo_hexdump(data_ind->Data.abyDataContent,
-			    data_ind->Data.ulDataLength),
-	       data_ind->Data.ulDataLength);
+	DEBUGPFN(DL1C, fn, "Rx PH-DATA.ind %s: %s data_len:%d \n",
+		 get_value_string(octphy_l1sapi_names, sapi),
+		 osmo_hexdump(data_ind->Data.abyDataContent, data_ind->Data.ulDataLength),
+		 data_ind->Data.ulDataLength);
 
 	/* check for TCH */
 	if (sapi == cOCTVC1_GSM_SAPI_ENUM_TCHF ||
@@ -1105,8 +1109,6 @@ static int handle_ph_data_ind(struct octphy_hdl *fl1,
 		return rc;
 	}
 
-	/* get rssi, rssi is in q8 format */
-	rssi = (int8_t) (data_ind->MeasurementInfo.sRSSIDbm >> 8);
 	/* get data pointer and length */
 	data = data_ind->Data.abyDataContent;
 	len = data_ind->Data.ulDataLength;
@@ -1136,13 +1138,11 @@ static int handle_ph_data_ind(struct octphy_hdl *fl1,
 	l1sap->u.data.fn = fn;
 #endif
 
-	l1sap->u.data.rssi = rssi;
-	b_total = data_ind->MeasurementInfo.usBERTotalBitCnt;
-	b_error =data_ind->MeasurementInfo.usBERCnt;
-	l1sap->u.data.ber10k = b_total ? BER_10K * b_error / b_total : 0;
+	l1sap->u.data.rssi = oct_meas2rssi_dBm(&data_ind->MeasurementInfo);
+	l1sap->u.data.ber10k = oct_meas2ber10k(&data_ind->MeasurementInfo);
 
-	/* FIXME::burst timing  in 1x but PCU is expecting 4X */
-	l1sap->u.data.ta_offs_qbits = (data_ind->MeasurementInfo.sBurstTiming * 4);
+	/* burst timing  in 1x but PCU is expecting 4X */
+	l1sap->u.data.ta_offs_256bits = data_ind->MeasurementInfo.sBurstTiming4x*64;
 	snr = data_ind->MeasurementInfo.sSNRDb;
 	/* FIXME: better converion formulae for SnR -> C / I?
 	l1sap->u.data.lqual_cb = (snr ? snr : (snr - 65536)) * 10 / 256;
@@ -1162,59 +1162,55 @@ static int handle_ph_rach_ind(struct octphy_hdl *fl1,
 		struct msgb *l1p_msg)
 {
 	struct gsm_bts_trx *trx = trx_by_l1h(fl1, ra_ind->TrxId.byTrxId);
-	struct gsm_bts *bts = trx->bts;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
-	struct gsm_lchan *lchan;
 	struct osmo_phsap_prim *l1sap;
-	uint32_t fn;
-	uint8_t ra, acc_delay;
 	int rc;
-
-	/* increment number of busy RACH slots, if required */
-	if (trx == bts->c0 &&
-	    ra_ind->MeasurementInfo.sRSSIDbm >= btsb->load.rach.busy_thresh) {
-		btsb->load.rach.busy++;
-	}
-	/* FIXME: Check min_qual_rach */
-
-	lchan = get_lchan_by_lchid(trx, &ra_ind->LchId);
-	if (trx == bts->c0 && !(lchan && lchan->ho.active == HANDOVER_ENABLED))
-		btsb->load.rach.access++;
+	struct ph_rach_ind_param rach_ind_param;
 
 	dump_meas_res(LOGL_DEBUG, &ra_ind->MeasurementInfo);
 
 	if (ra_ind->ulMsgLength != 1) {
-		LOGP(DL1C, LOGL_ERROR, "Rx PH-RACH.ind has lenghth %d > 1\n",
-		     ra_ind->ulMsgLength);
+		LOGPFN(DL1C, LOGL_ERROR, ra_ind->ulFrameNumber,
+			"Rx PH-RACH.ind has lenghth %d > 1\n", ra_ind->ulMsgLength);
 		msgb_free(l1p_msg);
 		return 0;
 	}
 
-	fn = ra_ind->ulFrameNumber;
-	ra = ra_ind->abyMsg[0];
+	/* We need to evaluate ra_ind before below msgb_trim(), since that invalidates *ra_ind. */
+	rach_ind_param = (struct ph_rach_ind_param) {
+		/* .chan_nr set below */
+		.ra = ra_ind->abyMsg[0],
+		/* .acc_delay set below */
+		.fn = ra_ind->ulFrameNumber,
+		.is_11bit = 0,
+		/* .burst_type remains unset */
+		.rssi = oct_meas2rssi_dBm(&ra_ind->MeasurementInfo),
+		.ber10k = oct_meas2ber10k(&ra_ind->MeasurementInfo),
+		.acc_delay_256bits = ra_ind->MeasurementInfo.sBurstTiming4x * 64,
+	};
+
+	if (ra_ind->LchId.bySubChannelNb == cOCTVC1_GSM_ID_SUB_CHANNEL_NB_ENUM_ALL &&
+	    ra_ind->LchId.bySAPI == cOCTVC1_GSM_SAPI_ENUM_RACH) {
+		rach_ind_param.chan_nr = 0x88;
+	} else {
+		struct gsm_lchan *lchan = get_lchan_by_lchid(trx, &ra_ind->LchId);
+		rach_ind_param.chan_nr = gsm_lchan2chan_nr(lchan);
+	}
+
 	/* check for under/overflow / sign */
 	if (ra_ind->MeasurementInfo.sBurstTiming < 0)
-		acc_delay = 0;
+		rach_ind_param.acc_delay = 0;
 	else
-		acc_delay = ra_ind->MeasurementInfo.sBurstTiming;
+		rach_ind_param.acc_delay = ra_ind->MeasurementInfo.sBurstTiming;
 
+	/* msgb_trim() invalidates ra_ind, make that abundantly clear: */
+	ra_ind = NULL;
 	rc = msgb_trim(l1p_msg, sizeof(*l1sap));
 	if (rc < 0)
 		MSGB_ABORT(l1p_msg, "No room for primitive\n");
 	l1sap = msgb_l1sap_prim(l1p_msg);
 	osmo_prim_init(&l1sap->oph, SAP_GSM_PH, PRIM_PH_RACH, PRIM_OP_INDICATION,
 			l1p_msg);
-	l1sap->u.rach_ind.ra = ra;
-	l1sap->u.rach_ind.acc_delay = acc_delay;
-	l1sap->u.rach_ind.fn = fn;
-	l1sap->u.rach_ind.is_11bit = 0;
-
-	if (!lchan || lchan->ts->pchan == GSM_PCHAN_CCCH ||
-	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4 ||
-	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH)
-		l1sap->u.rach_ind.chan_nr = 0x88;
-	else
-		l1sap->u.rach_ind.chan_nr = gsm_lchan2chan_nr(lchan);
+	l1sap->u.rach_ind = rach_ind_param;
 
 	l1sap_up(trx, l1sap);
 
