@@ -390,7 +390,7 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 		else
 			sapi = GsmL1_Sapi_Agch;
 	} else {
-		LOGP(DL1C, LOGL_NOTICE, "unknown prim %d op %d "
+		LOGPFN(DL1C, LOGL_NOTICE, u32Fn, "unknown prim %d op %d "
 			"chan_nr %d link_id %d\n", l1sap->oph.primitive,
 			l1sap->oph.operation, chan_nr, link_id);
 		msgb_free(l1msg);
@@ -453,7 +453,7 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 			memcpy(l1p->u.phDataReq.msgUnitParam.u8Buffer, msg->l2h,
 			       msgb_l2len(msg));
 		}
-		LOGP(DL1P, LOGL_DEBUG, "PH-DATA.req(%s)\n",
+		LOGPFN(DL1P, LOGL_DEBUG, u32Fn, "PH-DATA.req(%s)\n",
 		     osmo_hexdump(l1p->u.phDataReq.msgUnitParam.u8Buffer,
 					  l1p->u.phDataReq.msgUnitParam.u8Size));
 	} else {
@@ -465,7 +465,7 @@ static int ph_data_req(struct gsm_bts_trx *trx, struct msgb *msg,
 
 	/* send message to DSP's queue */
 	if (osmo_wqueue_enqueue(&fl1->write_q[MQ_L1_WRITE], l1msg) != 0) {
-		LOGP(DL1P, LOGL_ERROR, "MQ_L1_WRITE queue full. Dropping msg.\n");
+		LOGPFN(DL1P, LOGL_ERROR, u32Fn, "MQ_L1_WRITE queue full. Dropping msg.\n");
 		msgb_free(l1msg);
 	} else
 		dtx_int_signal(lchan);
@@ -834,8 +834,7 @@ static int handle_ph_readytosend_ind(struct femtol1_hdl *fl1,
 
 	gsm_fn2gsmtime(&g_time, rts_ind->u32Fn);
 
-	DEBUGP(DL1P, "Rx PH-RTS.ind %02u/%02u/%02u SAPI=%s\n",
-		g_time.t1, g_time.t2, g_time.t3,
+	DEBUGPGT(DL1P, &g_time, "Rx PH-RTS.ind SAPI=%s\n",
 		get_value_string(femtobts_l1sapi_names, rts_ind->sapi));
 
 	/* in all other cases, we need to allocate a new PH-DATA.ind
@@ -873,7 +872,7 @@ tx:
 
 	/* transmit */
 	if (osmo_wqueue_enqueue(&fl1->write_q[MQ_L1_WRITE], resp_msg) != 0) {
-		LOGP(DL1C, LOGL_ERROR, "MQ_L1_WRITE queue full. Dropping msg.\n");
+		LOGPGT(DL1C, LOGL_ERROR, &g_time, "MQ_L1_WRITE queue full. Dropping msg.\n");
 		msgb_free(resp_msg);
 	}
 
@@ -905,8 +904,8 @@ static int process_meas_res(struct gsm_bts_trx *trx, uint8_t chan_nr,
 		PRIM_OP_INDICATION, NULL);
 	l1sap.u.info.type = PRIM_INFO_MEAS;
 	l1sap.u.info.u.meas_ind.chan_nr = chan_nr;
-	l1sap.u.info.u.meas_ind.ta_offs_qbits = m->i16BurstTiming;
-	l1sap.u.info.u.meas_ind.ber10k = (unsigned int) (m->fBer * 100);
+	l1sap.u.info.u.meas_ind.ta_offs_256bits = m->i16BurstTiming * 64;
+	l1sap.u.info.u.meas_ind.ber10k = (unsigned int) (m->fBer * 10000);
 	l1sap.u.info.u.meas_ind.inv_rssi = (uint8_t) (m->fRssi * -1);
 	l1sap.u.info.u.meas_ind.fn = fn;
 
@@ -929,7 +928,7 @@ static int handle_ph_data_ind(struct femtol1_hdl *fl1, GsmL1_PhDataInd_t *data_i
 	chan_nr = chan_nr_by_sapi(&trx->ts[data_ind->u8Tn], data_ind->sapi,
 		data_ind->subCh, data_ind->u8Tn, data_ind->u32Fn);
 	if (!chan_nr) {
-		LOGP(DL1C, LOGL_ERROR, "PH-DATA-INDICATION for unknown sapi %s (%d)\n",
+		LOGPFN(DL1C, LOGL_ERROR, data_ind->u32Fn, "PH-DATA-INDICATION for unknown sapi %s (%d)\n",
 		     get_value_string(femtobts_l1sapi_names, data_ind->sapi), data_ind->sapi);
 		msgb_free(l1p_msg);
 		return ENOTSUP;
@@ -941,11 +940,9 @@ static int handle_ph_data_ind(struct femtol1_hdl *fl1, GsmL1_PhDataInd_t *data_i
 
 	gsm_fn2gsmtime(&g_time, fn);
 
-	DEBUGP(DL1P, "Rx PH-DATA.ind %s %s (hL2 %08x): %s\n",
-		get_value_string(femtobts_l1sapi_names, data_ind->sapi),
-		osmo_dump_gsmtime(&g_time), data_ind->hLayer2,
-		osmo_hexdump(data_ind->msgUnitParam.u8Buffer,
-			     data_ind->msgUnitParam.u8Size));
+	DEBUGPGT(DL1P, &g_time, "Rx PH-DATA.ind %s (hL2 %08x): %s\n",
+		get_value_string(femtobts_l1sapi_names, data_ind->sapi), data_ind->hLayer2,
+		osmo_hexdump(data_ind->msgUnitParam.u8Buffer, data_ind->msgUnitParam.u8Size));
 	dump_meas_res(LOGL_DEBUG, &data_ind->measParam);
 
 	/* check for TCH */
@@ -968,7 +965,7 @@ static int handle_ph_data_ind(struct femtol1_hdl *fl1, GsmL1_PhDataInd_t *data_i
 	l1sap->u.data.rssi = (int8_t) (data_ind->measParam.fRssi);
 	if (!pcu_direct) { /* FIXME: if pcu_direct=1, then this is not set, what to do in pcu_tx_data_ind() in this case ?*/
 		l1sap->u.data.ber10k = data_ind->measParam.fBer * 10000;
-		l1sap->u.data.ta_offs_qbits = data_ind->measParam.i16BurstTiming;
+		l1sap->u.data.ta_offs_256bits = data_ind->measParam.i16BurstTiming * 64;
 		l1sap->u.data.lqual_cb = data_ind->measParam.fLinkQuality * 10;
 	}
 	/* copy data from L1 primitive to L1SAP primitive */
@@ -987,92 +984,94 @@ static int handle_ph_ra_ind(struct femtol1_hdl *fl1, GsmL1_PhRaInd_t *ra_ind,
 {
 	struct gsm_bts_trx *trx = femtol1_hdl_trx(fl1);
 	struct gsm_bts *bts = trx->bts;
-	struct gsm_bts_role_bts *btsb = bts->role;
 	struct gsm_lchan *lchan;
 	struct osmo_phsap_prim *l1sap;
-	uint32_t fn;
-	uint8_t acc_delay = 0;
-	uint16_t ra = 0, is_11bit = 0, burst_type = 0, temp = 0;
 	int rc;
+	struct ph_rach_ind_param rach_ind_param;
 
-	/* increment number of busy RACH slots, if required */
-	if (trx == bts->c0 &&
-	    ra_ind->measParam.fRssi >= btsb->load.rach.busy_thresh)
-		btsb->load.rach.busy++;
-
-	if (ra_ind->measParam.fLinkQuality < btsb->min_qual_rach) {
+	/* FIXME: this should be deprecated/obsoleted as it bypasses rach.busy counting */
+	if (ra_ind->measParam.fLinkQuality < bts->min_qual_rach) {
 		msgb_free(l1p_msg);
 		return 0;
 	}
 
-	if (ra_ind->measParam.i16BurstTiming > 0)
-		acc_delay = ra_ind->measParam.i16BurstTiming >> 2;
-
-	/* increment number of RACH slots with valid non-handover RACH burst */
-	lchan = l1if_hLayer_to_lchan(trx, ra_ind->hLayer2);
-	if (trx == bts->c0 && !(lchan && lchan->ho.active == HANDOVER_ENABLED))
-		btsb->load.rach.access++;
-
 	dump_meas_res(LOGL_DEBUG, &ra_ind->measParam);
-	burst_type = ra_ind->burstType;
 
 	if ((ra_ind->msgUnitParam.u8Size != 1) &&
 		(ra_ind->msgUnitParam.u8Size != 2)) {
-		LOGP(DL1C, LOGL_ERROR, "PH-RACH-INDICATION has %d bits\n",
+		LOGPFN(DL1C, LOGL_ERROR, ra_ind->u32Fn, "PH-RACH-INDICATION has %d bits\n",
 			ra_ind->sapi);
 		msgb_free(l1p_msg);
 		return 0;
 	}
 
+	/* We need to evaluate ra_ind before below msgb_trim(), since that invalidates *ra_ind. */
+	rach_ind_param = (struct ph_rach_ind_param) {
+		/* .chan_nr set below */
+		/* .ra set below */
+		.acc_delay = 0,
+		.fn = ra_ind->u32Fn,
+		/* .is_11bit set below */
+		/* .burst_type set below */
+		.rssi = (int8_t) ra_ind->measParam.fRssi,
+		.ber10k = (unsigned int) (ra_ind->measParam.fBer * 10000.0),
+		.acc_delay_256bits = ra_ind->measParam.i16BurstTiming * 64,
+	};
+
+	lchan = l1if_hLayer_to_lchan(trx, ra_ind->hLayer2);
+	if (!lchan || lchan->ts->pchan == GSM_PCHAN_CCCH ||
+	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4 ||
+	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4_CBCH)
+		rach_ind_param.chan_nr = 0x88;
+	else
+		rach_ind_param.chan_nr = gsm_lchan2chan_nr(lchan);
+
 	if (ra_ind->msgUnitParam.u8Size == 2) {
-		is_11bit = 1;
-		ra = ra_ind->msgUnitParam.u8Buffer[0];
+		uint16_t temp;
+		uint16_t ra = ra_ind->msgUnitParam.u8Buffer[0];
 		ra = ra << 3;
 		temp = (ra_ind->msgUnitParam.u8Buffer[1] & 0x7);
 		ra = ra | temp;
+		rach_ind_param.is_11bit = 1;
+		rach_ind_param.ra = ra;
 	} else {
-		is_11bit = 0;
-		ra = ra_ind->msgUnitParam.u8Buffer[0];
+		rach_ind_param.is_11bit = 0;
+		rach_ind_param.ra = ra_ind->msgUnitParam.u8Buffer[0];
 	}
 
-	fn = ra_ind->u32Fn;
+	/* the old legacy full-bits acc_delay cannot express negative values */
+	if (ra_ind->measParam.i16BurstTiming > 0)
+		rach_ind_param.acc_delay = ra_ind->measParam.i16BurstTiming >> 2;
+
+	/* mapping of the burst type, the values are specific to osmo-bts-sysmo */
+	switch (ra_ind->burstType) {
+	case GsmL1_BurstType_Access_0:
+		rach_ind_param.burst_type =
+			GSM_L1_BURST_TYPE_ACCESS_0;
+		break;
+	case GsmL1_BurstType_Access_1:
+		rach_ind_param.burst_type =
+			GSM_L1_BURST_TYPE_ACCESS_1;
+		break;
+	case GsmL1_BurstType_Access_2:
+		rach_ind_param.burst_type =
+			GSM_L1_BURST_TYPE_ACCESS_2;
+		break;
+	default:
+		rach_ind_param.burst_type =
+			GSM_L1_BURST_TYPE_NONE;
+		break;
+	}
+
+	/* msgb_trim() invalidates ra_ind, make that abundantly clear: */
+	ra_ind = NULL;
 	rc = msgb_trim(l1p_msg, sizeof(*l1sap));
 	if (rc < 0)
 		MSGB_ABORT(l1p_msg, "No room for primitive data\n");
 	l1sap = msgb_l1sap_prim(l1p_msg);
 	osmo_prim_init(&l1sap->oph, SAP_GSM_PH, PRIM_PH_RACH, PRIM_OP_INDICATION,
 		l1p_msg);
-	l1sap->u.rach_ind.ra = ra;
-	l1sap->u.rach_ind.acc_delay = acc_delay;
-	l1sap->u.rach_ind.fn = fn;
-	l1sap->u.rach_ind.is_11bit = is_11bit;	/* no of bits in 11 bit RACH */
-
-	/*mapping of the burst type, the values are specific to osmo-bts-sysmo*/
-
-	switch (burst_type) {
-	case GsmL1_BurstType_Access_0:
-		l1sap->u.rach_ind.burst_type =
-			GSM_L1_BURST_TYPE_ACCESS_0;
-		break;
-	case GsmL1_BurstType_Access_1:
-		l1sap->u.rach_ind.burst_type =
-			GSM_L1_BURST_TYPE_ACCESS_1;
-		break;
-	case GsmL1_BurstType_Access_2:
-		l1sap->u.rach_ind.burst_type =
-			GSM_L1_BURST_TYPE_ACCESS_2;
-		break;
-	default:
-		l1sap->u.rach_ind.burst_type =
-			GSM_L1_BURST_TYPE_NONE;
-		break;
-	}
-
-	if (!lchan || lchan->ts->pchan == GSM_PCHAN_CCCH ||
-	    lchan->ts->pchan == GSM_PCHAN_CCCH_SDCCH4)
-		l1sap->u.rach_ind.chan_nr = 0x88;
-	else
-		l1sap->u.rach_ind.chan_nr = gsm_lchan2chan_nr(lchan);
+	l1sap->u.rach_ind = rach_ind_param;
 
 	return l1sap_up(trx, l1sap);
 }
@@ -1322,7 +1321,6 @@ int l1if_activate_rf(struct femtol1_hdl *hdl, int on)
 	return l1if_req_compl(hdl, msg, activate_rf_compl_cb, NULL);
 }
 
-#if SUPERFEMTO_API_VERSION >= SUPERFEMTO_API(3,6,0)
 static void mute_handle_ts(struct gsm_bts_trx_ts *ts, int is_muted)
 {
 	int i;
@@ -1350,6 +1348,7 @@ static void mute_handle_ts(struct gsm_bts_trx_ts *ts, int is_muted)
 	}
 }
 
+#if SUPERFEMTO_API_VERSION >= SUPERFEMTO_API(3,6,0)
 static int mute_rf_compl_cb(struct gsm_bts_trx *trx, struct msgb *resp,
 			    void *data)
 {
@@ -1388,8 +1387,11 @@ static int mute_rf_compl_cb(struct gsm_bts_trx *trx, struct msgb *resp,
 /* mute/unmute RF time slots */
 int l1if_mute_rf(struct femtol1_hdl *hdl, uint8_t mute[8], l1if_compl_cb *cb)
 {
+	const uint8_t unmuted[8] = { 0,0,0,0,0,0,0,0 };
 	struct msgb *msg = sysp_msgb_alloc();
 	SuperFemto_Prim_t *sysp = msgb_sysprim(msg);
+	struct gsm_bts_trx *trx = hdl->phy_inst->trx;
+	int i;
 
 	LOGP(DL1C, LOGL_INFO, "Tx RF-MUTE.req (%d, %d, %d, %d, %d, %d, %d, %d)\n",
 	     mute[0], mute[1], mute[2], mute[3],
@@ -1399,6 +1401,14 @@ int l1if_mute_rf(struct femtol1_hdl *hdl, uint8_t mute[8], l1if_compl_cb *cb)
 #if SUPERFEMTO_API_VERSION < SUPERFEMTO_API(3,6,0)
 	LOGP(DL1C, LOGL_ERROR, "RF-MUTE.req not supported by SuperFemto\n");
 	msgb_free(msg);
+	/* always acknowledge an un-MUTE (which is a no-op if MUTE is not supported */
+	if (!memcmp(mute, unmuted, ARRAY_SIZE(mute))) {
+		bts_update_status(BTS_STATUS_RF_MUTE, mute[0]);
+		oml_mo_rf_lock_chg(&trx->mo, mute, 1);
+		for (i = 0; i < ARRAY_SIZE(mute); ++i)
+			mute_handle_ts(&trx->ts[i], mute[i]);
+		return 0;
+	}
 	return -ENOTSUP;
 #else
 	sysp->id = SuperFemto_PrimId_MuteRfReq;
