@@ -135,7 +135,6 @@ int bts_model_lchan_deactivate_sacch(struct gsm_lchan *lchan)
  */
 int l1if_provision_transceiver_trx(struct trx_l1h *l1h)
 {
-	struct phy_link *plink = l1h->phy_inst->phy_link;
 	uint8_t tn;
 
 	if (!transceiver_available)
@@ -215,7 +214,6 @@ int l1if_provision_transceiver(struct gsm_bts *bts)
 
 	llist_for_each_entry(trx, &bts->trx_list, list) {
 		struct phy_instance *pinst = trx_phy_instance(trx);
-		struct phy_link *plink = pinst->phy_link;
 		struct trx_l1h *l1h = pinst->u.osmotrx.hdl;
 		l1h->config.arfcn_sent = 0;
 		l1h->config.tsc_sent = 0;
@@ -305,7 +303,6 @@ static uint8_t trx_set_bts(struct gsm_bts *bts, struct tlv_parsed *new_attr)
 {
 	struct gsm_bts_trx *trx;
 	uint8_t bsic = bts->bsic;
-	struct gsm_bts_role_bts *btsb = bts_role_bts(bts);
 
 	llist_for_each_entry(trx, &bts->trx_list, list) {
 		struct phy_instance *pinst = trx_phy_instance(trx);
@@ -327,7 +324,6 @@ static uint8_t trx_set_bts(struct gsm_bts *bts, struct tlv_parsed *new_attr)
 static uint8_t trx_set_trx(struct gsm_bts_trx *trx)
 {
 	struct phy_instance *pinst = trx_phy_instance(trx);
-	struct phy_link *plink = pinst->phy_link;
 	struct trx_l1h *l1h = pinst->u.osmotrx.hdl;
 	uint16_t arfcn = trx->arfcn;
 
@@ -491,7 +487,7 @@ int l1if_mph_time_ind(struct gsm_bts *bts, uint32_t fn)
 }
 
 
-void l1if_fill_meas_res(struct osmo_phsap_prim *l1sap, uint8_t chan_nr, float ta,
+static void l1if_fill_meas_res(struct osmo_phsap_prim *l1sap, uint8_t chan_nr, int16_t toa256,
 	float ber, float rssi, uint32_t fn)
 {
 	memset(l1sap, 0, sizeof(*l1sap));
@@ -499,26 +495,26 @@ void l1if_fill_meas_res(struct osmo_phsap_prim *l1sap, uint8_t chan_nr, float ta
 		PRIM_OP_INDICATION, NULL);
 	l1sap->u.info.type = PRIM_INFO_MEAS;
 	l1sap->u.info.u.meas_ind.chan_nr = chan_nr;
-	l1sap->u.info.u.meas_ind.ta_offs_qbits = (int16_t)(ta*4);
+	l1sap->u.info.u.meas_ind.ta_offs_256bits = toa256;
 	l1sap->u.info.u.meas_ind.ber10k = (unsigned int) (ber * 10000);
 	l1sap->u.info.u.meas_ind.inv_rssi = (uint8_t) (rssi * -1);
 	l1sap->u.info.u.meas_ind.fn = fn;
 }
 
 int l1if_process_meas_res(struct gsm_bts_trx *trx, uint8_t tn, uint32_t fn, uint8_t chan_nr,
-	int n_errors, int n_bits_total, float rssi, float toa)
+	int n_errors, int n_bits_total, float rssi, int16_t toa256)
 {
 	struct gsm_lchan *lchan = &trx->ts[tn].lchan[l1sap_chan2ss(chan_nr)];
 	struct osmo_phsap_prim l1sap;
 	/* 100% BER is n_bits_total is 0 */
 	float ber = n_bits_total==0 ? 1.0 : (float)n_errors / (float)n_bits_total;
 
-	LOGP(DMEAS, LOGL_DEBUG, "RX L1 frame %s fn=%u chan_nr=0x%02x MS pwr=%ddBm rssi=%.1f dBFS "
-		"ber=%.2f%% (%d/%d bits) L1_ta=%d rqd_ta=%d toa=%.2f\n",
+	LOGPFN(DMEAS, LOGL_DEBUG, fn, "RX L1 frame %s fn=%u chan_nr=0x%02x MS pwr=%ddBm rssi=%.1f dBFS "
+		"ber=%.2f%% (%d/%d bits) L1_ta=%d rqd_ta=%d toa256=%d\n",
 		gsm_lchan_name(lchan), fn, chan_nr, ms_pwr_dbm(lchan->ts->trx->bts->band, lchan->ms_power_ctrl.current),
-		rssi, ber*100, n_errors, n_bits_total, lchan->meas.l1_info[1], lchan->rqd_ta, toa);
+		rssi, ber*100, n_errors, n_bits_total, lchan->meas.l1_info[1], lchan->rqd_ta, toa256);
 
-	l1if_fill_meas_res(&l1sap, chan_nr, lchan->rqd_ta + toa, ber, rssi, fn);
+	l1if_fill_meas_res(&l1sap, chan_nr, toa256, ber, rssi, fn);
 
 	return l1sap_up(trx, &l1sap);
 }
