@@ -31,6 +31,7 @@
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/talloc.h>
 #include <osmocom/codec/codec.h>
+#include <osmocom/codec/ecu.h>
 #include <osmocom/core/bits.h>
 #include <osmocom/gsm/a5.h>
 #include <osmocom/coding/gsm0503_coding.h>
@@ -38,6 +39,7 @@
 #include <osmo-bts/gsm_data.h>
 #include <osmo-bts/logging.h>
 #include <osmo-bts/rsl.h>
+#include <osmo-bts/bts.h>
 #include <osmo-bts/l1sap.h>
 #include <osmo-bts/msg_utils.h>
 #include <osmo-bts/scheduler.h>
@@ -49,11 +51,6 @@
 
 extern void *tall_bts_ctx;
 
-/* Enable this to multiply TOA of RACH by 10.
- * This is useful to check tenth of timing advances with RSSI test tool.
- * Note that regular phones will not work when using this test! */
-//#define TA_TEST
-
 /* Maximum size of a EGPRS message in bytes */
 #define EGPRS_0503_MAX_BYTES		155
 
@@ -75,8 +72,7 @@ static inline uint16_t compute_ber10k(int n_bits_total, int n_errors)
 ubit_t *tx_idle_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	enum trx_chan_type chan, uint8_t bid, uint16_t *nbits)
 {
-	LOGP(DL1C, LOGL_DEBUG, "Transmitting %s fn=%u ts=%u trx=%u\n",
-		trx_chan_desc[chan].name, fn, tn, l1t->trx->nr);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Transmitting IDLE\n");
 
 	if (nbits)
 		*nbits = GSM_BURST_LEN;
@@ -88,8 +84,7 @@ ubit_t *tx_idle_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 ubit_t *tx_fcch_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	enum trx_chan_type chan, uint8_t bid, uint16_t *nbits)
 {
-	LOGP(DL1C, LOGL_DEBUG, "Transmitting %s fn=%u ts=%u trx=%u\n",
-		trx_chan_desc[chan].name, fn, tn, l1t->trx->nr);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Transmitting FCCH\n");
 
 	if (nbits)
 		*nbits = GSM_BURST_LEN;
@@ -108,8 +103,7 @@ ubit_t *tx_sch_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	struct	gsm_time t;
 	uint8_t t3p, bsic;
 
-	LOGP(DL1C, LOGL_DEBUG, "Transmitting %s fn=%u ts=%u trx=%u\n",
-		trx_chan_desc[chan].name, fn, tn, l1t->trx->nr);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Transmitting SCH\n");
 
 	/* BURST BYPASS */
 
@@ -173,9 +167,7 @@ ubit_t *tx_data_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	if (msg)
 		goto got_msg;
 
-	LOGP(DL1C, LOGL_INFO, "%s has not been served !! No prim for "
-		"trx=%u ts=%u at fn=%u to transmit.\n",
-		trx_chan_desc[chan].name, l1t->trx->nr, tn, fn);
+	LOGL1S(DL1P, LOGL_INFO, l1t, tn, chan, fn, "No prim for transmit.\n");
 
 no_msg:
 	/* free burst memory */
@@ -188,7 +180,7 @@ no_msg:
 got_msg:
 	/* check validity of message */
 	if (msgb_l2len(msg) != GSM_MACBLOCK_LEN) {
-		LOGP(DL1C, LOGL_FATAL, "Prim not 23 bytes, please FIX! "
+		LOGL1S(DL1P, LOGL_FATAL, l1t, tn, chan, fn, "Prim not 23 bytes, please FIX! "
 			"(len=%d)\n", msgb_l2len(msg));
 		/* free message */
 		msgb_free(msg);
@@ -239,8 +231,7 @@ send_burst:
 	if (nbits)
 		*nbits = GSM_BURST_LEN;
 
-	LOGP(DL1C, LOGL_DEBUG, "Transmitting %s fn=%u ts=%u trx=%u burst=%u\n",
-		trx_chan_desc[chan].name, fn, tn, l1t->trx->nr, bid);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Transmitting burst=%u.\n", bid);
 
 	return bits;
 }
@@ -269,9 +260,7 @@ ubit_t *tx_pdtch_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	if (msg)
 		goto got_msg;
 
-	LOGP(DL1C, LOGL_INFO, "%s has not been served !! No prim for "
-		"trx=%u ts=%u at fn=%u to transmit.\n",
-		trx_chan_desc[chan].name, l1t->trx->nr, tn, fn);
+	LOGL1S(DL1P, LOGL_INFO, l1t, tn, chan, fn, "No prim for transmit.\n");
 
 no_msg:
 	/* free burst memory */
@@ -299,7 +288,7 @@ got_msg:
 
 	/* check validity of message */
 	if (rc < 0) {
-		LOGP(DL1C, LOGL_FATAL, "Prim invalid length, please FIX! "
+		LOGL1S(DL1P, LOGL_FATAL, l1t, tn, chan, fn, "Prim invalid length, please FIX! "
 			"(len=%ld)\n", msg->tail - msg->l2h);
 		/* free message */
 		msgb_free(msg);
@@ -337,8 +326,7 @@ send_burst:
 			*nbits = GSM_BURST_LEN;
 	}
 
-	LOGP(DL1C, LOGL_DEBUG, "Transmitting %s fn=%u ts=%u trx=%u burst=%u\n",
-		trx_chan_desc[chan].name, fn, tn, l1t->trx->nr, bid);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Transmitting burst=%u.\n", bid);
 
 	return bits;
 }
@@ -367,8 +355,8 @@ static void tx_tch_common(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 		uint8_t tch_data[GSM_FR_BYTES];
 		int len;
 
-		LOGP(DL1C, LOGL_NOTICE, "Missing TCH bursts detected, sending "
-			"BFI for %s\n", trx_chan_desc[chan].name);
+		LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn,
+			"Missing TCH bursts detected, sending BFI\n");
 
 		/* indicate bad frame */
 		switch (tch_mode) {
@@ -399,8 +387,7 @@ static void tx_tch_common(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 			break;
 		default:
 inval_mode1:
-			LOGP(DL1C, LOGL_ERROR, "TCH mode invalid, please "
-				"fix!\n");
+			LOGL1S(DL1P, LOGL_ERROR, l1t, tn, chan, fn, "TCH mode invalid, please fix!\n");
 			len = 0;
 		}
 		if (len)
@@ -417,8 +404,8 @@ inval_mode1:
 			if (msg2) {
 				l1sap = msgb_l1sap_prim(msg2);
 				if (l1sap->oph.primitive == PRIM_TCH) {
-					LOGP(DL1C, LOGL_FATAL, "TCH twice, "
-						"please FIX! ");
+					LOGL1S(DL1P, LOGL_FATAL, l1t, tn, chan, fn,
+						"TCH twice, please FIX!\n");
 					msgb_free(msg2);
 				} else
 					msg_facch = msg2;
@@ -428,8 +415,8 @@ inval_mode1:
 			if (msg2) {
 				l1sap = msgb_l1sap_prim(msg2);
 				if (l1sap->oph.primitive != PRIM_TCH) {
-					LOGP(DL1C, LOGL_FATAL, "FACCH twice, "
-						"please FIX! ");
+					LOGL1S(DL1P, LOGL_FATAL, l1t, tn, chan, fn,
+						"FACCH twice, please FIX!\n");
 					msgb_free(msg2);
 				} else
 					msg_tch = msg2;
@@ -445,7 +432,7 @@ inval_mode1:
 
 	/* check validity of message */
 	if (msg_facch && msgb_l2len(msg_facch) != GSM_MACBLOCK_LEN) {
-		LOGP(DL1C, LOGL_FATAL, "Prim not 23 bytes, please FIX! "
+		LOGL1S(DL1P, LOGL_FATAL, l1t, tn, chan, fn, "Prim not 23 bytes, please FIX! "
 			"(len=%d)\n", msgb_l2len(msg_facch));
 		/* free message */
 		msgb_free(msg_facch);
@@ -462,10 +449,8 @@ inval_mode1:
 		int8_t sti, cmi;
 
 		if (rsl_cmode != RSL_CMOD_SPD_SPEECH) {
-			LOGP(DL1C, LOGL_NOTICE, "%s Dropping speech frame, "
-				"because we are not in speech mode trx=%u "
-				"ts=%u at fn=%u.\n", trx_chan_desc[chan].name,
-				l1t->trx->nr, tn, fn);
+			LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn, "Dropping speech frame, "
+				"because we are not in speech mode\n");
 			goto free_bad_msg;
 		}
 
@@ -502,43 +487,33 @@ inval_mode1:
 				trx_loop_amr_set(chan_state, 1);
 			}
 			if (ft < 0) {
-				LOGP(DL1C, LOGL_ERROR, "%s Codec (FT = %d) "
-					" of RTP frame not in list. "
-					"trx=%u ts=%u\n",
-					trx_chan_desc[chan].name, ft_codec,
-					l1t->trx->nr, tn);
+				LOGL1S(DL1P, LOGL_ERROR, l1t, tn, chan, fn,
+					"Codec (FT = %d) of RTP frame not in list\n", ft_codec);
 				goto free_bad_msg;
 			}
 			if (fn_is_codec_mode_request(fn) && chan_state->dl_ft != ft) {
-				LOGP(DL1C, LOGL_NOTICE, "%s Codec (FT = %d) "
-					" of RTP cannot be changed now, but in "
-					"next frame. trx=%u ts=%u\n",
-					trx_chan_desc[chan].name, ft_codec,
-					l1t->trx->nr, tn);
+				LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn, "Codec (FT = %d) "
+					" of RTP cannot be changed now, but in next frame\n", ft_codec);
 				goto free_bad_msg;
 			}
 			chan_state->dl_ft = ft;
 			if (bfi == AMR_BAD) {
-				LOGP(DL1C, LOGL_NOTICE, "%s Transmitting 'bad "
-					"AMR frame' trx=%u ts=%u at fn=%u.\n",
-					trx_chan_desc[chan].name,
-					l1t->trx->nr, tn, fn);
+				LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn,
+					"Transmitting 'bad AMR frame'\n");
 				goto free_bad_msg;
 			}
 			break;
 		default:
 inval_mode2:
-			LOGP(DL1C, LOGL_ERROR, "TCH mode invalid, please "
-				"fix!\n");
+			LOGL1S(DL1P, LOGL_ERROR, l1t, tn, chan, fn, "TCH mode invalid, please fix!\n");
 			goto free_bad_msg;
 		}
 		if (len < 0) {
-			LOGP(DL1C, LOGL_ERROR, "Cannot send invalid AMR "
-				"payload\n");
+			LOGL1S(DL1P, LOGL_ERROR, l1t, tn, chan, fn, "Cannot send invalid AMR payload\n");
 			goto free_bad_msg;
 		}
 		if (msgb_l2len(msg_tch) != len) {
-			LOGP(DL1C, LOGL_ERROR, "Cannot send payload with "
+			LOGL1S(DL1P, LOGL_ERROR, l1t, tn, chan, fn, "Cannot send payload with "
 				"invalid length! (expecting %d, received %d)\n",
 				len, msgb_l2len(msg_tch));
 free_bad_msg:
@@ -590,9 +565,7 @@ ubit_t *tx_tchf_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 
 	/* no message at all */
 	if (!msg_tch && !msg_facch) {
-		LOGP(DL1C, LOGL_INFO, "%s has not been served !! No prim for "
-			"trx=%u ts=%u at fn=%u to transmit.\n",
-			trx_chan_desc[chan].name, l1t->trx->nr, tn, fn);
+		LOGL1S(DL1P, LOGL_INFO, l1t, tn, chan, fn, "No TCH or FACCH prim for transmit.\n");
 		goto send_burst;
 	}
 
@@ -630,8 +603,7 @@ send_burst:
 	if (nbits)
 		*nbits = GSM_BURST_LEN;
 
-	LOGP(DL1C, LOGL_DEBUG, "Transmitting %s fn=%u ts=%u trx=%u burst=%u\n",
-		trx_chan_desc[chan].name, fn, tn, l1t->trx->nr, bid);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Transmitting burst=%u.\n", bid);
 
 	return bits;
 }
@@ -660,9 +632,8 @@ ubit_t *tx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 
 	/* check for FACCH alignment */
 	if (msg_facch && ((((fn + 4) % 26) >> 2) & 1)) {
-		LOGP(DL1C, LOGL_ERROR, "%s Cannot transmit FACCH starting on "
-			"even frames, please fix RTS!\n",
-			trx_chan_desc[chan].name);
+		LOGL1S(DL1P, LOGL_ERROR, l1t, tn, chan, fn, "Cannot transmit FACCH starting on "
+			"even frames, please fix RTS!\n");
 		msgb_free(msg_facch);
 		msg_facch = NULL;
 	}
@@ -687,9 +658,7 @@ ubit_t *tx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 
 	/* no message at all */
 	if (!msg_tch && !msg_facch && !chan_state->dl_ongoing_facch) {
-		LOGP(DL1C, LOGL_INFO, "%s has not been served !! No prim for "
-			"trx=%u ts=%u at fn=%u to transmit.\n",
-			trx_chan_desc[chan].name, l1t->trx->nr, tn, fn);
+		LOGL1S(DL1P, LOGL_INFO, l1t, tn, chan, fn, "No TCH or FACCH prim for transmit.\n");
 		goto send_burst;
 	}
 
@@ -729,8 +698,7 @@ send_burst:
 	if (nbits)
 		*nbits = GSM_BURST_LEN;
 
-	LOGP(DL1C, LOGL_DEBUG, "Transmitting %s fn=%u ts=%u trx=%u burst=%u\n",
-		trx_chan_desc[chan].name, fn, tn, l1t->trx->nr, bid);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Transmitting burst=%u.\n", bid);
 
 	return bits;
 }
@@ -742,23 +710,22 @@ send_burst:
 
 int rx_rach_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, float toa)
+	int8_t rssi, int16_t toa256)
 {
 	uint8_t chan_nr;
 	struct osmo_phsap_prim l1sap;
+	int n_errors, n_bits_total;
 	uint8_t ra;
 	int rc;
 
 	chan_nr = trx_chan_desc[chan].chan_nr | tn;
 
-	LOGP(DL1C, LOGL_DEBUG, "Received Access Burst on %s fn=%u toa=%.2f\n",
-		trx_chan_desc[chan].name, fn, toa);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Received RACH toa=%d\n", toa256);
 
 	/* decode */
-	rc = gsm0503_rach_decode(&ra, bits + 8 + 41, l1t->trx->bts->bsic);
+	rc = gsm0503_rach_decode_ber(&ra, bits + 8 + 41, l1t->trx->bts->bsic, &n_errors, &n_bits_total);
 	if (rc) {
-		LOGP(DL1C, LOGL_DEBUG, "Received bad AB frame at fn=%u "
-			"(%u/51)\n", fn, fn % 51);
+		LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Received bad AB frame\n");
 		return 0;
 	}
 
@@ -769,16 +736,15 @@ int rx_rach_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 		NULL);
 	l1sap.u.rach_ind.chan_nr = chan_nr;
 	l1sap.u.rach_ind.ra = ra;
-#ifdef TA_TEST
-#warning TIMING ADVANCE TEST-HACK IS ENABLED!!!
-	toa *= 10;
-#endif
-	l1sap.u.rach_ind.acc_delay = (toa >= 0) ? toa : 0;
+	l1sap.u.rach_ind.acc_delay = (toa256 >= 0) ? toa256/256 : 0;
 	l1sap.u.rach_ind.fn = fn;
 
 	/* 11bit RACH is not supported for osmo-trx */
 	l1sap.u.rach_ind.is_11bit = 0;
 	l1sap.u.rach_ind.burst_type = GSM_L1_BURST_TYPE_ACCESS_0;
+	l1sap.u.rach_ind.rssi = rssi;
+	l1sap.u.rach_ind.ber10k = compute_ber10k(n_bits_total, n_errors);
+	l1sap.u.rach_ind.acc_delay_256bits = toa256;
 
 	/* forward primitive */
 	l1sap_up(l1t->trx, &l1sap);
@@ -789,7 +755,7 @@ int rx_rach_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 /*! \brief a single (SDCCH/SACCH) burst was received by the PHY, process it */
 int rx_data_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, float toa)
+	int8_t rssi, int16_t toa256)
 {
 	struct l1sched_ts *l1ts = l1sched_trx_get_ts(l1t, tn);
 	struct l1sched_chan_state *chan_state = &l1ts->chan_state[chan];
@@ -798,7 +764,7 @@ int rx_data_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	uint8_t *mask = &chan_state->ul_mask;
 	float *rssi_sum = &chan_state->rssi_sum;
 	uint8_t *rssi_num = &chan_state->rssi_num;
-	float *toa_sum = &chan_state->toa_sum;
+	int32_t *toa256_sum = &chan_state->toa256_sum;
 	uint8_t *toa_num = &chan_state->toa_num;
 	uint8_t l2[GSM_MACBLOCK_LEN], l2_len;
 	int n_errors, n_bits_total;
@@ -807,10 +773,9 @@ int rx_data_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 
 	/* handle RACH, if handover RACH detection is turned on */
 	if (chan_state->ho_rach_detect == 1)
-		return rx_rach_fn(l1t, tn, fn, chan, bid, bits, GSM_BURST_LEN, rssi, toa);
+		return rx_rach_fn(l1t, tn, fn, chan, bid, bits, GSM_BURST_LEN, rssi, toa256);
 
-	LOGP(DL1C, LOGL_DEBUG, "Data received %s fn=%u ts=%u trx=%u bid=%u\n",
-		trx_chan_desc[chan].name, fn, tn, l1t->trx->nr, bid);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Received Data, bid=%u\n", bid);
 
 	/* allocate burst memory, if not already */
 	if (!*bursts_p) {
@@ -826,7 +791,7 @@ int rx_data_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 		*first_fn = fn;
 		*rssi_sum = 0;
 		*rssi_num = 0;
-		*toa_sum = 0;
+		*toa256_sum = 0;
 		*toa_num = 0;
 	}
 
@@ -834,7 +799,7 @@ int rx_data_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	*mask |= (1 << bid);
 	*rssi_sum += rssi;
 	(*rssi_num)++;
-	*toa_sum += toa;
+	*toa256_sum += toa256;
 	(*toa_num)++;
 
 	/* copy burst to buffer of 4 bursts */
@@ -845,7 +810,7 @@ int rx_data_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	/* send burst information to loops process */
 	if (L1SAP_IS_LINK_SACCH(trx_chan_desc[chan].link_id)) {
 		trx_loop_sacch_input(l1t, trx_chan_desc[chan].chan_nr | tn,
-			chan_state, rssi, toa);
+			chan_state, rssi, toa256);
 	}
 
 	/* wait until complete set of bursts */
@@ -854,10 +819,8 @@ int rx_data_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 
 	/* check for complete set of bursts */
 	if ((*mask & 0xf) != 0xf) {
-		LOGP(DL1C, LOGL_NOTICE, "Received incomplete data frame at "
-			"fn=%u (%u/%u) for %s\n", *first_fn,
-			(*first_fn) % l1ts->mf_period, l1ts->mf_period,
-			trx_chan_desc[chan].name);
+		LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn, "Received incomplete data (%u/%u)\n",
+			*first_fn, (*first_fn) % l1ts->mf_period);
 
 		/* we require first burst to have correct FN */
 		if (!(*mask & 0x1)) {
@@ -870,44 +833,42 @@ int rx_data_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	/* decode */
 	rc = gsm0503_xcch_decode(l2, *bursts_p, &n_errors, &n_bits_total);
 	if (rc) {
-		LOGP(DL1C, LOGL_DEBUG, "Received bad data frame at fn=%u "
-			"(%u/%u) for %s\n", *first_fn,
-			(*first_fn) % l1ts->mf_period, l1ts->mf_period,
-			trx_chan_desc[chan].name);
+		LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn, "Received bad data (%u/%u)\n",
+			*first_fn, (*first_fn) % l1ts->mf_period);
 		l2_len = 0;
 	} else
 		l2_len = GSM_MACBLOCK_LEN;
 
 	/* Send uplink measurement information to L2 */
-	l1if_process_meas_res(l1t->trx, tn, fn, trx_chan_desc[chan].chan_nr | tn,
-		n_errors, n_bits_total, *rssi_sum / *rssi_num, *toa_sum / *toa_num);
+	l1if_process_meas_res(l1t->trx, tn, *first_fn, trx_chan_desc[chan].chan_nr | tn,
+		n_errors, n_bits_total, *rssi_sum / *rssi_num, *toa256_sum / *toa_num);
 	ber10k = compute_ber10k(n_bits_total, n_errors);
 	return _sched_compose_ph_data_ind(l1t, tn, *first_fn, chan, l2, l2_len,
 					  *rssi_sum / *rssi_num,
-					  4 * (*toa_sum) / *toa_num, 0, ber10k,
+					  4 * (*toa256_sum) / *toa_num, 0, ber10k,
 					  PRES_INFO_UNKNOWN);
 }
 
 /*! \brief a single PDTCH burst was received by the PHY, process it */
 int rx_pdtch_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, float toa)
+	int8_t rssi, int16_t toa256)
 {
 	struct l1sched_ts *l1ts = l1sched_trx_get_ts(l1t, tn);
 	struct l1sched_chan_state *chan_state = &l1ts->chan_state[chan];
 	sbit_t *burst, **bursts_p = &chan_state->ul_bursts;
+	uint32_t *first_fn = &chan_state->ul_first_fn;
 	uint8_t *mask = &chan_state->ul_mask;
 	float *rssi_sum = &chan_state->rssi_sum;
 	uint8_t *rssi_num = &chan_state->rssi_num;
-	float *toa_sum = &chan_state->toa_sum;
+	int32_t *toa256_sum = &chan_state->toa256_sum;
 	uint8_t *toa_num = &chan_state->toa_num;
 	uint8_t l2[EGPRS_0503_MAX_BYTES];
 	int n_errors, n_bursts_bits, n_bits_total;
 	uint16_t ber10k;
 	int rc;
 
-	LOGP(DL1C, LOGL_DEBUG, "PDTCH received %s fn=%u ts=%u trx=%u bid=%u\n",
-		trx_chan_desc[chan].name, fn, tn, l1t->trx->nr, bid);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Received PDTCH bid=%u\n", bid);
 
 	/* allocate burst memory, if not already */
 	if (!*bursts_p) {
@@ -921,9 +882,10 @@ int rx_pdtch_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	if (bid == 0) {
 		memset(*bursts_p, 0, GSM0503_EGPRS_BURSTS_NBITS);
 		*mask = 0x0;
+		*first_fn = fn;
 		*rssi_sum = 0;
 		*rssi_num = 0;
-		*toa_sum = 0;
+		*toa256_sum = 0;
 		*toa_num = 0;
 	}
 
@@ -931,7 +893,7 @@ int rx_pdtch_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	*mask |= (1 << bid);
 	*rssi_sum += rssi;
 	(*rssi_num)++;
-	*toa_sum += toa;
+	*toa256_sum += toa256;
 	(*toa_num)++;
 
 	/* copy burst to buffer of 4 bursts */
@@ -953,10 +915,8 @@ int rx_pdtch_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 
 	/* check for complete set of bursts */
 	if ((*mask & 0xf) != 0xf) {
-		LOGP(DL1C, LOGL_DEBUG, "Received incomplete PDTCH block "
-			"ending at fn=%u (%u/%u) for %s\n", fn,
-			fn % l1ts->mf_period, l1ts->mf_period,
-			trx_chan_desc[chan].name);
+		LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Received incomplete frame (%u/%u)\n",
+			fn % l1ts->mf_period, l1ts->mf_period);
 	}
 	*mask = 0x0;
 
@@ -976,44 +936,44 @@ int rx_pdtch_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 
 
 	/* Send uplink measurement information to L2 */
-	l1if_process_meas_res(l1t->trx, tn, fn, trx_chan_desc[chan].chan_nr | tn,
-		n_errors, n_bits_total, *rssi_sum / *rssi_num, *toa_sum / *toa_num);
+	l1if_process_meas_res(l1t->trx, tn, *first_fn, trx_chan_desc[chan].chan_nr | tn,
+		n_errors, n_bits_total, *rssi_sum / *rssi_num, *toa256_sum / *toa_num);
 
 	if (rc <= 0) {
-		LOGP(DL1C, LOGL_DEBUG, "Received bad PDTCH block ending at "
-			"fn=%u (%u/%u) for %s\n", fn, fn % l1ts->mf_period,
-			l1ts->mf_period, trx_chan_desc[chan].name);
+		LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Received bad PDTCH (%u/%u)\n",
+			fn % l1ts->mf_period, l1ts->mf_period);
 		return 0;
 	}
 	ber10k = compute_ber10k(n_bits_total, n_errors);
 	return _sched_compose_ph_data_ind(l1t, tn, (fn + GSM_HYPERFRAME - 3) % GSM_HYPERFRAME, chan,
-		l2, rc, *rssi_sum / *rssi_num, 4 * (*toa_sum) / *toa_num, 0,
+		l2, rc, *rssi_sum / *rssi_num, 4 * (*toa256_sum) / *toa_num, 0,
 					  ber10k, PRES_INFO_BOTH);
 }
 
 /*! \brief a single TCH/F burst was received by the PHY, process it */
 int rx_tchf_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, float toa)
+	int8_t rssi, int16_t toa256)
 {
 	struct l1sched_ts *l1ts = l1sched_trx_get_ts(l1t, tn);
 	struct l1sched_chan_state *chan_state = &l1ts->chan_state[chan];
 	sbit_t *burst, **bursts_p = &chan_state->ul_bursts;
+	uint32_t *first_fn = &chan_state->ul_first_fn;
 	uint8_t *mask = &chan_state->ul_mask;
 	uint8_t rsl_cmode = chan_state->rsl_cmode;
 	uint8_t tch_mode = chan_state->tch_mode;
 	uint8_t tch_data[128]; /* just to be safe */
 	int rc, amr = 0;
 	int n_errors, n_bits_total;
+	bool bfi_flag = false;
 	struct gsm_lchan *lchan =
 		get_lchan_by_chan_nr(l1t->trx, trx_chan_desc[chan].chan_nr | tn);
 
 	/* handle rach, if handover rach detection is turned on */
 	if (chan_state->ho_rach_detect == 1)
-		return rx_rach_fn(l1t, tn, fn, chan, bid, bits, GSM_BURST_LEN, rssi, toa);
+		return rx_rach_fn(l1t, tn, fn, chan, bid, bits, GSM_BURST_LEN, rssi, toa256);
 
-	LOGP(DL1C, LOGL_DEBUG, "TCH/F received %s fn=%u ts=%u trx=%u bid=%u\n",
-		trx_chan_desc[chan].name, fn, tn, l1t->trx->nr, bid);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Received TCH/F, bid=%u\n", bid);
 
 	/* allocate burst memory, if not already */
 	if (!*bursts_p) {
@@ -1026,6 +986,7 @@ int rx_tchf_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	if (bid == 0) {
 		memset(*bursts_p + 464, 0, 464);
 		*mask = 0x0;
+		*first_fn = fn;
 	}
 
 	/* update mask */
@@ -1042,10 +1003,8 @@ int rx_tchf_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 
 	/* check for complete set of bursts */
 	if ((*mask & 0xf) != 0xf) {
-		LOGP(DL1C, LOGL_NOTICE, "Received incomplete TCH frame ending "
-			"at fn=%u (%u/%u) for %s\n", fn,
-			fn % l1ts->mf_period, l1ts->mf_period,
-			trx_chan_desc[chan].name);
+		LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn, "Received incomplete frame (%u/%u)\n",
+			fn % l1ts->mf_period, l1ts->mf_period);
 	}
 	*mask = 0x0;
 
@@ -1055,7 +1014,8 @@ int rx_tchf_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 								: tch_mode) {
 	case GSM48_CMODE_SPEECH_V1: /* FR */
 		rc = gsm0503_tch_fr_decode(tch_data, *bursts_p, 1, 0, &n_errors, &n_bits_total);
-		lchan_set_marker(osmo_fr_check_sid(tch_data, rc), lchan); /* DTXu */
+		if (rc >= 0)
+			lchan_set_marker(osmo_fr_check_sid(tch_data, rc), lchan); /* DTXu */
 		break;
 	case GSM48_CMODE_SPEECH_EFR: /* EFR */
 		rc = gsm0503_tch_fr_decode(tch_data, *bursts_p, 1, 1, &n_errors, &n_bits_total);
@@ -1082,26 +1042,27 @@ int rx_tchf_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 		}
 		break;
 	default:
-		LOGP(DL1C, LOGL_ERROR, "TCH mode %u invalid, please fix!\n",
+		LOGL1S(DL1P, LOGL_ERROR, l1t, tn, chan, fn, "TCH mode %u invalid, please fix!\n",
 			tch_mode);
 		return -EINVAL;
 	}
 	memcpy(*bursts_p, *bursts_p + 464, 464);
 
 	/* Send uplink measurement information to L2 */
-	l1if_process_meas_res(l1t->trx, tn, fn, trx_chan_desc[chan].chan_nr|tn,
-		n_errors, n_bits_total, rssi, toa);
+	l1if_process_meas_res(l1t->trx, tn, *first_fn, trx_chan_desc[chan].chan_nr|tn,
+		n_errors, n_bits_total, rssi, toa256);
 
 	/* Check if the frame is bad */
 	if (rc < 0) {
-		LOGP(DL1C, LOGL_NOTICE, "Received bad TCH frame ending at "
-			"fn=%u for %s\n", fn, trx_chan_desc[chan].name);
+		LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn, "Received bad data (%u/%u)\n",
+			fn % l1ts->mf_period, l1ts->mf_period);
+		bfi_flag = true;
 		goto bfi;
 	}
 	if (rc < 4) {
-		LOGP(DL1C, LOGL_NOTICE, "Received bad TCH frame ending at "
-			"fn=%u for %s with codec mode %d (out of range)\n",
-			fn, trx_chan_desc[chan].name, rc);
+		LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn, "Received bad data (%u/%u) "
+			"with invalid codec mode %d\n", fn % l1ts->mf_period, l1ts->mf_period, rc);
+		bfi_flag = true;
 		goto bfi;
 	}
 
@@ -1109,7 +1070,7 @@ int rx_tchf_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	if (rc == GSM_MACBLOCK_LEN) {
 		uint16_t ber10k = compute_ber10k(n_bits_total, n_errors);
 		_sched_compose_ph_data_ind(l1t, tn, (fn + GSM_HYPERFRAME - 7) % GSM_HYPERFRAME, chan,
-			tch_data + amr, GSM_MACBLOCK_LEN, rssi, 4 * toa, 0,
+			tch_data + amr, GSM_MACBLOCK_LEN, rssi, 4 * toa256, 0,
 					   ber10k, PRES_INFO_UNKNOWN);
 bfi:
 		if (rsl_cmode == RSL_CMOD_SPD_SPEECH) {
@@ -1118,8 +1079,14 @@ bfi:
 			case GSM48_CMODE_SPEECH_V1: /* FR */
 				if (lchan->tch.dtx.ul_sid)
 					return 0; /* DTXu: pause in progress */
-				memset(tch_data, 0, GSM_FR_BYTES);
-				tch_data[0] = 0xd0;
+
+				/* Perform error concealment if possible */
+				rc = osmo_ecu_fr_conceal(&lchan->ecu_state.fr, tch_data);
+				if (rc) {
+					memset(tch_data, 0, GSM_FR_BYTES);
+					tch_data[0] = 0xd0;
+				}
+
 				rc = GSM_FR_BYTES;
 				break;
 			case GSM48_CMODE_SPEECH_EFR: /* EFR */
@@ -1137,8 +1104,8 @@ bfi:
 				memset(tch_data + 2, 0, rc - 2);
 				break;
 			default:
-				LOGP(DL1C, LOGL_ERROR, "TCH mode invalid, "
-					"please fix!\n");
+				LOGL1S(DL1P, LOGL_ERROR, l1t, tn, chan, fn,
+					"TCH mode %u invalid, please fix!\n", tch_mode);
 				return -EINVAL;
 			}
 		}
@@ -1147,6 +1114,10 @@ bfi:
 	if (rsl_cmode != RSL_CMOD_SPD_SPEECH)
 		return 0;
 
+	/* Reset ECU with a good frame */
+	if (!bfi_flag && tch_mode == GSM48_CMODE_SPEECH_V1)
+		osmo_ecu_fr_reset(&lchan->ecu_state.fr, tch_data);
+
 	/* TCH or BFI */
 	return _sched_compose_tch_ind(l1t, tn, (fn + GSM_HYPERFRAME - 7) % GSM_HYPERFRAME, chan,
 		tch_data, rc);
@@ -1155,11 +1126,12 @@ bfi:
 /*! \brief a single TCH/H burst was received by the PHY, process it */
 int rx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	enum trx_chan_type chan, uint8_t bid, sbit_t *bits, uint16_t nbits,
-	int8_t rssi, float toa)
+	int8_t rssi, int16_t toa256)
 {
 	struct l1sched_ts *l1ts = l1sched_trx_get_ts(l1t, tn);
 	struct l1sched_chan_state *chan_state = &l1ts->chan_state[chan];
 	sbit_t *burst, **bursts_p = &chan_state->ul_bursts;
+	uint32_t *first_fn = &chan_state->ul_first_fn;
 	uint8_t *mask = &chan_state->ul_mask;
 	uint8_t rsl_cmode = chan_state->rsl_cmode;
 	uint8_t tch_mode = chan_state->tch_mode;
@@ -1176,10 +1148,9 @@ int rx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 
 	/* handle RACH, if handover RACH detection is turned on */
 	if (chan_state->ho_rach_detect == 1)
-		return rx_rach_fn(l1t, tn, fn, chan, bid, bits, GSM_BURST_LEN, rssi, toa);
+		return rx_rach_fn(l1t, tn, fn, chan, bid, bits, GSM_BURST_LEN, rssi, toa256);
 
-	LOGP(DL1C, LOGL_DEBUG, "TCH/H received %s fn=%u ts=%u trx=%u bid=%u\n",
-		trx_chan_desc[chan].name, fn, tn, l1t->trx->nr, bid);
+	LOGL1S(DL1P, LOGL_DEBUG, l1t, tn, chan, fn, "Received TCH/H, bid=%u\n", bid);
 
 	/* allocate burst memory, if not already */
 	if (!*bursts_p) {
@@ -1192,6 +1163,7 @@ int rx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	if (bid == 0) {
 		memset(*bursts_p + 464, 0, 232);
 		*mask = 0x0;
+		*first_fn = fn;
 	}
 
 	/* update mask */
@@ -1208,10 +1180,8 @@ int rx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 
 	/* check for complete set of bursts */
 	if ((*mask & 0x3) != 0x3) {
-		LOGP(DL1C, LOGL_NOTICE, "Received incomplete TCH frame ending "
-			"at fn=%u (%u/%u) for %s\n", fn,
-			fn % l1ts->mf_period, l1ts->mf_period,
-			trx_chan_desc[chan].name);
+		LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn, "Received incomplete frame (%u/%u)\n",
+			fn % l1ts->mf_period, l1ts->mf_period);
 	}
 	*mask = 0x0;
 
@@ -1259,7 +1229,7 @@ int rx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 		}
 		break;
 	default:
-		LOGP(DL1C, LOGL_ERROR, "TCH mode %u invalid, please fix!\n",
+		LOGL1S(DL1P, LOGL_ERROR, l1t, tn, chan, fn, "TCH mode %u invalid, please fix!\n",
 			tch_mode);
 		return -EINVAL;
 	}
@@ -1267,19 +1237,18 @@ int rx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 	memcpy(*bursts_p + 232, *bursts_p + 464, 232);
 
 	/* Send uplink measurement information to L2 */
-	l1if_process_meas_res(l1t->trx, tn, fn, trx_chan_desc[chan].chan_nr|tn,
-		n_errors, n_bits_total, rssi, toa);
+	l1if_process_meas_res(l1t->trx, tn, *first_fn, trx_chan_desc[chan].chan_nr|tn,
+		n_errors, n_bits_total, rssi, toa256);
 
 	/* Check if the frame is bad */
 	if (rc < 0) {
-		LOGP(DL1C, LOGL_NOTICE, "Received bad TCH frame ending at "
-			"fn=%u for %s\n", fn, trx_chan_desc[chan].name);
+		LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn, "Received bad data (%u/%u)\n",
+			fn % l1ts->mf_period, l1ts->mf_period);
 		goto bfi;
 	}
 	if (rc < 4) {
-		LOGP(DL1C, LOGL_NOTICE, "Received bad TCH frame ending at "
-			"fn=%u for %s with codec mode %d (out of range)\n",
-			fn, trx_chan_desc[chan].name, rc);
+		LOGL1S(DL1P, LOGL_NOTICE, l1t, tn, chan, fn, "Received bad data (%u/%u) "
+			"with invalid codec mode %d\n", fn % l1ts->mf_period, l1ts->mf_period, rc);
 		goto bfi;
 	}
 
@@ -1289,7 +1258,7 @@ int rx_tchh_fn(struct l1sched_trx *l1t, uint8_t tn, uint32_t fn,
 		uint16_t ber10k = compute_ber10k(n_bits_total, n_errors);
 		_sched_compose_ph_data_ind(l1t, tn,
 			(fn + GSM_HYPERFRAME - 10 - ((fn % 26) >= 19)) % GSM_HYPERFRAME, chan,
-			tch_data + amr, GSM_MACBLOCK_LEN, rssi, 4 * toa, 0,
+			tch_data + amr, GSM_MACBLOCK_LEN, rssi, toa256/64, 0,
 					   ber10k, PRES_INFO_UNKNOWN);
 bfi:
 		if (rsl_cmode == RSL_CMOD_SPD_SPEECH) {
@@ -1312,8 +1281,8 @@ bfi:
 				memset(tch_data + 2, 0, rc - 2);
 				break;
 			default:
-				LOGP(DL1C, LOGL_ERROR, "TCH mode invalid, "
-					"please fix!\n");
+				LOGL1S(DL1P, LOGL_ERROR, l1t, tn, chan, fn,
+					"TCH mode %u invalid, please fix!\n", tch_mode);
 				return -EINVAL;
 			}
 		}
@@ -1340,7 +1309,7 @@ static int trx_sched_fn(struct gsm_bts *bts, uint32_t fn)
 	uint8_t tn;
 	const ubit_t *bits;
 	uint8_t gain;
-	uint16_t nbits;
+	uint16_t nbits = 0;
 
 	/* send time indication */
 	l1if_mph_time_ind(bts, fn);
@@ -1612,7 +1581,7 @@ int trx_sched_clock(struct gsm_bts *bts, uint32_t fn)
 	struct osmo_trx_clock_state *tcs = &g_clk_s;
 	struct timespec tv_now;
 	int elapsed_us, elapsed_fn;
-	int elapsed_us_since_clk, elapsed_fn_since_clk, error_us_since_clk;
+	int64_t elapsed_us_since_clk, elapsed_fn_since_clk, error_us_since_clk;
 	unsigned int fn_caught_up = 0;
 	const struct timespec interval = { .tv_sec = 0, .tv_nsec = FRAME_DURATION_nS };
 
@@ -1656,7 +1625,8 @@ int trx_sched_clock(struct gsm_bts *bts, uint32_t fn)
 	elapsed_fn_since_clk = compute_elapsed_fn(tcs->last_clk_ind.fn, fn);
 	/* error (delta) between local clock since last CLK and CLK based on FN clock at TRX */
 	error_us_since_clk = elapsed_us_since_clk - (FRAME_DURATION_uS * elapsed_fn_since_clk);
-	LOGP(DL1C, LOGL_INFO, "TRX Clock Ind: elapsed_us=%7d, elapsed_fn=%3d, error_us=%+5d\n",
+	LOGP(DL1C, LOGL_INFO, "TRX Clock Ind: elapsed_us=%7"PRId64", "
+		"elapsed_fn=%3"PRId64", error_us=%+5"PRId64"\n",
 		elapsed_us_since_clk, elapsed_fn_since_clk, error_us_since_clk);
 
 	/* TODO: put this computed error_us_since_clk into some filter
